!
! $Id: modcurgridfunctions.F,v 1.2 2003/12/17 13:56:10 pmarches Exp $
!
C     AGRIF (Adaptive Grid Refinement In Fortran)
C
C     Copyright (C) 2003 Laurent Debreu (Laurent.Debreu@imag.fr)
C                        Christophe Vouland (Christophe.Vouland@imag.fr)    
C
C     This program is free software; you can redistribute it and/or modify
C     it under the terms of the GNU General Public License as published by
C     the Free Software Foundation; either version 2 of the License, or
C     (at your option) any later version.
C
C     This program is distributed in the hope that it will be useful,
C     but WITHOUT ANY WARRANTY; without even the implied warranty of
C     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
C     GNU General Public License for more details.
C
C     You should have received a copy of the GNU General Public License
C     along with this program; if not, write to the Free Software
C     Foundation, Inc., 59 Temple Place -  Suite 330, Boston, MA 02111-1307, USA.
C
C
C
CCC   Module Agrif_CurgridFunctions      
C      
      Module Agrif_CurgridFunctions    
C
CCC   Description:
CCC   Module to define some procedures concerning the current grid
C
C     Modules used: 
C
      Use Agrif_Types
      Use Agrif_Init
      Use Agrif_Initbis
C
C
      interface Agrif_Parent
C      
          module procedure Agrif_Parent_0D_Int,Agrif_Parent_1D_Int,
     &                     Agrif_Parent_2D_Int,Agrif_Parent_3D_Int,
     &                     Agrif_Parent_4D_Int,Agrif_Parent_5D_Int,
     &                     Agrif_Parent_0D_Real,Agrif_Parent_1D_Real,
     &                     Agrif_Parent_2D_Real,Agrif_Parent_3D_Real,
     &                     Agrif_Parent_4D_Real,Agrif_Parent_5D_Real,
     &                     Agrif_Parent_0D_Logical,
     &                     Agrif_Parent_1D_Logical,
     &                     Agrif_Parent_2D_Logical,
     &                     Agrif_Parent_3D_Logical,
     &                     Agrif_Parent_4D_Logical,
     &                     Agrif_Parent_5D_Logical
C 
      end interface     	  
C
C
      Contains
C     Define procedures contained in this module
C
C
C
C     **************************************************************************
CCC   Function Agrif_Parent_0D_Int
C     **************************************************************************
C
      Function Agrif_Parent_0D_Int(namevariable,variable)
C  
CCC   Description:
CCC   Function to get the values on the parent grid for a 0D integer variable.
C
C     Declarations:
C  
      Implicit none
C
C     Arguments
      character(*) :: namevariable
      Integer :: variable
C
C     Local variables       
      Integer :: res
      Integer :: Agrif_Parent_0D_Int
      Integer :: i
C
C
      Do i=1,AGRIF_NbVariables
C
        If (namevariable == AGRIF_Curgrid%tabvars(i)%var%namevar) then
C         
            res = 
     &      AGRIF_Curgrid % tabvars(i) % parent_var % var % iarray0
C	   
            exit
C   
         End If
C             	 	
      End Do
C
      Agrif_Parent_0D_Int = res             	
C
C
      End function Agrif_Parent_0D_Int
C
C
C
C     **************************************************************************
CCC   Function Agrif_Parent_1D_Int
C     **************************************************************************
C
      Function Agrif_Parent_1D_Int(namevariable,variable)
C  
CCC   Description:
CCC   Function to get the values on the parent grid for a 1D integer variable.
C
C     Declarations:
C  
      Implicit none
C
C     Arguments
      character(*) :: namevariable
      Integer, Dimension(:), Pointer :: variable
C
C     Local variables       
      Integer, Dimension(:), Pointer :: res
      Integer, Dimension(:), Pointer :: Agrif_Parent_1D_Int
      Integer :: i
C
C
      Do i=1,AGRIF_NbVariables
C
        If (namevariable == AGRIF_Curgrid%tabvars(i)%var%namevar) then
C         
            res =>
     &      AGRIF_Curgrid % tabvars(i) % parent_var % var % iarray1
C	   
            exit
C   
         End If
C             	 	
      End Do
C
      Agrif_Parent_1D_Int => res             	
C
C
      End function Agrif_Parent_1D_Int
C
C
C
C     **************************************************************************
CCC   Function Agrif_Parent_2D_Int
C     **************************************************************************
C
      Function Agrif_Parent_2D_Int(namevariable,variable)
C  
CCC   Description:
CCC   Function to get the values on the parent grid for a 2D integer variable.
C
C     Declarations:
C  
      Implicit none
C
C     Arguments
      character(*) :: namevariable
      Integer, Dimension(:,:), Pointer :: variable
C
C     Local variables       
      Integer, Dimension(:,:), Pointer :: res
      Integer, Dimension(:,:), Pointer :: Agrif_Parent_2D_Int
      Integer :: i
C
C
      Do i=1,AGRIF_NbVariables
C
        If (namevariable == AGRIF_Curgrid%tabvars(i)%var%namevar) then
C         
            res =>
     &      AGRIF_Curgrid % tabvars(i) % parent_var % var % iarray2
C	   
            exit
C   
         End If
C             	 	
      End Do
C
      Agrif_Parent_2D_Int => res             	
C
C
      End function Agrif_Parent_2D_Int
C
C
C
C     **************************************************************************
CCC   Function Agrif_Parent_3D_Int
C     **************************************************************************
C
      Function Agrif_Parent_3D_Int(namevariable,variable)
C  
CCC   Description:
CCC   Function to get the values on the parent grid for a 3D integer variable.
C
C     Declarations:
C  
      Implicit none
C
C     Arguments
      character(*) :: namevariable
      Integer, Dimension(:,:,:), Pointer :: variable
C
C     Local variables       
      Integer, Dimension(:,:,:), Pointer :: res
      Integer, Dimension(:,:,:), Pointer :: Agrif_Parent_3D_Int
      Integer :: i
C
C
      Do i=1,AGRIF_NbVariables
C
        If (namevariable == AGRIF_Curgrid%tabvars(i)%var%namevar) then
C         
            res =>
     &      AGRIF_Curgrid % tabvars(i) % parent_var % var % iarray3
C	   
            exit
C   
         End If
C             	 	
      End Do
C
      Agrif_Parent_3D_Int => res             	
C
C
      End function Agrif_Parent_3D_Int
C
C
C
C     **************************************************************************
CCC   Function Agrif_Parent_4D_Int
C     **************************************************************************
C
      Function Agrif_Parent_4D_Int(namevariable,variable)
C  
CCC   Description:
CCC   Function to get the values on the parent grid for a 3D integer variable.
C
C     Declarations:
C  
      Implicit none
C
C     Arguments
      character(*) :: namevariable
      Integer, Dimension(:,:,:,:), Pointer :: variable
C
C     Local variables       
      Integer, Dimension(:,:,:,:), Pointer :: res
      Integer, Dimension(:,:,:,:), Pointer :: Agrif_Parent_4D_Int
      Integer :: i
C
C
      Do i=1,AGRIF_NbVariables
C
        If (namevariable == AGRIF_Curgrid%tabvars(i)%var%namevar) then
C         
            res =>
     &      AGRIF_Curgrid % tabvars(i) % parent_var % var % iarray4
C	   
            exit
C   
         End If
C             	 	
      End Do
C
      Agrif_Parent_4D_Int => res             	
C
C
      End function Agrif_Parent_4D_Int
C
C
C
C     **************************************************************************
CCC   Function Agrif_Parent_5D_Int
C     **************************************************************************
C
      Function Agrif_Parent_5D_Int(namevariable,variable)
C  
CCC   Description:
CCC   Function to get the values on the parent grid for a 5D integer variable.
C
C     Declarations:
C  
      Implicit none
C
C     Arguments
      character(*) :: namevariable
      Integer, Dimension(:,:,:,:,:), Pointer :: variable
C
C     Local variables       
      Integer, Dimension(:,:,:,:,:), Pointer :: res
      Integer, Dimension(:,:,:,:,:), Pointer :: Agrif_Parent_5D_Int
      Integer :: i
C
C
      Do i=1,AGRIF_NbVariables
C
        If (namevariable == AGRIF_Curgrid%tabvars(i)%var%namevar) then
C         
            res =>
     &      AGRIF_Curgrid % tabvars(i) % parent_var % var % iarray5
C	   
            exit
C   
         End If
C             	 	
      End Do
C
      Agrif_Parent_5D_Int => res             	
C
C
      End function Agrif_Parent_5D_Int
C
C
C
C     **************************************************************************
CCC   Function Agrif_Parent_0D_Real
C     **************************************************************************
C
      Function Agrif_Parent_0D_Real(namevariable,variable)
C  
CCC   Description:
CCC   Function to get the values on the parent grid for a 0D real variable.
C
C     Declarations:
C  
      Implicit none
C
C     Arguments
      character(*) :: namevariable
      Real :: variable
C
C     Local variables      
      Real :: res
      Real :: Agrif_Parent_0D_Real
      Integer :: i
C
C
      Do i=1,AGRIF_NbVariables
C      
        If (namevariable == AGRIF_Curgrid%tabvars(i)%var%namevar) then
C        
            res = 
     &      AGRIF_Curgrid % tabvars(i) % parent_var % var % array0
C	   
            exit
C  
        End If
C            	 	
      End Do
C
      Agrif_Parent_0D_Real = res             	
C
C
      End function Agrif_Parent_0D_Real      
C
C
C
C     **************************************************************************
CCC   Function Agrif_Parent_1D_Real
C     **************************************************************************
C
      Function Agrif_Parent_1D_Real(namevariable,variable)
C  
CCC   Description:
CCC   Function to get the values on the parent grid for a 1D real variable.
C
C     Declarations:
C  
      Implicit none
C
C     Arguments 
      character(*) :: namevariable
      Real, Dimension(:), Pointer :: variable
C
C     Local variables      
      Real, Dimension(:), Pointer :: res
      Real, Dimension(:), Pointer :: Agrif_Parent_1D_Real
      Integer :: i
C
C
      Do i=1,AGRIF_NbVariables
C
        If (namevariable == AGRIF_Curgrid%tabvars(i)%var%namevar) then
C         
            res => 
     &      AGRIF_Curgrid % tabvars(i) % parent_var % var % array1
C	
           exit
C      
        End If
C            	 	
      End Do
C
      Agrif_Parent_1D_Real => res             	
C
C
      End function Agrif_Parent_1D_Real
C
C
C
C     **************************************************************************
CCC   Function Agrif_Parent_2D_Real
C     **************************************************************************
C
      Function Agrif_Parent_2D_Real(namevariable,variable)
C  
CCC   Description:
CCC   Function to get the values on the parent grid for a 2D real variable.
C
C     Declarations:
C  
      Implicit none
C
C     Arguments  
      character(*) :: namevariable
      Real, Dimension(:,:), Pointer :: variable
C
C     Local variables      
      Real, Dimension(:,:), Pointer :: res
      Real, Dimension(:,:), Pointer :: Agrif_Parent_2D_Real
      Integer :: i
C 
C
      Do i=1,AGRIF_NbVariables
C
        If (namevariable == AGRIF_Curgrid%tabvars(i)%var%namevar) then
C        
            res => 
     &      AGRIF_Curgrid % tabvars(i) % parent_var % var % array2
C	   
           exit
C  
         End If    	 	
C 
      End Do
C
      Agrif_Parent_2D_Real => res             	
C
C
      End function Agrif_Parent_2D_Real
C
C
C
C     **************************************************************************
CCC   Function Agrif_Parent_3D_Real
C     **************************************************************************
C
      Function Agrif_Parent_3D_Real(namevariable,variable)
C  
CCC   Description:
CCC   Function to get the values on the parent grid for a 3D real variable.
C
C     Declarations:
C  
      Implicit none
C
C     Arguments
      character(*) :: namevariable
      Real, Dimension(:,:,:), Pointer :: variable
C
C     Local variables      
      Real, Dimension(:,:,:), Pointer :: res
      Real, Dimension(:,:,:), Pointer :: Agrif_Parent_3D_Real
      Integer :: i
C
C
      Do i=1,AGRIF_NbVariables
C      
        If (namevariable == AGRIF_Curgrid%tabvars(i)%var%namevar) then
C        
           res => 
     &     AGRIF_Curgrid % tabvars(i) % parent_var % var % array3
C	   
           exit
C        
        End If
C            	 	
      End Do
C
      Agrif_Parent_3D_Real => res             	
C
C
      End function Agrif_Parent_3D_Real
C
C
C
C     **************************************************************************
CCC   Function Agrif_Parent_4D_Real
C     **************************************************************************
C
      Function Agrif_Parent_4D_Real(namevariable,variable)
C  
CCC   Description:
CCC   Function to get the values on the parent grid for a 4D real variable.
C
C     Declarations:
C  
      Implicit none
C
C     Arguments
      character(*) :: namevariable
      Real, Dimension(:,:,:,:), Pointer :: variable
C
C     Local variables      
      Real, Dimension(:,:,:,:), Pointer :: res
      Real, Dimension(:,:,:,:), Pointer :: Agrif_Parent_4D_Real
      Integer :: i
C
C
      Do i=1,AGRIF_NbVariables
C      
        If (namevariable == AGRIF_Curgrid%tabvars(i)%var%namevar) then
C        
           res => 
     &     AGRIF_Curgrid % tabvars(i) % parent_var % var % array4
C	   
           exit
C        
        End If
C            	 	
      End Do
C
      Agrif_Parent_4D_Real => res             	
C
C
      End function Agrif_Parent_4D_Real 
C
C
C
C     **************************************************************************
CCC   Function Agrif_Parent_5D_Real
C     **************************************************************************
C
      Function Agrif_Parent_5D_Real(namevariable,variable)
C  
CCC   Description:
CCC   Function to get the values on the parent grid for a 5D real variable.
C
C     Declarations:
C  
      Implicit none
C
C     Arguments
      character(*) :: namevariable
      Real, Dimension(:,:,:,:,:), Pointer :: variable
C
C     Local variables      
      Real, Dimension(:,:,:,:,:), Pointer :: res
      Real, Dimension(:,:,:,:,:), Pointer :: Agrif_Parent_5D_Real
      Integer :: i
C
C
      Do i=1,AGRIF_NbVariables
C      
        If (namevariable == AGRIF_Curgrid%tabvars(i)%var%namevar) then
C        
           res => 
     &     AGRIF_Curgrid % tabvars(i) % parent_var % var % array5
C	   
           exit
C        
        End If
C            	 	
      End Do
C
      Agrif_Parent_5D_Real => res             	
C
C
      End function Agrif_Parent_5D_Real 
C
C
C
C     **************************************************************************
CCC   Function Agrif_Parent_0D_Logical
C     **************************************************************************
C
      Function Agrif_Parent_0D_Logical(namevariable,variable)
C  
CCC   Description:
CCC   Function to get the values on the parent grid for a 0D logical variable.
C
C     Declarations:
C  
      Implicit none
C
C     Arguments
      character(*) :: namevariable
      Logical :: variable
C
C     Local variables      
      Logical :: res
      Logical :: Agrif_Parent_0D_Logical
      Integer :: i
C
C
      Do i=1,AGRIF_NbVariables
C      
        If (namevariable == AGRIF_Curgrid%tabvars(i)%var%namevar) then
C        
            res = 
     &      AGRIF_Curgrid % tabvars(i) % parent_var % var % larray0
C	   
            exit
C  
        End If
C            	 	
      End Do
C
      Agrif_Parent_0D_Logical = res             	
C
C
      End function Agrif_Parent_0D_Logical      
C
C
C
C     **************************************************************************
CCC   Function Agrif_Parent_1D_Logical
C     **************************************************************************
C
      Function Agrif_Parent_1D_Logical(namevariable,variable)
C  
CCC   Description:
CCC   Function to get the values on the parent grid for a 1D logical variable.
C
C     Declarations:
C  
      Implicit none
C
C     Arguments 
      character(*) :: namevariable
      Logical, Dimension(:), Pointer :: variable
C
C     Local variables      
      Logical, Dimension(:), Pointer :: res
      Logical, Dimension(:), Pointer :: Agrif_Parent_1D_Logical
      Integer :: i
C
C
      Do i=1,AGRIF_NbVariables
C
        If (namevariable == AGRIF_Curgrid%tabvars(i)%var%namevar) then
C         
            res => 
     &      AGRIF_Curgrid % tabvars(i) % parent_var % var % larray1
C	
           exit
C      
        End If
C            	 	
      End Do
C
      Agrif_Parent_1D_Logical => res             	
C
C
      End function Agrif_Parent_1D_Logical
C
C
C
C     **************************************************************************
CCC   Function Agrif_Parent_2D_Logical
C     **************************************************************************
C
      Function Agrif_Parent_2D_Logical(namevariable,variable)
C  
CCC   Description:
CCC   Function to get the values on the parent grid for a 2D logical variable.
C
C     Declarations:
C  
      Implicit none
C
C     Arguments  
      character(*) :: namevariable
      Logical, Dimension(:,:), Pointer :: variable
C
C     Local variables      
      Logical, Dimension(:,:), Pointer :: res
      Logical, Dimension(:,:), Pointer :: Agrif_Parent_2D_Logical
      Integer :: i
C 
C
      Do i=1,AGRIF_NbVariables
C
        If (namevariable == AGRIF_Curgrid%tabvars(i)%var%namevar) then
C        
            res => 
     &      AGRIF_Curgrid % tabvars(i) % parent_var % var % larray2
C	   
           exit
C  
         End If    	 	
C 
      End Do
C
      Agrif_Parent_2D_Logical => res             	
C
C
      End function Agrif_Parent_2D_Logical
C
C
C
C     **************************************************************************
CCC   Function Agrif_Parent_3D_Logical
C     **************************************************************************
C
      Function Agrif_Parent_3D_Logical(namevariable,variable)
C  
CCC   Description:
CCC   Function to get the values on the parent grid for a 3D logical variable.
C
C     Declarations:
C  
      Implicit none
C
C     Arguments
      character(*) :: namevariable
      Logical, Dimension(:,:,:), Pointer :: variable
C
C     Local variables      
      Logical, Dimension(:,:,:), Pointer :: res
      Logical, Dimension(:,:,:), Pointer :: Agrif_Parent_3D_Logical
      Integer :: i
C
C
      Do i=1,AGRIF_NbVariables
C      
        If (namevariable == AGRIF_Curgrid%tabvars(i)%var%namevar) then
C        
           res => 
     &     AGRIF_Curgrid % tabvars(i) % parent_var % var % larray3
C	   
           exit
C        
        End If
C            	 	
      End Do
C
      Agrif_Parent_3D_Logical => res             	
C
C
      End function Agrif_Parent_3D_Logical
C
C
C
C     **************************************************************************
CCC   Function Agrif_Parent_4D_Logical
C     **************************************************************************
C
      Function Agrif_Parent_4D_Logical(namevariable,variable)
C  
CCC   Description:
CCC   Function to get the values on the parent grid for a 4D logical variable.
C
C     Declarations:
C  
      Implicit none
C
C     Arguments
      character(*) :: namevariable
      Logical, Dimension(:,:,:,:), Pointer :: variable
C
C     Local variables      
      Logical, Dimension(:,:,:,:), Pointer :: res
      Logical, Dimension(:,:,:,:), Pointer :: Agrif_Parent_4D_Logical
      Integer :: i
C
C
      Do i=1,AGRIF_NbVariables
C      
        If (namevariable == AGRIF_Curgrid%tabvars(i)%var%namevar) then
C        
           res => 
     &     AGRIF_Curgrid % tabvars(i) % parent_var % var % larray4
C	   
           exit
C        
        End If
C            	 	
      End Do
C
      Agrif_Parent_4D_Logical => res             	
C
C
      End function Agrif_Parent_4D_Logical
C
C
C
C     **************************************************************************
CCC   Function Agrif_Parent_5D_Logical
C     **************************************************************************
C
      Function Agrif_Parent_5D_Logical(namevariable,variable)
C  
CCC   Description:
CCC   Function to get the values on the parent grid for a 5D logical variable.
C
C     Declarations:
C  
      Implicit none
C
C     Arguments
      character(*) :: namevariable
      Logical, Dimension(:,:,:,:,:), Pointer :: variable
C
C     Local variables      
      Logical, Dimension(:,:,:,:,:), Pointer :: res
      Logical, Dimension(:,:,:,:,:), Pointer :: Agrif_Parent_5D_Logical
      Integer :: i
C
C
      Do i=1,AGRIF_NbVariables
C      
        If (namevariable == AGRIF_Curgrid%tabvars(i)%var%namevar) then
C        
           res => 
     &     AGRIF_Curgrid % tabvars(i) % parent_var % var % larray5
C	   
           exit
C        
        End If
C            	 	
      End Do
C
      Agrif_Parent_5D_Logical => res             	
C
C
      End function Agrif_Parent_5D_Logical          
C
C
C
C     **************************************************************************
CCC   Function Agrif_Rhot
C     **************************************************************************
C
      Function Agrif_Rhot()
C  
CCC   Description:
CCC   Function returning the time refinement factor of the current grid.
C
C     Declarations:
C  
      Implicit none
C
      Real :: Agrif_Rhot  ! Result
C
C     Local scalar      
      Integer :: res         ! Intermediate result	
C
C
      res=1
C	
      res = max(res, AGRIF_Curgrid % timerefx)
C
      If (Agrif_Probdim >= 2) Then
C
          res = max(res, AGRIF_Curgrid % timerefy)
C
      End If
C
      If (Agrif_Probdim == 3) Then
C            
          res = max(res, AGRIF_Curgrid % timerefz)
C
      End If 
C	
      Agrif_Rhot = float(res)
C
C	
      End function Agrif_rhot
C
C
C
C     **************************************************************************
CCC   Function Agrif_Parent_Rhot
C     **************************************************************************
C
      Function Agrif_Parent_Rhot()
C  
CCC   Description:
CCC   Function returning the time refinement factor of the parent grid of the 
CCC   current grid.
C
C     Declarations:
C  
      Implicit none
C
      Real :: Agrif_Parent_Rhot  ! Result
C
C     Local scalar      
      Integer :: res         ! Intermediate result	
C
C
      res=1
C
      res = max(res, AGRIF_Curgrid % parent % timerefx)
C
      If (Agrif_Probdim >= 2) Then
C
          res = max(res, AGRIF_Curgrid % parent % timerefy)
C
      End If  	
C
      If (Agrif_Probdim == 3) Then
C
          res = max(res, AGRIF_Curgrid % parent % timerefz)
C
      End If 	
C
      Agrif_Parent_Rhot = float(res)
C
C	
      End function Agrif_Parent_Rhot      
C
C
C
C     **************************************************************************
CCC   Function Agrif_Rhox
C     **************************************************************************
C
      Function Agrif_Rhox()
C  
CCC   Description:
CCC   Function returning the space refinement factor in the x direction of the
CCC   current grid.
C
C     Declarations:
C  
      Implicit none
C
      Real :: Agrif_Rhox  ! Result
C
C	
      Agrif_Rhox = real(Agrif_Curgrid % spacerefx)
C
C	
      End function Agrif_Rhox
C
C
C
C     **************************************************************************
CCC   Function Agrif_Parent_Rhox
C     **************************************************************************
C
      Function Agrif_Parent_Rhox()
C  
CCC   Description:
CCC   Function returning the space refinement factor in the x direction of the
CCC   parent grid of the current grid.
C
C     Declarations:
C  
      Implicit none
C
      Real :: Agrif_Parent_Rhox  ! Result
C
C	
      Agrif_Parent_Rhox = real(Agrif_Curgrid % parent % spacerefx)
C
C	
      End function Agrif_Parent_Rhox      
C
C
C
C     **************************************************************************
CCC   Function Agrif_Irhox
C     **************************************************************************
C
      Function Agrif_Irhox()
C  
CCC   Description:
CCC   Function returning the space refinement factor in the x direction of the
CCC   current grid.
C
C     Declarations:
C  
      Implicit none
C
      Integer :: Agrif_Irhox  ! Result
C
C	
      Agrif_Irhox = Agrif_Curgrid % spacerefx
C
C	
      End function Agrif_Irhox      	
C
C
C
C     **************************************************************************
CCC   Function Agrif_Parent_Irhox
C     **************************************************************************
C
      Function Agrif_Parent_Irhox()
C  
CCC   Description:
CCC   Function returning the space refinement factor in the x direction of the
CCC   parent grid of the current grid.
C
C     Declarations:
C  
      Implicit none
C
      Integer :: Agrif_Parent_Irhox  ! Result
C
C	
      Agrif_Parent_Irhox = Agrif_Curgrid % parent % spacerefx
C
C	
      End function Agrif_Parent_Irhox
C
C
C
C     **************************************************************************
CCC   Function Agrif_Rhoy
C     **************************************************************************
C
      Function Agrif_Rhoy()
C  
CCC   Description:
CCC   Function returning the space refinement factor in the y direction of the
CCC   current grid.
C
C     Declarations:
C  
      Implicit none
C
      Real :: Agrif_Rhoy  ! Result
C
C	
      Agrif_Rhoy = real(Agrif_Curgrid % spacerefy)
C
C	
      End function Agrif_Rhoy	
C
C
C
C     **************************************************************************
CCC   Function Agrif_Parent_Rhoy
C     **************************************************************************
C
      Function Agrif_Parent_Rhoy()
C  
CCC   Description:
CCC   Function returning the space refinement factor in the y direction of the
CCC   parent grid of the current grid.
C
C     Declarations:
C  
      Implicit none
C
      Real :: Agrif_Parent_Rhoy  ! Result
C
C	
      Agrif_Parent_Rhoy = real(Agrif_Curgrid % parent % spacerefy)
C
C	
      End function Agrif_Parent_Rhoy
C
C
C
C     **************************************************************************
CCC   Function Agrif_Irhoy
C     **************************************************************************
C
      Function Agrif_Irhoy()
C  
CCC   Description:
CCC   Function returning the space refinement factor in the y direction of the
CCC   current grid.
C
C     Declarations:
C  
      Implicit none
C
      Integer :: Agrif_Irhoy  ! Result
C
C	
      Agrif_Irhoy = Agrif_Curgrid % spacerefy
C
C	
      End function Agrif_Irhoy
C
C
C
C     **************************************************************************
CCC   Function Agrif_Parent_Irhoy
C     **************************************************************************
C
      Function Agrif_Parent_Irhoy()
C  
CCC   Description:
CCC   Function returning the space refinement factor in the y direction of the
CCC   parent grid of the current grid.
C
C     Declarations:
C  
      Implicit none
C
      Integer :: Agrif_Parent_Irhoy  ! Result
C
C	
      Agrif_Parent_Irhoy = Agrif_Curgrid % parent % spacerefy
C
C	
      End function Agrif_Parent_Irhoy
C
C
C
C     **************************************************************************
CCC   Function Agrif_Rhoz
C     **************************************************************************
C
      Function Agrif_Rhoz()
C  
CCC   Description:
CCC   Function returning the space refinement factor in the z direction of the
CCC   current grid.
C
C     Declarations:
C  
      Implicit none
C
      Real :: Agrif_Rhoz  ! Result
C
C	
      Agrif_Rhoz = real(Agrif_Curgrid % spacerefz)
C
C	
      End function Agrif_Rhoz
C
C
C
C     **************************************************************************
CCC   Function Agrif_Parent_Rhoz
C     **************************************************************************
C
      Function Agrif_Parent_Rhoz()
C  
CCC   Description:
CCC   Function returning the space refinement factor in the z direction of the
CCC   parent grid of the current grid.
C
C     Declarations:
C  
      Implicit none
C
      Real :: Agrif_Parent_Rhoz  ! Result
C
C	
      Agrif_Parent_Rhoz = real(Agrif_Curgrid % parent % spacerefz)
C
C	
      End function Agrif_Parent_Rhoz
C
C
C
C     **************************************************************************
CCC   Function Agrif_Irhoz
C     **************************************************************************
C
      Function Agrif_Irhoz()
C  
CCC   Description:
CCC   Function returning the space refinement factor in the z direction of the
CCC   current grid.
C
C     Declarations:
C  
      Implicit none
C
      Integer :: Agrif_Irhoz  ! Result
C
C	
      Agrif_Irhoz = Agrif_Curgrid % spacerefz
C
C	
      End function Agrif_Irhoz
C      
C
C
C     **************************************************************************
CCC   Function Agrif_Parent_Irhoz
C     **************************************************************************
C
      Function Agrif_Parent_Irhoz()
C  
CCC   Description:
CCC   Function returning the space refinement factor in the z direction of the
CCC   parent grid of the current grid.
C
C     Declarations:
C  
      Implicit none
C
      Integer :: Agrif_Parent_Irhoz  ! Result
C
C	
      Agrif_Parent_Irhoz = Agrif_Curgrid % parent % spacerefz
C
C	
      End function Agrif_Parent_Irhoz
C
C
C
C     **************************************************************************
CCC   Function Agrif_Nbstepint
C     **************************************************************************
C
      Function Agrif_Nbstepint()
C  
CCC   Description:
CCC   Function for the calculation of the coefficients used for the time 
CCC   interpolation (module Agrif_Boundary).
C
C     Declarations:
C  
      Implicit none
C            
      Integer :: Agrif_nbstepint  ! result
C
C	
      Agrif_nbstepint = mod(AGRIF_CURGRID % ngridstep,
     &                      int(AGRIF_rhot()))
C
C
      End function Agrif_Nbstepint
C
C
C
CC    **************************************************************************
CCC   Function Agrif_Parent_Nbstepint
C     **************************************************************************
C
      Function Agrif_Parent_Nbstepint()
C  
CCC   Description:
CCC   Function for the calculation of the coefficients used for the time 
CCC   interpolation (module Agrif_Boundary).
C
C     Declarations:
C  
      Implicit none
C            
      Integer :: Agrif_Parent_Nbstepint  ! result
C
C	
      Agrif_Parent_Nbstepint = mod(AGRIF_CURGRID % parent % ngridstep, 
     &                             int(AGRIF_Parent_Rhot()))
C
C
      End function Agrif_Parent_Nbstepint
C
C
C
C     **************************************************************************
CCC   Function Agrif_NearCommonBorderX
C     **************************************************************************
C
      Function Agrif_NearCommonBorderX()
C  
CCC   Description:
CCC   Function indicating if the current grid has its near border common with 
CCC   those of the root grid (in the x direction).
C
C     Declarations:
C  
      Implicit none
C            
      Logical :: Agrif_NearCommonBorderX  ! result
C
C
      Agrif_NearCommonBorderX = AGRIF_CURGRID % NearRootBorderX       	  		            
C
C
      End function Agrif_NearCommonBorderX
C
C
C
C     **************************************************************************
CCC   Function Agrif_NearCommonBorderY
C     **************************************************************************
C
      Function Agrif_NearCommonBorderY()
C  
CCC   Description:
CCC   Function indicating if the current grid has its near border common with 
CCC   those of the root grid (in the y direction).
C
C     Declarations:
C  
      Implicit none
C            
      Logical :: Agrif_NearCommonBorderY  ! result
C
C
      Agrif_NearCommonBorderY = AGRIF_CURGRID % NearRootBorderY       	  		            
C
C
      End function Agrif_NearCommonBorderY
C
C
C
C     **************************************************************************
CCC   Function Agrif_NearCommonBorderZ
C     **************************************************************************
C
      Function Agrif_NearCommonBorderZ()
C  
CCC   Description:
CCC   Function indicating if the current grid has its near border common with 
CCC   those of the root grid (in the z direction).
C
C     Declarations:
C  
      Implicit none
C            
      Logical :: Agrif_NearCommonBorderZ  ! result
C
C
      Agrif_NearCommonBorderZ = AGRIF_CURGRID % NearRootBorderZ       	  		            
C
C
      End function Agrif_NearCommonBorderZ
C
C
C
C     **************************************************************************
CCC   Function Agrif_DistantCommonBorderX
C     **************************************************************************
C
      Function Agrif_DistantCommonBorderX()
C  
CCC   Description:
CCC   Function indicating if the current grid has its distant border common with 
CCC   those of the root grid (in the x direction).
C
C     Declarations:
C  
      Implicit none
C            
      Logical :: Agrif_DistantCommonBorderX  ! result
C
C	
      Agrif_DistantCommonBorderX = AGRIF_CURGRID % DistantRootBorderX 	  		            
C
C
      End function Agrif_DistantCommonBorderX
C
C
C
C     **************************************************************************
CCC   Function Agrif_DistantCommonBorderY
C     **************************************************************************
C
      Function Agrif_DistantCommonBorderY()
C  
CCC   Description:
CCC   Function indicating if the current grid has its distant border common with 
CCC   those of the root grid (in the y direction).
C
C     Declarations:
C  
      Implicit none
C            
      Logical :: Agrif_DistantCommonBorderY  ! result
C
C	
      Agrif_DistantCommonBorderY = AGRIF_CURGRID % DistantRootBorderY 	  		            
C
C
      End function Agrif_DistantCommonBorderY      
C
C
C
C     **************************************************************************
CCC   Function Agrif_DistantCommonBorderZ
C     **************************************************************************
C
      Function Agrif_DistantCommonBorderZ()
C  
CCC   Description:
CCC   Function indicating if the current grid has its distant border common with 
CCC   those of the root grid (in the z direction).
C
C     Declarations:
C  
      Implicit none
C            
      Logical :: Agrif_DistantCommonBorderZ  ! result
C
C	
      Agrif_DistantCommonBorderZ = AGRIF_CURGRID % DistantRootBorderZ 	  		            
C
C
      End function Agrif_DistantCommonBorderZ
C
C
C
C     **************************************************************************
CCC   Subroutine Agrif_InterpNearBorderX
C     **************************************************************************
C
      Subroutine Agrif_InterpNearBorderX()
C  
CCC   Description:
CCC   Subroutine allowing to interpole (in the x direction) on a near border of 
CCC   the current grid if this one has a common border with the root coarse 
CCC   grid.   
C
C     Declarations:
C  
      Implicit none
C
C	
      AGRIF_CURGRID % NearRootBorderX = .FALSE. 	            
C
C
      End Subroutine Agrif_InterpNearBorderX 
C
C
C
C     **************************************************************************
CCC   Subroutine Agrif_InterpDistantBorderX
C     **************************************************************************
C
      Subroutine Agrif_InterpDistantBorderX()
C  
CCC   Description:
CCC   Subroutine allowing to interpole (in the x direction) on a distant border 
CCC   of the current grid if this one has a common border with the root coarse  
CCC   grid.
C
C     Declarations:
C  
      Implicit none
C
C	
      AGRIF_CURGRID % DistantRootBorderX = .FALSE. 	            
C
C
      End Subroutine Agrif_InterpDistantBorderX                 
C
C
C
C     **************************************************************************
CCC   Subroutine Agrif_InterpNearBorderY
C     **************************************************************************
C
      Subroutine Agrif_InterpNearBorderY()
C  
CCC   Description:
CCC   Subroutine allowing to interpole (in the y direction) on a near border of 
CCC   the current grid if this one has a common border with the root coarse 
CCC   grid.
C
C     Declarations:
C  
      Implicit none
C
C	
      AGRIF_CURGRID % NearRootBorderY = .FALSE. 	            
C
C
      End Subroutine Agrif_InterpNearBorderY
C
C
C
C     **************************************************************************
CCC   Subroutine Agrif_InterpDistantBorderY
C     **************************************************************************
C
      Subroutine Agrif_InterpDistantBorderY()
C  
CCC   Description:
CCC   Subroutine allowing to interpole (in the y direction) on a distant border 
CCC   of the current grid if this one has a common border with the root coarse  
CCC   grid.
C
C     Declarations:
C  
      Implicit none
C
C	
      AGRIF_CURGRID % DistantRootBorderY = .FALSE. 	            
C
C
      End Subroutine Agrif_InterpDistantBorderY      
C
C
C
C     **************************************************************************
CCC   Subroutine Agrif_InterpNearBorderZ
C     **************************************************************************
C
      Subroutine Agrif_InterpNearBorderZ()
C  
CCC   Description:
CCC   Subroutine allowing to interpole (in the z direction) on a near border of 
CCC   the current grid if this one has a common border with the root coarse 
CCC   grid.
C
C     Declarations:
C  
      Implicit none
C
C	
      AGRIF_CURGRID % NearRootBorderZ = .FALSE. 	            
C
C
      End Subroutine Agrif_InterpNearBorderZ
C
C
C
C     **************************************************************************
CCC   Subroutine Agrif_InterpDistantBorderZ
C     **************************************************************************
C
      Subroutine Agrif_InterpDistantBorderZ()
C  
CCC   Description:
CCC   Subroutine allowing to interpole (in the z direction) on a distant border 
CCC   of the current grid if this one has a common border with the root coarse  
CCC   grid.
C
C     Declarations:
C  
      Implicit none
C
C	
      AGRIF_CURGRID % DistantRootBorderZ = .FALSE. 	            
C
C
      End Subroutine Agrif_InterpDistantBorderZ
C
C
C
C     **************************************************************************
CCC   Function Agrif_Nb_Step
C     **************************************************************************      
C 
      Function Agrif_Nb_Step()
C  
CCC   Description:
CCC   Function returning the number of time steps of the current grid.
C
C     Declarations:
C  
      Implicit none
C       
      Integer :: AGRIF_Nb_Step ! Result 
C 
C 	
      Agrif_Nb_Step = AGRIF_CURGRID % ngridstep
C
C     
      End function AGRIF_Nb_Step 
C 
C
C 
C     **************************************************************************
CCC   Function Agrif_Parent_Nb_Step
C     ************************************************************************** 
C 
      Function AGRIF_Parent_Nb_Step() 
C  
CCC   Description:
CCC   Function returning the number of time steps of the parent grid of the
CCC   current grid.
C
C     Declarations:
C    
      Implicit none
C
      Integer :: AGRIF_Parent_Nb_Step ! Result 
C
C
      If (Agrif_Root()) then  
C     
          Agrif_Parent_Nb_Step = -1 
C       
        Else
C       
          Agrif_Parent_Nb_Step = Agrif_Curgrid % parent % ngridstep
C     
      End if
C
C     
      End function Agrif_Parent_Nb_Step    
C 
C
C 
C     **************************************************************************
CCC   Function Agrif_Root
C     **************************************************************************
C
      Function Agrif_Root()
C     	
CCC   Description:
CCC   Function indicating if the current grid is or not the root grid.	
C
C     Declarations:
C	
      Implicit none
C
      Logical :: Agrif_Root   ! Result
C    
C
      If (AGRIF_CURGRID % fixedrank .EQ. 0) then
C     	     
          Agrif_Root = .TRUE.
C    
        Else
C    
          Agrif_Root = .FALSE.
C     	
      End If
C
C    
      End function Agrif_Root	
C
C
C
C     **************************************************************************
CCC   Function Agrif_Parent_Root
C     **************************************************************************
C
      Function Agrif_Parent_Root()
C     	
CCC   Description:
CCC   Function indicating if the parent grid of the current grid is or not the 
CCC   root grid.	
C
C     Declarations:
C	
      Implicit none
C
      Logical :: Agrif_Parent_Root   ! Result
C    
C
      If (AGRIF_CURGRID % parent % fixedrank .EQ. 0) then
C     	     
          Agrif_Parent_Root = .TRUE.
C    
        Else
C    
          Agrif_Parent_Root = .FALSE.
C     	
      End If
C
C    
      End function Agrif_Parent_Root		     
C 
C
C 
C     **************************************************************************
CCC   Function Agrif_Fixed
C     **************************************************************************
C 
      Function Agrif_Fixed()
C  
CCC   Description:
CCC   Function returning the number of the current grid.
C
C     Declarations:
C   
      Implicit none
C
      Integer Agrif_Fixed   ! Result 
C	 
C
      If (Agrif_Curgrid % fixed) then
C     
          Agrif_Fixed = Agrif_Curgrid % fixedrank 
C
        Else
C     
         Agrif_Fixed = -1
C        
      End If
C
C     
      End function Agrif_Fixed 
C
C
C
C     **************************************************************************
CCC   Function Agrif_Parent_Fixed
C     **************************************************************************
 
      Function Agrif_Parent_Fixed()
C  
CCC   Description:
CCC   Function returning the number of the parent grid of the current grid.
C
C     Declarations:
C   
      Implicit none
C
      Integer Agrif_Parent_Fixed   ! Result 
C	 
C
      If (Agrif_Curgrid % parent % fixed) then
C     
          Agrif_Parent_Fixed = AGRIF_CURGRID % parent % fixedrank 
C
        Else
C     
         Agrif_Parent_Fixed = 0
C        
      End If
C
C     
      End function Agrif_Parent_Fixed
C
C
C
C     **************************************************************************
CCC   Function Agrif_Is_Fixed
C     **************************************************************************
 
      Function Agrif_Is_Fixed()
C  
CCC   Description:
CCC   Function returning true if the current grid is fixed.
C
C     Declarations:
C   
      Implicit none
C
      Logical Agrif_Is_Fixed   ! Result 
C	 
C
      If (Agrif_Curgrid % fixed) then
C     
          Agrif_Is_Fixed = .true.
C
        Else
C     
          Agrif_Is_Fixed = .false.
C        
      End If
C
C     
      End function Agrif_Is_Fixed
C
C
C
C     **************************************************************************
CCC   Function Agrif_Parent_Is_Fixed
C     **************************************************************************
 
      Function Agrif_Parent_Is_Fixed()
C  
CCC   Description:
CCC   Function returning true if the parent grid of the current grid is fixed.
C
C     Declarations:
C   
      Implicit none
C
      Logical Agrif_Parent_Is_Fixed   ! Result 
C	 
C
      If (Agrif_Curgrid % parent % fixed) then
C     
          Agrif_Parent_Is_Fixed = .true.
C
        Else
C     
          Agrif_Parent_Is_Fixed = .false.
C        
      End If
C
C     
      End function Agrif_Parent_Is_Fixed            
C
C	 
C
C     **************************************************************************
CCC   Function AGRIF_CFixed
C     **************************************************************************
 
      Function AGRIF_CFixed()
C  
CCC   Description:
CCC   Function returning the number of the current grid.
C
C     Declarations:
C   
      Implicit none
C
      Character*3 AGRIF_CFixed   ! Result 
C
C     Local variables      
      Character*3 cfixed
      integer fixed
C	 
C
      fixed = Agrif_Fixed()
C      
      If (fixed .LE. 9) Then
C	
          write(cfixed,'(i1)')fixed
C 
        Else
C   
          write(cfixed,'(i2)')fixed
C      
      End If
C	
      AGrif_Cfixed=cfixed
C
C     
      End function AGRIF_CFixed
C
C
C
C     **************************************************************************
CCC   Function AGRIF_Parent_CFixed
C     **************************************************************************
 
      Function AGRIF_Parent_CFixed()
C  
CCC   Description:
CCC   Function returning the number of the parent grid of the current grid.
C
C     Declarations:
C   
      Implicit none
C
      Character*3 AGRIF_Parent_CFixed   ! Result 
C
C     Local variables      
      Character*3 cfixed
      integer fixed
C	 
C
      fixed = Agrif_Parent_Fixed()
C      
      If (fixed .LE. 9) Then
C	
          write(cfixed,'(i1)')fixed
C      
        Else
C        
          write(cfixed,'(i2)')fixed
C      
      End If
C	
      AGrif_Parent_Cfixed=cfixed
C
C     
      End function AGRIF_Parent_CFixed      
C
C
C 
C     **************************************************************************
CCC   Function Agrif_Nb_Fine_Grids
C     **************************************************************************
C   
      Function Agrif_Nb_Fine_Grids() 
C  
CCC   Description:
CCC   Function returning the number of fixed grids of the grid hierarchy
CCC   (Agrif_nbfixedgrids is declared in the Agrif_Types module).
C
C     Declarations:
C  
      Implicit none
C
      Integer :: Agrif_Nb_Fine_Grids   ! Result 
C	 
C
      Agrif_Nb_Fine_Grids = Agrif_nbfixedgrids 
C
C	 
      End function Agrif_Nb_Fine_Grids
C  	 
C
C 
C     **************************************************************************
CCC   Function Agrif_Ix
C     **************************************************************************
C 
      Function Agrif_Ix() 
C  
CCC   Description:
CCC   Function returning the minimal posiiton of the current grid in the x 
CCC   direction.
CCC    
C
C     Declarations:
C  
      Implicit none
C
      Integer :: Agrif_Ix   ! Result
C 
C	 
      Agrif_Ix = Agrif_CURGRID % ix 
C
C	 
      End function Agrif_Ix 
C
C
C
C     **************************************************************************
CCC   Function Agrif_Parent_Ix
C     **************************************************************************
C 
      Function Agrif_Parent_Ix() 
C  
CCC   Description:
CCC   Function returning the minimal posiiton of the parent grid of the current 
CCC   grid in the x direction.
CCC    
C
C     Declarations:
C  
      Implicit none
C
      Integer :: Agrif_Parent_Ix   ! Result
C 
C	 
      Agrif_Parent_Ix = Agrif_CURGRID % parent % ix 
C
C	 
      End function Agrif_Parent_Ix       
C	
C
C	 
C     **************************************************************************
CCC   Function Agrif_Iy
C     **************************************************************************
C 
      Function Agrif_Iy() 
C  
CCC   Description:
CCC   Function returning the minimal posiiton of the current grid in the y 
CCC   direction.
C
C     Declarations:
C  
      Implicit none
C
      Integer :: Agrif_Iy   ! Result 
C
C	 
      Agrif_Iy = AGRIF_CURGRID % iy 
C
C	 
      End function Agrif_Iy 
C
C
C
C     **************************************************************************
CCC   Function Agrif_Parent_Iy
C     **************************************************************************
C 
      Function Agrif_Parent_Iy() 
C  
CCC   Description:
CCC   Function returning the minimal posiiton of the parent grid of the current 
CCC   grid in the y direction.
CCC    
C
C     Declarations:
C  
      Implicit none
C
      Integer :: Agrif_Parent_Iy   ! Result
C 
C	 
      Agrif_Parent_Iy = Agrif_CURGRID % parent % iy 
C
C	 
      End function Agrif_Parent_Iy      
C	
C
C
C     **************************************************************************
CCC   Function Agrif_Iz
C     **************************************************************************
C 
      Function Agrif_Iz() 
C  
CCC   Description:
CCC   Function returning the minimal posiiton of the current grid in the z 
CCC   direction.
C
C     Declarations:
C  
      Implicit none
C
      Integer :: Agrif_Iz   ! Result 
C
C	 
      Agrif_Iz = AGRIF_CURGRID % iz 
C
C	 
      End function Agrif_Iz  	
C
C
C
C     **************************************************************************
CCC   Function Agrif_Parent_Iz
C     **************************************************************************
C 
      Function Agrif_Parent_Iz() 
C  
CCC   Description:
CCC   Function returning the minimal posiiton of the parent grid of the current 
CCC   grid in the z direction.
CCC    
C
C     Declarations:
C  
      Implicit none
C
      Integer :: Agrif_Parent_Iz   ! Result
C 
C	 
      Agrif_Parent_Iz = Agrif_CURGRID % parent % iz 
C
C	 
      End function Agrif_Parent_Iz
C
C
C
C     **************************************************************************
CCC   Subroutine Agrif_ChildGrid_to_ParentGrid
C     **************************************************************************
C
      Subroutine Agrif_ChildGrid_to_ParentGrid()
C  
CCC   Description:
CCC   Subroutine allowing to make the pointer AGRIF_CURGRID point on the parent
CCC   grid of the current grid.
C
C     Declarations:
C  
      Implicit none
C
C	
      AGRIF_saveCURGRID => AGRIF_CURGRID
C
      Call AGRIF_BACK_INSTANCE(AGRIF_CURGRID)
C
      Call AGRIF_INSTANCE(AGRIF_CURGRID%parent)	            
C
C
      End Subroutine Agrif_ChildGrid_to_ParentGrid
C
C
C
C     **************************************************************************
CCC   Subroutine Agrif_ParentGrid_to_ChildGrid
C     **************************************************************************
C
      Subroutine Agrif_ParentGrid_to_ChildGrid()
C  
CCC   Description:
CCC   Subroutine allowing to make the pointer AGRIF_CURGRID point on the child 
CCC   grid after having called the Agrif_ChildGrid_to_ParentGrid subroutine.
C
C     Declarations:
C  
      Implicit none
C
C	
      Call AGRIF_BACK_INSTANCE(AGRIF_CURGRID)
C
      Call AGRIF_INSTANCE(AGRIF_saveCURGRID)	            
C
C
      End Subroutine Agrif_ParentGrid_to_ChildGrid
C
C
C
C     ****************************************************************************
CCC   Function Agrif_Get_Unit
C     ***************************************************************************
C
      Function Agrif_Get_Unit()

CCC   Description : return a unit not connected to any file
C
C     Declarations
C
      Implicit none
C
      Integer Agrif_Get_Unit
C
C     Local scalars
      Integer n
      Logical op
C
C 
      Do n = 10,99
C 
        Inquire(Unit=n,Opened=op)
C      
        If (.not.op) exit
C      
      End Do
C
      Agrif_Get_Unit=n
C
C
      End Function Agrif_Get_Unit
C
C
C
      End Module Agrif_CurgridFunctions       
