!
! $Id: modmask.F,v 1.2 2003/12/17 13:56:10 pmarches Exp $
!
C     AGRIF (Adaptive Grid Refinement In Fortran)
C
C     Copyright (C) 2003 Laurent Debreu (Laurent.Debreu@imag.fr)
C                        Christophe Vouland (Christophe.Vouland@imag.fr)    
C
C     This program is free software; you can redistribute it and/or modify
C     it under the terms of the GNU General Public License as published by
C     the Free Software Foundation; either version 2 of the License, or
C     (at your option) any later version.
C
C     This program is distributed in the hope that it will be useful,
C     but WITHOUT ANY WARRANTY; without even the implied warranty of
C     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
C     GNU General Public License for more details.
C
C     You should have received a copy of the GNU General Public License
C     along with this program; if not, write to the Free Software
C     Foundation, Inc., 59 Temple Place -  Suite 330, Boston, MA 02111-1307, USA.
C
C
C
CCC   Module Agrif_Mask
C
      Module Agrif_Mask
C
CCC   Description:
CCC   Module for masks
C
C     Modules used: 
C
      Use Agrif_Types       
C
C
      CONTAINS
C     Define procedures contained in this module
C
C
C
C     **************************************************************************
C     Subroutine Agrif_CheckMask1D
C     **************************************************************************
C       
      Subroutine Agrif_CheckMask1D(tempP,parent,pbtab,petab,ppbtab,
     &            ppetab,noraftab)
C
CCC   Description:
CCC   Subroutine called in the procedure Agrif_Interp1D to recalculate the value 
CCC   of the parent grid variable when this one is equal to Agrif_SpecialValue. 
C
C     Declarations:
C
      Implicit none 
C
C     Arrays arguments      
      Integer,Dimension(1) :: pbtab  ! Limits of the parent grid used for the  
      Integer,Dimension(1) :: petab  ! interpolation of the child grid
      Logical,Dimension(1) :: noraftab
      Integer,Dimension(1) :: ppbtab,ppetab      
C
C     Pointer argument
      Real, Dimension(pbtab(1):petab(1)) :: tempP  ! Part of the parent grid used for 
                                             ! the interpolation of the child grid                     
C     Data type argument                                   
      Real, Dimension(ppbtab(1):ppetab(1)) :: parent      ! The parent grid
C
C     Local scalar
      Integer :: i0
      Integer :: i,ii 
      Integer :: imin,imax
C      
C     Local arrays 

C
C
      Do i0 = pbtab(1),petab(1)
C
        If (tempP(i0) == Agrif_SpecialValue) then               
C
            i = 1
            
      EXTER:Do While(i <= max(ppetab(1)-i0,i0-ppbtab(1)))             
C
              If (noraftab(1)) then
C
                  imin = i0
                  imax = i0
C
                Else
C
                  imin = max(i0 - i,ppbtab(1))
                  imax = min(i0 + i,ppetab(1))
C
              Endif
C
              Do ii = imin,imax
C
               If (parent (ii) /= Agrif_SpecialValue) then
C
                  tempP(i0) = parent (ii)
C
                  exit EXTER
C
               End if
C
              End do
C              
              i = i + 1                      
C
              If (i > 5) exit EXTER
C            
            End do EXTER            
C
        End if
          
      End do
C      
C     
      End Subroutine Agrif_CheckMask1D
C
C
C
C     **************************************************************************
C     Subroutine Agrif_CheckMask2D
C     **************************************************************************
C       
      Subroutine Agrif_CheckMask2D(tempP,parent,pbtab,petab,ppbtab,
     &               ppetab,noraftab)
C
CCC   Description:
CCC   Subroutine called in the procedure Agrif_Interp2D to recalculate the value 
CCC   of the parent grid variable when this one is equal to Agrif_SpecialValue. 
C
C     Declarations:
C
      Implicit none 
C
C     Arrays arguments      
      Integer,Dimension(2) :: pbtab  ! Limits of the parent grid used for the  
      Integer,Dimension(2) :: petab  ! interpolation of the child grid
      Logical,Dimension(2) :: noraftab
      Integer,Dimension(2) :: ppbtab,ppetab
C
C     Pointer argument
      Real, Dimension(pbtab(1):petab(1),pbtab(2):petab(2)) :: tempP  ! Part of the parent grid used for 
                                             ! the interpolation of the child grid                     
C     Data type argument                                   
      Real, Dimension(ppbtab(1):ppetab(1),ppbtab(2):ppetab(2)) :: parent      ! The parent grid
C
C     Local scalar
      Integer :: i0,j0
      Integer :: i,ii,jj 
      Integer :: imin,imax
      Integer :: jmin,jmax
C      
C     Local arrays      

C                                         
C
      Do j0 = pbtab(2),petab(2)
C
        Do i0 = pbtab(1),petab(1)
C
          If (tempP(i0,j0) == Agrif_SpecialValue) then               
C
              i = 1
            
      EXTER:  Do While(i <= max(ppetab(1)-i0,i0-ppbtab(1),
     &                          ppetab(2)-j0,j0-ppbtab(2)))             
C
                If (noraftab(1)) then
C
                    imin = i0
                    imax = i0
C
                  Else
C
                    imin = max(i0 - i,ppbtab(1))
                    imax = min(i0 + i,ppetab(1))
C
                Endif
C
                If (noraftab(2)) then
C
                    jmin = j0
                    jmax = j0
C
                  Else
C
                    jmin = max(j0 - i,ppbtab(2))
                    jmax = min(j0 + i,ppetab(2))
C
                Endif
C
                Do jj = jmin,jmax
C
                  Do ii = imin,imax
C
                    If (parent (ii,jj)
     &                            /= Agrif_SpecialValue) then
C
                        tempP(i0,j0) = parent (ii,jj)
C
                        exit EXTER
C
                    End if 
C
                  End do
C
                End do
C              
                i = i + 1                      
C
                If (i > 5) exit EXTER
C            
              End do EXTER            
C
          End if
C
        End do
C
      End do
C      
C     
      End Subroutine Agrif_CheckMask2D
C
C
C
C     **************************************************************************
C     Subroutine Agrif_CheckMask3D
C     **************************************************************************
C       
      Subroutine Agrif_CheckMask3D(tempP,parent,pbtab,petab,ppbtab,
     &         ppetab,noraftab)
C
CCC   Description:
CCC   Subroutine called in the procedure Agrif_Interp3D to recalculate the value 
CCC   of the parent grid variable when this one is equal to Agrif_SpecialValue. 
C
C     Declarations:
C
      Implicit none 
C
C     Arrays arguments      
      Integer,Dimension(3) :: pbtab  ! Limits of the parent grid used for the  
      Integer,Dimension(3) :: petab          ! interpolation of the child grid
      Logical,Dimension(3) :: noraftab
      Integer,Dimension(3) :: ppbtab,ppetab
C
C     Pointer argument
      Real, Dimension(pbtab(1):petab(1),pbtab(2):petab(2),
     &             pbtab(3):petab(3)) :: tempP  ! Part of the parent grid used for 
                                             ! the interpolation of the child grid                     
C     Data type argument                                   
      Real, Dimension(ppbtab(1):ppetab(1),ppbtab(2):ppetab(2),
     &             ppbtab(3):ppetab(3)) :: parent      ! The parent grid
C
C     Local scalar
      Integer :: i0,j0,k0
      Integer :: i,ii,jj,kk 
      Integer :: imin,imax
      Integer :: jmin,jmax
      Integer :: kmin,kmax
C      
C     Local arrays      
         
C
      Do k0=pbtab(3),petab(3)
C
        Do j0 = pbtab(2),petab(2)
C
          Do i0 = pbtab(1),petab(1)
C
            If (tempP(i0,j0,k0) == Agrif_SpecialValue) then               
C
                i = 1
            
      EXTER:    Do While (i <= max(ppetab(1)-i0,i0-ppbtab(1),
     &                             ppetab(2)-j0,j0-ppbtab(2),
     &                             ppetab(3)-k0,k0-ppbtab(3)))
C
                  If (noraftab(1)) then
C
                      imin = i0
                      imax = i0
C
                    Else
C
                      imin = max(i0 - i,ppbtab(1))
                      imax = min(i0 + i,ppetab(1))
C
                  Endif
C
                  If (noraftab(2)) then
C
                      jmin = j0
                      jmax = j0
C
                    Else
C
                      jmin = max(j0 - i,ppbtab(2))
                      jmax = min(j0 + i,ppetab(2))
C
                  Endif
C
                  If (noraftab(3)) then
C
                      kmin = k0
                      kmax = k0
C
                    Else
C
                      kmin = max(k0 - i,ppbtab(3))
                      kmax = min(k0 + i,ppetab(3))
C
                  Endif
C
                  Do kk = kmin,kmax 
C
                    Do jj = jmin,jmax
C
                      Do ii = imin,imax
C
                        If (parent(ii,jj,kk)
     &                      /= Agrif_SpecialValue) then
C
                            tempP(i0,j0,k0) =
     &                      parent (ii,jj,kk)
C
                            exit EXTER
C
                        End if
C
                      End do 
C
                    End do
C
                  End do
C              
                  i = i + 1                      
C
                  If (i > 5) exit EXTER
C            
                End do EXTER            
C
            End if
C
          End do
C
        End do
C
      End do
C      
C     
      End Subroutine Agrif_CheckMask3D
C
C
C
C     **************************************************************************
C     Subroutine Agrif_CheckMask4D
C     **************************************************************************
C       
      Subroutine Agrif_CheckMask4D(tempP,parent,pbtab,petab,ppbtab,
     &    ppetab,noraftab)
C
CCC   Description:
CCC   Subroutine called in the procedure Agrif_Interp4D to recalculate the value 
CCC   of the parent grid variable when this one is equal to Agrif_SpecialValue. 
C
C     Declarations:
C
      Implicit none 
C
C     Arrays arguments      
      Integer,Dimension(4) :: pbtab  ! Limits of the parent grid used for the  
      Integer,Dimension(4) :: petab          ! interpolation of the child grid
      Logical,Dimension(4) :: noraftab
      Integer,Dimension(4) :: ppbtab,ppetab
C
C     Pointer argument
      Real, Dimension(pbtab(1):petab(1),pbtab(2):petab(2),
     &   pbtab(3):petab(3),pbtab(4):petab(4)) :: tempP  ! Part of the parent grid used for 
                                             ! the interpolation of the child grid                     
C     Data type argument                                   
      Real, Dimension(ppbtab(1):ppetab(1),ppbtab(2):ppetab(2),
     &   ppbtab(3):ppetab(3),ppbtab(4):ppetab(4)) :: parent      ! The parent grid
C
C     Local scalar
      Integer :: i0,j0,k0,l0
      Integer :: i,ii,jj,kk,ll 
      Integer :: imin,imax
      Integer :: jmin,jmax
      Integer :: kmin,kmax
      Integer :: lmin,lmax
C      
C     Local arrays      

C                                      
C
      Do l0=pbtab(4),petab(4) 

        Do k0=pbtab(3),petab(3)
C
          Do j0 = pbtab(2),petab(2)
C
            Do i0 = pbtab(1),petab(1)
C
              If (tempP(i0,j0,k0,l0) == Agrif_SpecialValue) then               
C
                  i = 1
            
      EXTER:      Do While (i <= max(ppetab(1)-i0,i0-ppbtab(1),
     &                               ppetab(2)-j0,j0-ppbtab(2),
     &                               ppetab(3)-k0,k0-ppbtab(3),
     &                               ppetab(4)-l0,l0-ppbtab(4)))             
C
                    If (noraftab(1)) then
C
                        imin = i0
                        imax = i0
C
                      Else
C
                        imin = max(i0 - i,ppbtab(1))
                        imax = min(i0 + i,ppetab(1))
C
                    Endif
C
                    If (noraftab(2)) then
C
                        jmin = j0
                        jmax = j0
C
                      Else
C
                        jmin = max(j0 - i,ppbtab(2))
                        jmax = min(j0 + i,ppetab(2))
C
                    Endif
C
                    If (noraftab(3)) then
C
                        kmin = k0
                        kmax = k0
C
                      Else
C
                        kmin = max(k0 - i,ppbtab(3))
                        kmax = min(k0 + i,ppetab(3))
C
                    Endif
            
                    If (noraftab(4)) then
C
                        lmin = l0
                        lmax = l0
C
                      Else
C
                        lmin = max(l0 - i,ppbtab(4))
                        lmax = min(l0 + i,ppetab(4))
C
                    Endif
C
                    Do ll = lmin,lmax 
C
                      Do kk = kmin,kmax 
C
                        Do jj = jmin,jmax
C
                          Do ii = imin,imax
C
                            If (parent(ii,jj,kk,ll)
     &                          /= Agrif_SpecialValue) then
C
                                tempP(i0,j0,k0,l0) =
     &                          parent(ii,jj,kk,ll)
C
                                exit EXTER
C
                            End if
C
                          End do 
C
                        End do
C
                      End do
            
                    End do
C              
                    i = i + 1                      
C
                    If (i > 5) exit EXTER
C            
                  End do EXTER            
C
              End if
C
            End do
C
          End do
C
        End do
C
      End do      
C      
C     
      End Subroutine Agrif_CheckMask4D
C
C
C
C     **************************************************************************
C     Subroutine Agrif_CheckMask5D
C     **************************************************************************
C       
      Subroutine Agrif_CheckMask5D(tempP,parent,pbtab,petab,ppbtab,
     &      ppetab,noraftab)
C
CCC   Description:
CCC   Subroutine called in the procedure Agrif_Interp5D to recalculate the value 
CCC   of the parent grid variable when this one is equal to Agrif_SpecialValue. 
C
C     Declarations:
C
      Implicit none 
C
C     Arrays arguments      
      Integer,Dimension(5) :: pbtab  ! Limits of the parent grid used for the  
      Integer,Dimension(5) :: petab          ! interpolation of the child grid
      Logical,Dimension(5) :: noraftab
      Integer,Dimension(5) :: ppbtab,ppetab      
C
C     Pointer argument
      Real, Dimension(pbtab(1):petab(1),pbtab(2):petab(2),
     & pbtab(3):petab(3),pbtab(4):petab(4),pbtab(5):petab(5)) :: tempP  ! Part of the parent grid used for 
                                             ! the interpolation of the child grid                     
C     Data type argument                                   
      Real, Dimension(ppbtab(1):ppetab(1),ppbtab(2):ppetab(2),
     & ppbtab(3):ppetab(3),ppbtab(4):ppetab(4),
     & ppbtab(5):ppetab(5)) :: parent      ! The parent grid
C
C     Local scalar
      Integer :: i0,j0,k0,l0,m0
      Integer :: i,ii,jj,kk,ll,mm 
      Integer :: imin,imax
      Integer :: jmin,jmax
      Integer :: kmin,kmax
      Integer :: lmin,lmax
      Integer :: mmin,mmax
C
C     Local arrays      
                                    
C
      Do m0=pbtab(5),petab(5) 
C
        Do l0=pbtab(4),petab(4) 
C
          Do k0=pbtab(3),petab(3)
C
            Do j0 = pbtab(2),petab(2)
C
              Do i0 = pbtab(1),petab(1)
C
                If (tempP(i0,j0,k0,l0,m0) == Agrif_SpecialValue) then               
C
                    i = 1
            
      EXTER:        Do While(i <= max(ppetab(1)-i0,i0-ppbtab(1),
     &                                ppetab(2)-j0,j0-ppbtab(2),
     &                                ppetab(3)-k0,k0-ppbtab(3),
     &                                ppetab(4)-l0,l0-ppbtab(4),
     &                                ppetab(5)-m0,m0-ppbtab(5)))             
C
                      If (noraftab(1)) then
C
                          imin = i0
                          imax = i0
C
                        Else
C
                          imin = max(i0 - i,ppbtab(1))
                          imax = min(i0 + i,ppetab(1))
C
                      Endif
C
                      If (noraftab(2)) then
C
                          jmin = j0
                          jmax = j0
C
                        Else
C
                          jmin = max(j0 - i,ppbtab(2))
                          jmax = min(j0 + i,ppetab(2))
C
                      Endif
C
                      If (noraftab(3)) then
C
                          kmin = k0
                          kmax = k0
C
                        Else
C
                          kmin = max(k0 - i,ppbtab(3))
                          kmax = min(k0 + i,ppetab(3))
C
                      Endif
            
                      If (noraftab(4)) then
C
                          lmin = l0
                          lmax = l0
C
                        Else
C
                          lmin = max(l0 - i,ppbtab(4))
                          lmax = min(l0 + i,ppetab(4))
C
                      Endif
            
                      If (noraftab(5)) then
C
                          mmin = m0
                          mmax = m0
C
                        Else
C
                          mmin = max(m0 - i,ppbtab(5))
                          mmax = min(m0 + i,ppetab(5))
C
                      Endif
C
                      Do mm = mmin,mmax   

                        Do ll = lmin,lmax 
C
                          Do kk = kmin,kmax 
C
                            Do jj = jmin,jmax
C
                              Do ii = imin,imax
C
                                If (parent
     &                              (ii,jj,kk,ll,mm)
     &                              /= Agrif_SpecialValue) then
C
                                    tempP(i0,j0,k0,l0,m0) =
     &                              parent
     &                              (ii,jj,kk,ll,mm)
C
                                    exit EXTER
C
                                End if
C
                              End do 
C
                            End do
C
                          End do
            
                        End do
            
                      End do
C              
                      i = i + 1                      
C
                      If (i > 5) exit EXTER
C            
                    End do EXTER            
C
                End if
C
              End do
C
            End do
C
          End do
C
        End do      
      
      End do
C      
C     
      End Subroutine Agrif_CheckMask5D
C
C
C
      End Module Agrif_Mask       
