!
! $Id: modupdate.F,v 1.4 2004/11/29 13:26:08 pmarches Exp $
!
C     AGRIF (Adaptive Grid Refinement In Fortran)
C
C     Copyright (C) 2003 Laurent Debreu (Laurent.Debreu@imag.fr)
C                        Christophe Vouland (Christophe.Vouland@imag.fr)    
C
C     This program is free software; you can redistribute it and/or modify
C     it under the terms of the GNU General Public License as published by
C     the Free Software Foundation; either version 2 of the License, or
C     (at your option) any later version.
C
C     This program is distributed in the hope that it will be useful,
C     but WITHOUT ANY WARRANTY; without even the implied warranty of
C     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
C     GNU General Public License for more details.
C
C     You should have received a copy of the GNU General Public License
C     along with this program; if not, write to the Free Software
C     Foundation, Inc., 59 Temple Place -  Suite 330, Boston, MA 02111-1307, USA.
C
C
C
CCC   Module Agrif_Update
C
      Module Agrif_Update
C
CCC   Description:
CCC   Module to update a parent grid from its child grids
C
C     Modules used:
C   
      Use Agrif_Interpolation
      Use Agrif_Updatebasic
C      
      CONTAINS
C     Define procedures contained in this module
C
C
C
C     **************************************************************************
CCC   Subroutine Agrif_Update_1d
C     **************************************************************************
C 
      Subroutine Agrif_Update_1d(TypeUpdate,parent,child,tab)      	
C
CCC   Description:
CCC   Subroutine to update a 1D grid variable on the parent grid.
C
C     Declarations:
C	
      Implicit none
C
C     Arguments      
      Integer :: TypeUpdate                  ! Type of update (copy or average)
      Type(AGRIF_PVariable) :: parent        ! Variable on the parent grid
      Type(AGRIF_PVariable) :: child         ! Variable on the child grid
      Type(AGRIF_PVariable) :: childtemp     ! Temporary variable on the child grid
      Real, Dimension(lbound(child%var%array1,1):
     &                ubound(child%var%array1,1)), Target :: tab  ! Results
C
C
C     Definition of a temporary AGRIF_PVariable data type  
      allocate(childtemp % var)  
C
C     Pointer on the root variable
      childtemp % var % root_var => child % var %root_var
C      
C     Number of dimensions of the grid variable
      childtemp % var % nbdim = 1  
C      
C     Values on the current grid used for the update
      childtemp % var % array1 => tab      
C	
      Call Agrif_UpdateVariable
     &     (TypeUpdate,parent,childtemp)
C      
      deallocate(childtemp % var)
C
C       
      End Subroutine Agrif_Update_1D
C
C
C
C     **************************************************************************
CCC   Subroutine Agrif_Update_2d
C     **************************************************************************
C 
      Subroutine Agrif_Update_2d(TypeUpdate,parent,child,tab)      	
C
CCC   Description:
CCC   Subroutine to update a 2D grid variable on the parent grid.
C
C     Declarations:
C	
      Implicit none
C
C     Arguments      
      Integer :: TypeUpdate                  ! Type of update (copy or average)
      Type(AGRIF_PVariable) :: parent        ! Variable on the parent grid
      Type(AGRIF_PVariable) :: child         ! Variable on the child grid
      Type(AGRIF_PVariable) :: childtemp     ! Temporary variable on the child grid
      Real, Dimension(
     &      lbound(child%var%array2,1):ubound(child%var%array2,1),
     &      lbound(child%var%array2,2):ubound(child%var%array2,2)),
     &      Target :: tab  ! Results
C
C
C     Definition of a temporary AGRIF_PVariable data type 
      allocate(childtemp % var)  
C
C     Pointer on the root variable
      childtemp % var % root_var => child % var %root_var
C      
C     Number of dimensions of the grid variable
      childtemp % var % nbdim = 2  
C      
C     Values on the current grid used for the update
      childtemp % var % array2 => tab      
C	
      Call Agrif_UpdateVariable
     &     (TypeUpdate,parent,childtemp)
C      
      deallocate(childtemp % var)
C
C       
      End Subroutine Agrif_Update_2D
C
C
C
C     **************************************************************************
CCC   Subroutine Agrif_Update_3d
C     **************************************************************************
C 
      Subroutine Agrif_Update_3d(TypeUpdate,parent,child,tab)      	
C
CCC   Description:
CCC   Subroutine to update a 3D grid variable on the parent grid.
C
C     Declarations:
C	
      Implicit none
C
C     Arguments      
      Integer :: TypeUpdate                  ! Type of update (copy or average)
      Type(AGRIF_PVariable) :: parent        ! Variable on the parent grid
      Type(AGRIF_PVariable) :: child         ! Variable on the child grid
      Type(AGRIF_PVariable) :: childtemp     ! Temporary variable on the child grid
      Real, Dimension(
     &      lbound(child%var%array3,1):ubound(child%var%array3,1),
     &      lbound(child%var%array3,2):ubound(child%var%array3,2),
     &      lbound(child%var%array3,3):ubound(child%var%array3,3)),
     &      Target :: tab  ! Results
C
C
C     Definition of a temporary AGRIF_PVariable data type  
      allocate(childtemp % var)  
C
C     Pointer on the root variable
      childtemp % var % root_var => child % var %root_var
C      
C     Number of dimensions of the grid variable
      childtemp % var % nbdim = 3  
C      
C     Values on the current grid used for the update
      childtemp % var % array3 => tab     
C	
      Call Agrif_UpdateVariable
     &     (TypeUpdate,parent,childtemp)
C      
      deallocate(childtemp % var)
C
C       
      End Subroutine Agrif_Update_3D
C
C
C
C     **************************************************************************
CCC   Subroutine Agrif_Update_4d
C     **************************************************************************
C 
      Subroutine Agrif_Update_4d(TypeUpdate,parent,child,tab)      	
C
CCC   Description:
CCC   Subroutine to update a 4D grid variable on the parent grid.
C
C     Declarations:
C	
      Implicit none
C
C     Arguments      
      Integer :: TypeUpdate                  ! Type of update (copy or average)
      Type(AGRIF_PVariable) :: parent        ! Variable on the parent grid
      Type(AGRIF_PVariable) :: child         ! Variable on the child grid
      Type(AGRIF_PVariable) :: childtemp     ! Temporary variable on the child grid
      Real, Dimension(
     &      lbound(child%var%array4,1):ubound(child%var%array4,1),
     &      lbound(child%var%array4,2):ubound(child%var%array4,2),
     &      lbound(child%var%array4,3):ubound(child%var%array4,3),
     &      lbound(child%var%array4,4):ubound(child%var%array4,4)),
     &      Target :: tab  ! Results
C
C
C     Definition of a temporary AGRIF_PVariable data type  
      allocate(childtemp % var)  
C
C     Pointer on the root variable
      childtemp % var % root_var => child % var %root_var
C      
C     Number of dimensions of the grid variable
      childtemp % var % nbdim = 4  
C      
C     Values on the current grid used for the update
      childtemp % var % array4 => tab     
C	
      Call Agrif_UpdateVariable
     &     (TypeUpdate,parent,childtemp)
C      
      deallocate(childtemp % var)
C
C       
      End Subroutine Agrif_Update_4D
C
C
C
C     **************************************************************************
CCC   Subroutine Agrif_Update_5d
C     **************************************************************************
C 
      Subroutine Agrif_Update_5d(TypeUpdate,parent,child,tab)      	
C
CCC   Description:
CCC   Subroutine to update a 5D grid variable on the parent grid.
C
C     Declarations:
C	
      Implicit none
C
C     Arguments      
      Integer :: TypeUpdate                  ! Type of update (copy or average)
      Type(AGRIF_PVariable) :: parent        ! Variable on the parent grid
      Type(AGRIF_PVariable) :: child         ! Variable on the child grid
      Type(AGRIF_PVariable) :: childtemp     ! Temporary variable on the child grid
      Real, Dimension(
     &      lbound(child%var%array5,1):ubound(child%var%array5,1),
     &      lbound(child%var%array5,2):ubound(child%var%array5,2),
     &      lbound(child%var%array5,3):ubound(child%var%array5,3),
     &      lbound(child%var%array5,4):ubound(child%var%array5,4),
     &      lbound(child%var%array5,5):ubound(child%var%array5,5)),
     &      Target :: tab  ! Results
C
C
C     Definition of a temporary AGRIF_PVariable data type  
      allocate(childtemp % var)  
C
C     Pointer on the root variable
      childtemp % var % root_var => child % var %root_var
C      
C     Number of dimensions of the grid variable
      childtemp % var % nbdim = 5  
C      
C     Values on the current grid used for the update
      childtemp % var % array5 => tab      
C	
      Call Agrif_UpdateVariable
     &     (TypeUpdate,parent,childtemp)
C      
      deallocate(childtemp % var)
C
C       
      End Subroutine Agrif_Update_5D
C
C
C
C     **************************************************************************  
C     Subroutine Agrif_UpdateVariable    
C     **************************************************************************  
C   
      Subroutine Agrif_UpdateVariable(TypeUpdate,parent,child)   
C
CCC   Description:
CCC   Subroutine to set arguments of Agrif_UpdatenD, n being the number of dimensions
CCC   of the grid variable.
C
CC    Declarations:
C      
c      Implicit none
C      
#include "include_agrif.h"
C
C     Scalar argument
      Integer :: TypeUpdate             ! Type of interpolation (linear,spline,...)
C     Data type arguments                                   
      Type(AGRIF_PVariable) :: parent   ! Variable on the parent grid
      Type(AGRIF_PVariable) :: child    ! Variable on the child grid  
C
C     Local scalars      
      Integer :: nbdim                  ! Number of dimensions of the current grid      
      Integer :: nn      
      Integer,Dimension(5) :: pttab_child  
      Integer,Dimension(5) :: petab_child      
      Integer,Dimension(5) :: pttab_parent  
      Integer,Dimension(5) :: petab_parent      
      Integer,Dimension(5) :: itab_child      
      Integer,Dimension(5) :: rhotab_child    ! Space refinement factors 
      Integer,Dimension(5) :: nbtab_child     ! Number of cells in each direction
      Integer,Dimension(5) :: posvartab_child ! Position of the variable on a cell
      Type(AGRIF_Variable), Pointer :: root ! Pointer on the variable of the 
                                            ! root grid
      Type(Agrif_Grid), Pointer :: Agrif_Child_Gr,Agrif_Parent_Gr
      Real, Dimension(5) :: s_child,s_parent
      Real, Dimension(5) :: ds_child,ds_parent
C
C
      Agrif_Child_Gr => Agrif_Curgrid
      Agrif_Parent_Gr => Agrif_Curgrid % parent
C
      root => child % var % root_var 
C
C     Number of dimensions of the current grid
      nbdim = root % nbdim
C       
      Do nn=1,nbdim
C      
        posvartab_child(nn) = root % posvar(nn)      
C      
      End Do            
C      
C      
      Do nn=1,nbdim
C            
        Select case(root % interptab(nn))
C
C       Value of interptab(nn) can be either x,y,z or noint for a no space 
C       dimension            
C
C         The dimension is 'x'
          case('x')
C
            pttab_Child(nn) = root % pointX 
C            
            pttab_Parent(nn) = root % pointx
C        
            s_Child(nn) = Agrif_Child_Gr % Agrif_x
C
            s_Parent(nn) = Agrif_Parent_Gr % Agrif_x 
C        
            ds_Child(nn) = Agrif_Child_Gr % Agrif_dx
C
            ds_Parent(nn) = Agrif_Parent_Gr % Agrif_dx
C                      
            If (root % posvar(nn).EQ.1) Then
C          
            petab_Child(nn)=pttab_Child(nn)+Agrif_Child_Gr%nbx
C          
            petab_Parent(nn)=pttab_Parent(nn)+Agrif_Parent_Gr%nbx    
	
c		pttab_Child(nn) = pttab_Child(nn)+1
c		petab_Child(nn) = petab_Child(nn)-1   
c		s_Child(nn) = s_Child(nn) + ds_Child(nn)  
C        
              Else
C          
                petab_Child(nn) = pttab_Child(nn) + 
     &                           Agrif_Child_Gr%nbx - 1
C          
                petab_Parent(nn) = pttab_Parent(nn) + 
     &                           Agrif_Parent_Gr%nbx - 1
C
                s_Child(nn) = s_Child(nn) + ds_Child(nn)/2.
C
                s_Parent(nn) = s_Parent(nn) + ds_Parent(nn)/2.                                
C        
            End If                  
C
C         The dimension is 'y'
          case('y')
C
            pttab_Child(nn) = root % pointY
C            
            pttab_Parent(nn) = root % pointy
C        
            s_Child(nn) = Agrif_Child_Gr % Agrif_y
C
            s_Parent(nn) = Agrif_Parent_Gr % Agrif_y 
C        
            ds_Child(nn) = Agrif_Child_Gr % Agrif_dy
C
            ds_Parent(nn) = Agrif_Parent_Gr % Agrif_dy
C                      
            If (root % posvar(nn).EQ.1) Then
C          
             petab_Child(nn)=pttab_Child(nn)+Agrif_Child_Gr%nby
C          
             petab_Parent(nn)=pttab_Parent(nn)+Agrif_Parent_Gr%nby         
C        
c		pttab_Child(nn) = pttab_Child(nn)+1
c		petab_Child(nn) = petab_Child(nn)-1   
c		s_Child(nn)=s_Child(nn)+ds_Child(nn)
              Else
C          
                petab_Child(nn) = pttab_Child(nn) + 
     &                            Agrif_Child_Gr%nby - 1
C          
                petab_Parent(nn) = pttab_Parent(nn) + 
     &                             Agrif_Parent_Gr%nby - 1
C
                s_Child(nn) = s_Child(nn) + ds_Child(nn)/2.
C
                s_Parent(nn) = s_Parent(nn) + ds_Parent(nn)/2.                                
C        
            End If
            
C
C         The dimension is 'z'                            
          case('z')
C
            pttab_Child(nn) = root % pointZ
C            
            pttab_Parent(nn) = root % pointz
C        
            s_Child(nn) = Agrif_Child_Gr % Agrif_z
C
            s_Parent(nn) = Agrif_Parent_Gr % Agrif_z 
C        
            ds_Child(nn) = Agrif_Child_Gr % Agrif_dz
C
            ds_Parent(nn) = Agrif_Parent_Gr % Agrif_dz
C                      
            If (root % posvar(n).EQ.1) Then
C          
               petab_Child(nn)=pttab_Child(nn)+Agrif_Child_Gr%nbz
C          
               petab_Parent(nn)=pttab_Parent(nn)+Agrif_Parent_Gr%nbz         
C        
c		pttab_Child(nn) = pttab_Child(nn)+1
c		petab_Child(nn) = petab_Child(nn)-1   
c		s_Child(nn) = s_Child(nn) + ds_Child(nn)
              Else
C          
                petab_Child(nn) = pttab_Child(nn) + 
     &                            Agrif_Child_Gr%nbz - 1
C          
                petab_Parent(nn) = pttab_Parent(nn) + 
     &                            Agrif_Parent_Gr%nbz - 1
C
                s_Child(nn) = s_Child(nn) + ds_Child(nn)/2.
C
                s_Parent(nn) = s_Parent(nn) + ds_Parent(nn)/2.                                
C        
            End If       
C 
C         The dimension is not space                            
          case('noint')
C
C         The next coefficients are calculated in order to do a simple copy of the 
C         values of the grid variable when the procedure of interpolation is 
C         called for this dimension 
C      
            select case (nbdim) 
C      
              case(1)
C      
                pttab_Child(nn) = lbound(child % var % array1,nn)
                petab_Child(nn) = ubound(child % var % array1,nn)
C        
              case(2)
C      
                pttab_Child(nn) = lbound(child % var % array2,nn)
                petab_Child(nn) = ubound(child % var % array2,nn)
C      
              case(3)
C      
                pttab_Child(nn) = lbound(child % var % array3,nn)
                petab_Child(nn) = ubound(child % var % array3,nn)  
C      
              case(4)
C      
                pttab_Child(nn) = lbound(child % var % array4,nn)
                petab_Child(nn) = ubound(child % var % array4,nn)
C
              case(5)
C      
                pttab_Child(nn) = lbound(child % var % array5,nn) 
                petab_Child(nn) = ubound(child % var % array5,nn)     
C
            end select
C
C           No interpolation but only a copy of the values of the grid variable      
C
            pttab_Parent(nn) = pttab_Child(nn)
C
            petab_Parent(nn) = petab_Child(nn)
C              
            s_Child(nn)=0.
C      
            s_Parent(nn)=0. 
C      
            ds_Child(nn)=1.
C      
            ds_Parent(nn)=1.
C
        End select
C            
      End do     
C
C
C     Call to a procedure of update according to the number of dimensions of 
C     the grid variable
C
      If (nbdim .EQ. 1) then
C
          Call AGRIF_Update1D
     &            (TypeUpdate,parent,child,
     &             pttab_Child(1:nbdim),petab_Child(1:nbdim),
     &             pttab_Child(1:nbdim),petab_Parent(1:nbdim),
     &             s_Child(1:nbdim),s_Parent(1:nbdim),
     &             ds_Child(1:nbdim),ds_Parent(1:nbdim),nbdim)
C
        Else If (nbdim .EQ. 2) then      
C
          Call AGRIF_Update2D
     &            (TypeUpdate,parent,child,
     &             pttab_Child(1:nbdim),petab_Child(1:nbdim),
     &             pttab_Child(1:nbdim),petab_Parent(1:nbdim),
     &             s_Child(1:nbdim),s_Parent(1:nbdim),
     &             ds_Child(1:nbdim),ds_Parent(1:nbdim),nbdim)
C
        Else If (nbdim .EQ. 3) then      
C
          Call AGRIF_Update3D
     &            (TypeUpdate,parent,child,
     &             pttab_Child(1:nbdim),petab_Child(1:nbdim),
     &             pttab_Child(1:nbdim),petab_Parent(1:nbdim),
     &             s_Child(1:nbdim),s_Parent(1:nbdim),
     &             ds_Child(1:nbdim),ds_Parent(1:nbdim),nbdim)     
C
        Else If (nbdim .EQ. 4) then      
C
          Call AGRIF_Update4D
     &            (TypeUpdate,parent,child,
     &             pttab_Child(1:nbdim),petab_Child(1:nbdim),
     &             pttab_Child(1:nbdim),petab_Parent(1:nbdim),
     &             s_Child(1:nbdim),s_Parent(1:nbdim),
     &             ds_Child(1:nbdim),ds_Parent(1:nbdim),nbdim)  
C
        Else If (nbdim .EQ. 5) then      
C
          Call AGRIF_Update5D
     &            (TypeUpdate,parent,child,
     &             pttab_Child(1:nbdim),petab_Child(1:nbdim),
     &             pttab_Child(1:nbdim),petab_Parent(1:nbdim),
     &             s_Child(1:nbdim),s_Parent(1:nbdim),
     &             ds_Child(1:nbdim),ds_Parent(1:nbdim),nbdim) 
C     
      End if
C
      Return
C
C
      End subroutine Agrif_UpdateVariable
C
C
C
C     **************************************************************************  
CCC   Subroutine Agrif_Update1D  
C     **************************************************************************  
C  
      Subroutine Agrif_Update1D(TypeUpdate,parent,child,
     &                          pttab_Child,petab_child,
     &                          pttab_Parent,petab_Parent,
     &                          s_Child,s_Parent,ds_Child,ds_Parent,
     &                          nbdim)  
C
C     Description:
C     Subroutine to update a 1D grid variable on the parent grid of the current grid. 
C
C     Declarations:
C
      Implicit none
C
#ifdef AGRIF_MPI
C
#include "mpif.h"
C
#endif
C
C     Arguments
      Integer                    :: nbdim
      Integer                    :: TypeUpdate         ! Type of interpolation (linear,...)
      Type(AGRIF_PVARIABLE)      :: parent             ! Variable of the parent grid   
      Type(AGRIF_PVARIABLE)      :: child              ! Variable of the child grid
      Integer,Dimension(nbdim)   :: pttab_Child        ! Index of the first point inside the domain for the child grid variable 
      Integer,Dimension(nbdim)   :: petab_Child              
      Integer,Dimension(nbdim)   :: pttab_Parent       ! Index of the first point inside the domain for the parent grid variable
      Integer,Dimension(nbdim)   :: petab_Parent 
      Real,Dimension(nbdim)      :: s_Child,s_Parent   ! Positions of the parent and child grids
      Real,Dimension(nbdim)      :: ds_Child,ds_Parent ! Space steps of the parent and child grids                         
C
C     Local pointers
      Real, Dimension(:), Allocatable  :: tempP      ! Temporary parent grid variable	           							    
      Real, Dimension(:), Allocatable  :: tempC      ! Temporary child grid variable      							             
C
C     Local scalars        
      Integer                     :: i,lbglob,ubglob  
      Integer,Dimension(nbdim)    :: pttruetab,cetruetab
      Logical,Dimension(nbdim)    :: noraftab
      Integer,Dimension(nbdim)    :: indmin,indmax
      Real, Dimension(nbdim)      :: dim_newmin,dim_newmax
      Integer, Dimension(nbdim)   :: ind_gmin,ind_gmax  
      Real,Dimension(nbdim)       :: s_Child_temp,s_Parent_temp
      Integer,Dimension(nbdim,2)  :: parentglob
C
#ifdef AGRIF_MPI
C
      Integer                 :: code
      Integer,Dimension(nbdim,2) :: tab1
      Integer,Dimension(:,:,:),Allocatable :: tab2
      Integer,Dimension(nbdim,0:Agrif_Nbprocs-1) :: imin,imax
      Integer,Dimension(nbdim,0:Agrif_Nbprocs-1) :: iminloc,imaxloc
      Integer,Dimension(nbdim,0:Agrif_Nbprocs-1) :: imin_recv,imax_recv
      Logical,Dimension(0:Agrif_Nbprocs-1):: sendtoproc
      Logical,Dimension(0:Agrif_Nbprocs-1) :: recvfromproc
      Integer,Dimension(:,:,:),Allocatable :: parentarray
      Integer,Dimension(:,:,:),Allocatable :: childarray
      Integer                 :: j,k,size,pglob
      Logical                 :: res
      Integer,Parameter       :: etiquette = 100
      Integer,Dimension(MPI_STATUS_SIZE) :: statut  
      Real,Dimension(:),Allocatable  :: childvalues
C
#endif      
C
C
C     Calculation of the boundaries of the current grid necessary for the update of
C     the parent grid
      Call Agrif_Childbounds(nbdim,lbound(child%var%array1),
     &                       ubound(child%var%array1),pttab_child,
     &                       petab_child,pttruetab,cetruetab) 
C
C     Calculation of the boundaries of the parent grid
      parentglob(:,1) = lbound(parent % var % array1)
      parentglob(:,2) = ubound(parent % var % array1)
C
#ifdef AGRIF_MPI 
C
      Call Agrif_ChildGrid_to_ParentGrid()
C
      Do j = 1,2
C
        Do i = 1,nbdim
C  
          pglob = parentglob(i,j)
C
          Call Agrif_Invloc(pglob,Agrif_ProcRank,i,parentglob(i,j))
C
        End Do
C
      End Do
C
      Call Agrif_ParentGrid_to_ChildGrid()            
C
#endif
C
      Call Agrif_Prtbounds(nbdim,indmin,indmax,s_Parent_temp,
     &                     s_Child_temp,s_Child,ds_Child,
     &                     s_Parent,ds_Parent,
     &                     pttruetab,cetruetab,pttab_Child,
     &                     pttab_Parent,parentglob)      
C
C     Values on the parent grid
      allocate(tempP(indmin(1):indmax(1)))
C
C     Values on the child grid
      Allocate(tempC(pttruetab(1):cetruetab(1)))
C      
      tempC = 0.
C
C
#ifdef AGRIF_MPI
C
C
C     For a global index on the current grid, childarray gives the corresponding 
C     local index on the current processor
C
      Allocate(childarray(minval(pttruetab):maxval(cetruetab),nbdim,2))
C
      Call Agrif_GlobtoLocInd(childarray,lbound(child%var%array1),
     &                        ubound(child%var%array1),pttruetab,
     &                        cetruetab,nbdim,Agrif_Procrank)      
C   
C
      tempC(pttruetab(1):cetruetab(1)) = 
     &child % var % array1(childarray(pttruetab(1),1,2):
     &                     childarray(cetruetab(1),1,2))
C
#else
C
      tempC(pttruetab(1):cetruetab(1)) = 
     &child % var % array1(pttruetab(1):cetruetab(1))        
C
#endif           
C
C     Special values on the child grid
      If (Agrif_UseSpecialValueFineGrid) Then
C
          noraftab(1:nbdim) = 
     &    child % var % root_var % interptab(1:nbdim) .EQ. 'noint'
C
#ifdef AGRIF_MPI
C
          Allocate(childvalues(pttruetab(1):cetruetab(1)))
C
          childvalues = tempC
C
          Call Agrif_CheckMask1D(tempC,childvalues,
     &                           pttruetab(1:nbdim),cetruetab(1:nbdim),
     &                           pttruetab(1:nbdim),cetruetab(1:nbdim),
     &                           noraftab(1:nbdim))
C
          Deallocate(childvalues)
C
#else
C
          Call Agrif_CheckMask1D(tempC,child%var%array1,
     &                           pttruetab(1:nbdim),cetruetab(1:nbdim),
     &                           lbound(child%var%array1),
     &                           ubound(child%var%array1),
     &                           noraftab(1:nbdim))
C
#endif   	  
C
      End If
C
C
C     Update of the parent grid (tempP) from the child grid (tempC)
      Call Agrif_Update_1D_recursive(TypeUpdate,tempP,tempC,
     &           indmin,indmax,
     &           pttruetab,cetruetab,
     &           s_Child_temp,s_Parent_temp,
     &           ds_Child,ds_Parent,nbdim)
C
C
#ifdef AGRIF_MPI
C
C     For a global index on the parent grid, parentarray gives the corresponding 
C     local index on the current processor
C
      Allocate(parentarray(minval(parentglob(:,1)):
     &                     maxval(parentglob(:,2)),nbdim,2))
C
      Call Agrif_ChildGrid_to_ParentGrid()
C	
      Call Agrif_GlobtoLocInd(parentarray,lbound(parent%var%array1),
     &                        ubound(parent%var%array1),
     &                        parentglob(:,1),parentglob(:,2),
     &                        nbdim,Agrif_Procrank)      
C
      Call Agrif_ParentGrid_to_ChildGrid()  
C
#endif    
C
C
C     Special values on the parent grid
      If (Agrif_UseSpecialValue) Then
C
#ifdef AGRIF_MPI
C
          Where (parent % var % array1(
     &           parentarray(indmin(1),1,2):parentarray(indmax(1),1,2)) 
     &            == Agrif_SpecialValue)
C
#else
C	   
          Where (parent % var % array1(indmin(1):indmax(1)) 
     &            == Agrif_SpecialValue) 
C
#endif	
C   
       tempP(indmin(1):indmax(1))
     &      = Agrif_SpecialValue
C	      
          End Where
C
      End If	   
C
#ifdef AGRIF_MPI   
C 
      tab1(:,1) = indmin(:)
      tab1(:,2) = indmax(:) 
C
      Call MPI_BARRIER(MPI_COMM_WORLD,code)
C
C     tab2 contains boundaries of the parent grid on each processor
      Allocate(tab2(nbdim,2,0:Agrif_Nbprocs-1))
C     
      Call MPI_ALLGATHER(parentglob,2*nbdim,MPI_INTEGER,tab2,2*nbdim,
     &                   MPI_INTEGER,MPI_COMM_WORLD,code)                  
C      
C
C     Sentoproc indicates if some values of tempP on the current processor 
C     are sent to the others processors
C     Imin, imax, iminloc, imaxloc are the necessary boundaries of tempP
      Call Agrif_Sendtoproc2(Agrif_Nbprocs,Agrif_Nbprocs,sendtoproc,
     &                      nbdim,imin,imax,iminloc,imaxloc,tab1,
     &                      tab2,parentarray,lbound(parentarray,1),
     &                      ubound(parentarray,1))
C
C
      Do k = 0,Agrif_Nbprocs-1        
C
C       Copy of the values of tempP on the parent grid of the current grid
        If (k == Agrif_ProcRank) Then
C
            parent%var%array1(iminloc(1,k):imaxloc(1,k)) = 
     &      tempP(imin(1,k):imax(1,k))
C
          Else        
C
C           Parts of tempP on the current processor are sent to the others processors
            Call MPI_SEND(sendtoproc(k),1,MPI_LOGICAL,k,etiquette,
     &                    MPI_COMM_WORLD,code)
C
            If (sendtoproc(k) == .true.) Then
C
                Call MPI_SEND(imin(:,k),
     &                        nbdim,MPI_INTEGER,k,etiquette,
     &                        MPI_COMM_WORLD,code)
C
                Call MPI_SEND(imax(:,k),
     &                        nbdim,MPI_INTEGER,k,etiquette,
     &                        MPI_COMM_WORLD,code)
C
                size = 1
C
                Do i = 1,nbdim
C
                  size = size * (imax(i,k)-imin(i,k)+1)
C
                End Do
C
                Call MPI_SEND(tempP(
     &                        imin(1,k):imax(1,k)),
     &                        size,MPI_DOUBLE_PRECISION,k,etiquette,
     &                        MPI_COMM_WORLD,code)  
C
            End If
C
        End If
C
      End Do  
C
C
      Do k = 0,Agrif_Nbprocs-1        
C
        If (k /= Agrif_ProcRank) Then
C
            Call MPI_RECV(res,1,MPI_LOGICAL,k,etiquette,
     &                    MPI_COMM_WORLD,statut,code) 
C
            recvfromproc(k) = res       
C
C           Current processor receives parts of tempP from others processors
            If (recvfromproc(k) == .true.) Then
C
                Call MPI_RECV(imin_recv(:,k),
     &                        nbdim,MPI_INTEGER,k,etiquette,
     &                        MPI_COMM_WORLD,statut,code)
C
                Call MPI_RECV(imax_recv(:,k),
     &                        nbdim,MPI_INTEGER,k,etiquette,
     &                        MPI_COMM_WORLD,statut,code)
C
                size = 1
C
                Do i = 1,nbdim
C
                  size = size * (imax_recv(i,k)-imin_recv(i,k)+1)
C
                End Do
C
                Call MPI_RECV(parent%var%array1(
     &                        parentarray(imin_recv(1,k),1,2):
     &                        parentarray(imax_recv(1,k),1,2)),
     &                        size,MPI_DOUBLE_PRECISION,k,etiquette,
     &                        MPI_COMM_WORLD,statut,code)
C
            End If 
C
        End If
C
      End Do 
C      
#else
C
      parent % var % array1(indmin(1):indmax(1)) 
     &= tempP(indmin(1):indmax(1)) 
C
#endif               
C		  	      	    	      		   				  	       	  	                         		  	            
C     Deallocations      
      deallocate(tempP)
      deallocate(tempC)
C
#ifdef AGRIF_MPI
C
      deallocate(parentarray,childarray) 
C
#endif      
C       
C		  
      End Subroutine Agrif_Update1D
C
C
C
C     **************************************************************************  
CCC   Subroutine Agrif_Update2D  
C     **************************************************************************  
C  
      Subroutine Agrif_Update2D(TypeUpdate,parent,child,
     &                          pttab_Child,petab_child,
     &                          pttab_Parent,petab_Parent,
     &                          s_Child,s_Parent,ds_Child,ds_Parent,
     &                          nbdim)  
C
C     Description:
C     Subroutine to update a 2D grid variable on the parent grid of the current grid. 
C
C     Declarations:
C
      Implicit none
C
#ifdef AGRIF_MPI
C
#include "mpif.h"
C
#endif
C
C     Arguments
      Integer                    :: nbdim
      Integer                    :: TypeUpdate         ! Type of interpolation (linear,...)
      Type(AGRIF_PVARIABLE)      :: parent             ! Variable of the parent grid   
      Type(AGRIF_PVARIABLE)      :: child              ! Variable of the child grid
      Integer,Dimension(nbdim)   :: pttab_Child        ! Index of the first point inside the domain for the child grid variable 
      Integer,Dimension(nbdim)   :: petab_Child              
      Integer,Dimension(nbdim)   :: pttab_Parent       ! Index of the first point inside the domain for the parent grid variable
      Integer,Dimension(nbdim)   :: petab_Parent 
      Real,Dimension(nbdim)      :: s_Child,s_Parent   ! Positions of the parent and child grids
      Real,Dimension(nbdim)      :: ds_Child,ds_Parent ! Space steps of the parent and child grids                         
C
C     Local pointers
      Real, Dimension(:,:), Allocatable  :: tempP      ! Temporary parent grid variable	           							    
      Real, Dimension(:,:), Allocatable  :: tempC      ! Temporary child grid variable      							             
C
C     Local scalars        
      Integer                     :: i,lbglob,ubglob  
      Integer,Dimension(nbdim)    :: pttruetab,cetruetab
      Logical,Dimension(nbdim)    :: noraftab
      Integer,Dimension(nbdim)    :: indmin,indmax
      Real, Dimension(nbdim)      :: dim_newmin,dim_newmax
      Integer, Dimension(nbdim)   :: ind_gmin,ind_gmax  
      Real,Dimension(nbdim)       :: s_Child_temp,s_Parent_temp
      Integer,Dimension(nbdim,2)  :: parentglob
C
#ifdef AGRIF_MPI
C
      Integer                 :: code
      Integer,Dimension(nbdim,2) :: tab1
      Integer,Dimension(:,:,:),Allocatable :: tab2
      Integer,Dimension(nbdim,0:Agrif_Nbprocs-1) :: imin,imax
      Integer,Dimension(nbdim,0:Agrif_Nbprocs-1) :: iminloc,imaxloc
      Integer,Dimension(nbdim,0:Agrif_Nbprocs-1) :: imin_recv,imax_recv
      Logical,Dimension(0:Agrif_Nbprocs-1):: sendtoproc
      Logical,Dimension(0:Agrif_Nbprocs-1) :: recvfromproc
      Integer,Dimension(:,:,:),Allocatable :: parentarray
      Integer,Dimension(:,:,:),Allocatable :: childarray
      Integer                 :: j,k,size,pglob
      Logical                 :: res
      Integer,Parameter       :: etiquette = 100
      Integer,Dimension(MPI_STATUS_SIZE) :: statut  
      Real,Dimension(:,:),Allocatable  :: childvalues
C
#endif      
C
C
C     Calculation of the boundaries of the current grid necessary for the update of
C     the parent grid
      Call Agrif_Childbounds(nbdim,lbound(child%var%array2),
     &                       ubound(child%var%array2),pttab_child,
     &                       petab_child,pttruetab,cetruetab) 
C
C     Calculation of the boundaries of the parent grid
      parentglob(:,1) = lbound(parent % var % array2)
      parentglob(:,2) = ubound(parent % var % array2)
C
#ifdef AGRIF_MPI 
C
      Call Agrif_ChildGrid_to_ParentGrid()
C
      Do j = 1,2
C
        Do i = 1,nbdim
C  
          pglob = parentglob(i,j)
C
          Call Agrif_Invloc(pglob,Agrif_ProcRank,i,parentglob(i,j))
C
        End Do
C
      End Do
C
      Call Agrif_ParentGrid_to_ChildGrid()            
C
#endif
C
      Call Agrif_Prtbounds(nbdim,indmin,indmax,s_Parent_temp,
     &                     s_Child_temp,s_Child,ds_Child,
     &                     s_Parent,ds_Parent,
     &                     pttruetab,cetruetab,pttab_Child,
     &                     pttab_Parent,parentglob)      
C
C     Values on the parent grid
      allocate(tempP(indmin(1):indmax(1),indmin(2):indmax(2)))
C
C     Values on the child grid
      Allocate(tempC(pttruetab(1):cetruetab(1),
     &               pttruetab(2):cetruetab(2)))
C      
      tempC = 0.
C
C
#ifdef AGRIF_MPI
C
C
C     For a global index on the current grid, childarray gives the corresponding 
C     local index on the current processor
C
      Allocate(childarray(minval(pttruetab):maxval(cetruetab),nbdim,2))
C
      Call Agrif_GlobtoLocInd(childarray,lbound(child%var%array2),
     &                        ubound(child%var%array2),pttruetab,
     &                        cetruetab,nbdim,Agrif_Procrank)      
C   
C
      tempC(pttruetab(1):cetruetab(1),
     &      pttruetab(2):cetruetab(2)) = 
     &child % var % array2(childarray(pttruetab(1),1,2):
     &                     childarray(cetruetab(1),1,2),
     &                     childarray(pttruetab(2),2,2):
     &                     childarray(cetruetab(2),2,2))
C
#else
C
      tempC(pttruetab(1):cetruetab(1),
     &      pttruetab(2):cetruetab(2)) = 
     &child % var % array2(pttruetab(1):cetruetab(1),
     &                     pttruetab(2):cetruetab(2))        
C
#endif           
C
C     Special values on the child grid
      If (Agrif_UseSpecialValueFineGrid) Then
C
          noraftab(1:nbdim) = 
     &    child % var % root_var % interptab(1:nbdim) .EQ. 'noint'
C
#ifdef AGRIF_MPI
C
          Allocate(childvalues(pttruetab(1):cetruetab(1),
     &                         pttruetab(2):cetruetab(2)))
C
          childvalues = tempC
C
          Call Agrif_CheckMask2D(tempC,childvalues,
     &                           pttruetab(1:nbdim),cetruetab(1:nbdim),
     &                           pttruetab(1:nbdim),cetruetab(1:nbdim),
     &                           noraftab(1:nbdim))
C
          Deallocate(childvalues)
C
#else
C
          Call Agrif_CheckMask2D(tempC,child%var%array2,
     &                           pttruetab(1:nbdim),cetruetab(1:nbdim),
     &                           lbound(child%var%array2),
     &                           ubound(child%var%array2),
     &                           noraftab(1:nbdim))
C
#endif   	  
C
      End If
C
C
C     Update of the parent grid (tempP) from the child grid (tempC)
      Call Agrif_Update_2D_recursive(TypeUpdate,tempP,tempC,
     &           indmin,indmax,
     &           pttruetab,cetruetab,
     &           s_Child_temp,s_Parent_temp,
     &           ds_Child,ds_Parent,nbdim)
C
C
#ifdef AGRIF_MPI
C
C     For a global index on the parent grid, parentarray gives the corresponding 
C     local index on the current processor
C
      Allocate(parentarray(minval(parentglob(:,1)):
     &                     maxval(parentglob(:,2)),nbdim,2))
C
      Call Agrif_ChildGrid_to_ParentGrid()
C	
      Call Agrif_GlobtoLocInd(parentarray,lbound(parent%var%array2),
     &                        ubound(parent%var%array2),
     &                        parentglob(:,1),parentglob(:,2),
     &                        nbdim,Agrif_Procrank)      
C
      Call Agrif_ParentGrid_to_ChildGrid()  
C
#endif    
C
C
C     Special values on the parent grid
      If (Agrif_UseSpecialValue) Then
C
#ifdef AGRIF_MPI
C
          Where (parent % var % array2(
     &           parentarray(indmin(1),1,2):parentarray(indmax(1),1,2),
     &           parentarray(indmin(2),2,2):parentarray(indmax(2),2,2)) 
     &            == Agrif_SpecialValue)
C
#else
C	   
          Where (parent % var % array2(indmin(1):indmax(1),
     &                                 indmin(2):indmax(2)) 
     &            == Agrif_SpecialValue) 
C
#endif	
C   
          tempP(indmin(1):indmax(1),indmin(2):indmax(2))
     &      = Agrif_SpecialValue
C	      
          End Where
C
      End If	   
C
#ifdef AGRIF_MPI   
C 
      tab1(:,1) = indmin(:)
      tab1(:,2) = indmax(:) 
C
      Call MPI_BARRIER(MPI_COMM_WORLD,code)
C
C     tab2 contains boundaries of the parent grid on each processor
      Allocate(tab2(nbdim,2,0:Agrif_Nbprocs-1))
C     
      Call MPI_ALLGATHER(parentglob,2*nbdim,MPI_INTEGER,tab2,2*nbdim,
     &                   MPI_INTEGER,MPI_COMM_WORLD,code)                  
C      
C
C     Sentoproc indicates if some values of tempP on the current processor 
C     are sent to the others processors
C     Imin, imax, iminloc, imaxloc are the necessary boundaries of tempP
      Call Agrif_Sendtoproc2(Agrif_Nbprocs,Agrif_Nbprocs,sendtoproc,
     &                      nbdim,imin,imax,iminloc,imaxloc,tab1,
     &                      tab2,parentarray,lbound(parentarray,1),
     &                      ubound(parentarray,1))
C
C
      Do k = 0,Agrif_Nbprocs-1        
C
C       Copy of the values of tempP on the parent grid of the current grid
        If (k == Agrif_ProcRank) Then
C
            parent%var%array2(iminloc(1,k):imaxloc(1,k),
     &                        iminloc(2,k):imaxloc(2,k)) = 
     &      tempP(imin(1,k):imax(1,k),imin(2,k):imax(2,k))
C
          Else        
C
C           Parts of tempP on the current processor are sent to the others processors
            Call MPI_SEND(sendtoproc(k),1,MPI_LOGICAL,k,etiquette,
     &                    MPI_COMM_WORLD,code)
C
            If (sendtoproc(k) == .true.) Then
C
                Call MPI_SEND(imin(:,k),
     &                        nbdim,MPI_INTEGER,k,etiquette,
     &                        MPI_COMM_WORLD,code)
C
                Call MPI_SEND(imax(:,k),
     &                        nbdim,MPI_INTEGER,k,etiquette,
     &                        MPI_COMM_WORLD,code)
C
                size = 1
C
                Do i = 1,nbdim
C
                  size = size * (imax(i,k)-imin(i,k)+1)
C
                End Do
C
                Call MPI_SEND(tempP(
     &                        imin(1,k):imax(1,k),
     &                        imin(2,k):imax(2,k)),
     &                        size,MPI_DOUBLE_PRECISION,k,etiquette,
     &                        MPI_COMM_WORLD,code)  
C
            End If
C
        End If
C
      End Do  
C
C
      Do k = 0,Agrif_Nbprocs-1        
C
        If (k /= Agrif_ProcRank) Then
C
            Call MPI_RECV(res,1,MPI_LOGICAL,k,etiquette,
     &                    MPI_COMM_WORLD,statut,code) 
C
            recvfromproc(k) = res       
C
C           Current processor receives parts of tempP from others processors
            If (recvfromproc(k) == .true.) Then
C
                Call MPI_RECV(imin_recv(:,k),
     &                        nbdim,MPI_INTEGER,k,etiquette,
     &                        MPI_COMM_WORLD,statut,code)
C
                Call MPI_RECV(imax_recv(:,k),
     &                        nbdim,MPI_INTEGER,k,etiquette,
     &                        MPI_COMM_WORLD,statut,code)
C
                size = 1
C
                Do i = 1,nbdim
C
                  size = size * (imax_recv(i,k)-imin_recv(i,k)+1)
C
                End Do
C
                Call MPI_RECV(parent%var%array2(
     &                        parentarray(imin_recv(1,k),1,2):
     &                        parentarray(imax_recv(1,k),1,2),
     &                        parentarray(imin_recv(2,k),2,2):
     &                        parentarray(imax_recv(2,k),2,2)),
     &                        size,MPI_DOUBLE_PRECISION,k,etiquette,
     &                        MPI_COMM_WORLD,statut,code)
C
            End If 
C
        End If
C
      End Do 
C      
#else
C
      parent % var % array2(indmin(1):indmax(1),
     &                      indmin(2):indmax(2)) 
     &= tempP(indmin(1):indmax(1),
     &        indmin(2):indmax(2)) 
C
#endif               
C		  	      	    	      		   				  	       	  	                         		  	            
C     Deallocations      
      deallocate(tempP)
      deallocate(tempC)
C
#ifdef AGRIF_MPI
C
      deallocate(parentarray,childarray) 
C
#endif      
C       
C		  
      End Subroutine Agrif_Update2D
C
C
C
C     **************************************************************************  
CCC   Subroutine Agrif_Update3D  
C     **************************************************************************  
C  
      Subroutine Agrif_Update3D(TypeUpdate,parent,child,
     &                          pttab_Child,petab_child,
     &                          pttab_Parent,petab_Parent,
     &                          s_Child,s_Parent,ds_Child,ds_Parent,
     &                          nbdim)  
C
C     Description:
C     Subroutine to update a 3D grid variable on the parent grid of the current grid. 
C
C     Declarations:
C
      Implicit none
C
#ifdef AGRIF_MPI
C
#include "mpif.h"
C
#endif
C
C     Arguments
      Integer                    :: nbdim
      Integer                    :: TypeUpdate         ! Type of interpolation (linear,...)
      Type(AGRIF_PVARIABLE)      :: parent             ! Variable of the parent grid   
      Type(AGRIF_PVARIABLE)      :: child              ! Variable of the child grid
      Integer,Dimension(nbdim)   :: pttab_Child        ! Index of the first point inside the domain for the child grid variable 
      Integer,Dimension(nbdim)   :: petab_Child              
      Integer,Dimension(nbdim)   :: pttab_Parent       ! Index of the first point inside the domain for the parent grid variable
      Integer,Dimension(nbdim)   :: petab_Parent 
      Real,Dimension(nbdim)      :: s_Child,s_Parent   ! Positions of the parent and child grids
      Real,Dimension(nbdim)      :: ds_Child,ds_Parent ! Space steps of the parent and child grids                         
C
C     Local pointers
      Real, Dimension(:,:,:), Allocatable  :: tempP      ! Temporary parent grid variable	           							    
      Real, Dimension(:,:,:), Allocatable  :: tempC      ! Temporary child grid variable      							             
C
C     Local scalars        
      Integer                     :: i,lbglob,ubglob  
      Integer,Dimension(nbdim)    :: pttruetab,cetruetab
      Logical,Dimension(nbdim)    :: noraftab
      Integer,Dimension(nbdim)    :: indmin,indmax
      Real, Dimension(nbdim)      :: dim_newmin,dim_newmax
      Integer, Dimension(nbdim)   :: ind_gmin,ind_gmax  
      Real,Dimension(nbdim)       :: s_Child_temp,s_Parent_temp
      Integer,Dimension(nbdim,2)  :: parentglob
C
#ifdef AGRIF_MPI
C
      Integer                 :: code
      Integer,Dimension(nbdim,2) :: tab1
      Integer,Dimension(:,:,:),Allocatable :: tab2
      Integer,Dimension(nbdim,0:Agrif_Nbprocs-1) :: imin,imax
      Integer,Dimension(nbdim,0:Agrif_Nbprocs-1) :: iminloc,imaxloc
      Integer,Dimension(nbdim,0:Agrif_Nbprocs-1) :: imin_recv,imax_recv
      Logical,Dimension(0:Agrif_Nbprocs-1):: sendtoproc
      Logical,Dimension(0:Agrif_Nbprocs-1) :: recvfromproc
      Integer,Dimension(:,:,:),Allocatable :: parentarray
      Integer,Dimension(:,:,:),Allocatable :: childarray
      Integer                 :: j,k,size,pglob
      Logical                 :: res
      Integer,Parameter       :: etiquette = 100
      Integer,Dimension(MPI_STATUS_SIZE) :: statut  
      Real,Dimension(:,:,:),Allocatable  :: childvalues
C
#endif      
C
C
C     Calculation of the boundaries of the current grid necessary for the update of
C     the parent grid
      Call Agrif_Childbounds(nbdim,lbound(child%var%array3),
     &                       ubound(child%var%array3),pttab_child,
     &                       petab_child,pttruetab,cetruetab) 
C
C     Calculation of the boundaries of the parent grid
      parentglob(:,1) = lbound(parent % var % array3)
      parentglob(:,2) = ubound(parent % var % array3)
C
#ifdef AGRIF_MPI 
C
      Call Agrif_ChildGrid_to_ParentGrid()
C
      Do j = 1,2
C
        Do i = 1,nbdim
C  
          pglob = parentglob(i,j)
C
          Call Agrif_Invloc(pglob,Agrif_ProcRank,i,parentglob(i,j))
C
        End Do
C
      End Do
C
      Call Agrif_ParentGrid_to_ChildGrid()            
C
#endif
C
      Call Agrif_Prtbounds(nbdim,indmin,indmax,s_Parent_temp,
     &                     s_Child_temp,s_Child,ds_Child,
     &                     s_Parent,ds_Parent,
     &                     pttruetab,cetruetab,pttab_Child,
     &                     pttab_Parent,parentglob)      
C
C     Values on the parent grid
      allocate(tempP(indmin(1):indmax(1),indmin(2):indmax(2),
     &               indmin(3):indmax(3)))
C
C     Values on the child grid
      Allocate(tempC(pttruetab(1):cetruetab(1),
     &               pttruetab(2):cetruetab(2),
     &               pttruetab(3):cetruetab(3)))
C      
      tempC = 0.
C
C
#ifdef AGRIF_MPI
C
C
C     For a global index on the current grid, childarray gives the corresponding 
C     local index on the current processor
C
      Allocate(childarray(minval(pttruetab):maxval(cetruetab),nbdim,2))
C
      Call Agrif_GlobtoLocInd(childarray,lbound(child%var%array3),
     &                        ubound(child%var%array3),pttruetab,
     &                        cetruetab,nbdim,Agrif_Procrank)      
C   
C
      tempC(pttruetab(1):cetruetab(1),
     &      pttruetab(2):cetruetab(2),
     &      pttruetab(3):cetruetab(3)) = 
     &child % var % array3(childarray(pttruetab(1),1,2):
     &                     childarray(cetruetab(1),1,2),
     &                     childarray(pttruetab(2),2,2):
     &                     childarray(cetruetab(2),2,2),
     &                     childarray(pttruetab(3),3,2):
     &                     childarray(cetruetab(3),3,2))
C
#else
C
      tempC(pttruetab(1):cetruetab(1),
     &      pttruetab(2):cetruetab(2),
     &      pttruetab(3):cetruetab(3)) = 
     &child % var % array3(pttruetab(1):cetruetab(1),
     &                     pttruetab(2):cetruetab(2),
     &                     pttruetab(3):cetruetab(3))        
C
#endif           
C
C     Special values on the child grid
      If (Agrif_UseSpecialValueFineGrid) Then
C
          noraftab(1:nbdim) = 
     &    child % var % root_var % interptab(1:nbdim) .EQ. 'noint'
C
#ifdef AGRIF_MPI
C
          Allocate(childvalues(pttruetab(1):cetruetab(1),
     &                         pttruetab(2):cetruetab(2),
     &                         pttruetab(3):cetruetab(3)))
C
          childvalues = tempC
C
          Call Agrif_CheckMask3D(tempC,childvalues,
     &                           pttruetab(1:nbdim),cetruetab(1:nbdim),
     &                           pttruetab(1:nbdim),cetruetab(1:nbdim),
     &                           noraftab(1:nbdim))
C
          Deallocate(childvalues)
C
#else
C
          Call Agrif_CheckMask3D(tempC,child%var%array3,
     &                           pttruetab(1:nbdim),cetruetab(1:nbdim),
     &                           lbound(child%var%array3),
     &                           ubound(child%var%array3),
     &                           noraftab(1:nbdim))
C
#endif   	  
C
      End If
C
C
C     Update of the parent grid (tempP) from the child grid (tempC)
      Call Agrif_Update_3D_recursive(TypeUpdate,tempP,tempC,
     &           indmin,indmax,
     &           pttruetab,cetruetab,
     &           s_Child_temp,s_Parent_temp,
     &           ds_Child,ds_Parent,nbdim)
C
C
#ifdef AGRIF_MPI
C
C     For a global index on the parent grid, parentarray gives the corresponding 
C     local index on the current processor
C
      Allocate(parentarray(minval(parentglob(:,1)):
     &                     maxval(parentglob(:,2)),nbdim,2))
C
      Call Agrif_ChildGrid_to_ParentGrid()
C	
      Call Agrif_GlobtoLocInd(parentarray,lbound(parent%var%array3),
     &                        ubound(parent%var%array3),
     &                        parentglob(:,1),parentglob(:,2),
     &                        nbdim,Agrif_Procrank)      
C
      Call Agrif_ParentGrid_to_ChildGrid()  
C
#endif    
C
C
C     Special values on the parent grid
      If (Agrif_UseSpecialValue) Then
C
#ifdef AGRIF_MPI
C
          Where (parent % var % array3(
     &           parentarray(indmin(1),1,2):parentarray(indmax(1),1,2),
     &           parentarray(indmin(2),2,2):parentarray(indmax(2),2,2),
     &           parentarray(indmin(3),3,2):parentarray(indmax(3),3,2)) 
     &            == Agrif_SpecialValue)
C
#else
C	   
          Where (parent % var % array3(indmin(1):indmax(1),
     &                                 indmin(2):indmax(2),
     &                                 indmin(3):indmax(3)) 
     &            == Agrif_SpecialValue) 
C
#endif	
C   
          tempP(indmin(1):indmax(1),indmin(2):indmax(2),
     &            indmin(3):indmax(3))
     &      = Agrif_SpecialValue
C	      
          End Where
C
      End If	   
C
#ifdef AGRIF_MPI   
C 
      tab1(:,1) = indmin(:)
      tab1(:,2) = indmax(:) 
C
      Call MPI_BARRIER(MPI_COMM_WORLD,code)
C
C     tab2 contains boundaries of the parent grid on each processor
      Allocate(tab2(nbdim,2,0:Agrif_Nbprocs-1))
C     
      Call MPI_ALLGATHER(parentglob,2*nbdim,MPI_INTEGER,tab2,2*nbdim,
     &                   MPI_INTEGER,MPI_COMM_WORLD,code)                  
C      
C
C     Sentoproc indicates if some values of tempP on the current processor 
C     are sent to the others processors
C     Imin, imax, iminloc, imaxloc are the necessary boundaries of tempP
      Call Agrif_Sendtoproc2(Agrif_Nbprocs,Agrif_Nbprocs,sendtoproc,
     &                      nbdim,imin,imax,iminloc,imaxloc,tab1,
     &                      tab2,parentarray,lbound(parentarray,1),
     &                      ubound(parentarray,1))
C
C
      Do k = 0,Agrif_Nbprocs-1        
C
C       Copy of the values of tempP on the parent grid of the current grid
        If (k == Agrif_ProcRank) Then
C
            parent%var%array3(iminloc(1,k):imaxloc(1,k),
     &                        iminloc(2,k):imaxloc(2,k),
     &                        iminloc(3,k):imaxloc(3,k)) = 
     &      tempP(imin(1,k):imax(1,k),imin(2,k):imax(2,k),
     &            imin(3,k):imax(3,k))
C
          Else        
C
C           Parts of tempP on the current processor are sent to the others processors
            Call MPI_SEND(sendtoproc(k),1,MPI_LOGICAL,k,etiquette,
     &                    MPI_COMM_WORLD,code)
C
            If (sendtoproc(k) == .true.) Then
C
                Call MPI_SEND(imin(:,k),
     &                        nbdim,MPI_INTEGER,k,etiquette,
     &                        MPI_COMM_WORLD,code)
C
                Call MPI_SEND(imax(:,k),
     &                        nbdim,MPI_INTEGER,k,etiquette,
     &                        MPI_COMM_WORLD,code)
C
                size = 1
C
                Do i = 1,nbdim
C
                  size = size * (imax(i,k)-imin(i,k)+1)
C
                End Do
C
                Call MPI_SEND(tempP(
     &                        imin(1,k):imax(1,k),
     &                        imin(2,k):imax(2,k),
     &                        imin(3,k):imax(3,k)),
     &                        size,MPI_DOUBLE_PRECISION,k,etiquette,
     &                        MPI_COMM_WORLD,code)  
C
            End If
C
        End If
C
      End Do  
C
C
      Do k = 0,Agrif_Nbprocs-1        
C
        If (k /= Agrif_ProcRank) Then
C
            Call MPI_RECV(res,1,MPI_LOGICAL,k,etiquette,
     &                    MPI_COMM_WORLD,statut,code) 
C
            recvfromproc(k) = res       
C
C           Current processor receives parts of tempP from others processors
            If (recvfromproc(k) == .true.) Then
C
                Call MPI_RECV(imin_recv(:,k),
     &                        nbdim,MPI_INTEGER,k,etiquette,
     &                        MPI_COMM_WORLD,statut,code)
C
                Call MPI_RECV(imax_recv(:,k),
     &                        nbdim,MPI_INTEGER,k,etiquette,
     &                        MPI_COMM_WORLD,statut,code)
C
                size = 1
C
                Do i = 1,nbdim
C
                  size = size * (imax_recv(i,k)-imin_recv(i,k)+1)
C
                End Do
C
                Call MPI_RECV(parent%var%array3(
     &                        parentarray(imin_recv(1,k),1,2):
     &                        parentarray(imax_recv(1,k),1,2),
     &                        parentarray(imin_recv(2,k),2,2):
     &                        parentarray(imax_recv(2,k),2,2),
     &                        parentarray(imin_recv(3,k),3,2):
     &                        parentarray(imax_recv(3,k),3,2)),
     &                        size,MPI_DOUBLE_PRECISION,k,etiquette,
     &                        MPI_COMM_WORLD,statut,code)
C
            End If 
C
        End If
C
      End Do 
C      
#else
C
      parent % var % array3(indmin(1):indmax(1),
     &                      indmin(2):indmax(2),
     &                      indmin(3):indmax(3)) 
     &= tempP(indmin(1):indmax(1),
     &        indmin(2):indmax(2),
     &        indmin(3):indmax(3)) 
C
#endif               
C		  	      	    	      		   				  	       	  	                         		  	            
C     Deallocations      
      deallocate(tempP)
      deallocate(tempC)
C
#ifdef AGRIF_MPI
C
      deallocate(parentarray,childarray) 
C
#endif      
C       
C		  
      End Subroutine Agrif_Update3D
C
C
C
C     **************************************************************************  
CCC   Subroutine Agrif_Update4D  
C     **************************************************************************  
C  
      Subroutine Agrif_Update4D(TypeUpdate,parent,child,
     &                          pttab_Child,petab_child,
     &                          pttab_Parent,petab_Parent,
     &                          s_Child,s_Parent,ds_Child,ds_Parent,
     &                          nbdim)  
C
C     Description:
C     Subroutine to update a 4D grid variable on the parent grid of the current grid. 
C
C     Declarations:
C
      Implicit none
C
#ifdef AGRIF_MPI
C
#include "mpif.h"
C
#endif
C
C     Arguments
      Integer                    :: nbdim
      Integer                    :: TypeUpdate         ! Type of interpolation (linear,...)
      Type(AGRIF_PVARIABLE)      :: parent             ! Variable of the parent grid   
      Type(AGRIF_PVARIABLE)      :: child              ! Variable of the child grid
      Integer,Dimension(nbdim)   :: pttab_Child        ! Index of the first point inside the domain for the child grid variable 
      Integer,Dimension(nbdim)   :: petab_Child              
      Integer,Dimension(nbdim)   :: pttab_Parent       ! Index of the first point inside the domain for the parent grid variable
      Integer,Dimension(nbdim)   :: petab_Parent 
      Real,Dimension(nbdim)      :: s_Child,s_Parent   ! Positions of the parent and child grids
      Real,Dimension(nbdim)      :: ds_Child,ds_Parent ! Space steps of the parent and child grids                         
C
C     Local pointers
      Real, Dimension(:,:,:,:), Allocatable  :: tempP      ! Temporary parent grid variable	           							    
      Real, Dimension(:,:,:,:), Allocatable  :: tempC      ! Temporary child grid variable      							             
C
C     Local scalars        
      Integer                     :: i,lbglob,ubglob  
      Integer,Dimension(nbdim)    :: pttruetab,cetruetab
      Logical,Dimension(nbdim)    :: noraftab
      Integer,Dimension(nbdim)    :: indmin,indmax
      Real, Dimension(nbdim)      :: dim_newmin,dim_newmax
      Integer, Dimension(nbdim)   :: ind_gmin,ind_gmax  
      Real,Dimension(nbdim)       :: s_Child_temp,s_Parent_temp
      Integer,Dimension(nbdim,2)  :: parentglob
C
#ifdef AGRIF_MPI
C
      Integer                 :: code
      Integer,Dimension(nbdim,2) :: tab1
      Integer,Dimension(:,:,:),Allocatable :: tab2
      Integer,Dimension(nbdim,0:Agrif_Nbprocs-1) :: imin,imax
      Integer,Dimension(nbdim,0:Agrif_Nbprocs-1) :: iminloc,imaxloc
      Integer,Dimension(nbdim,0:Agrif_Nbprocs-1) :: imin_recv,imax_recv
      Logical,Dimension(0:Agrif_Nbprocs-1):: sendtoproc
      Logical,Dimension(0:Agrif_Nbprocs-1) :: recvfromproc
      Integer,Dimension(:,:,:),Allocatable :: parentarray
      Integer,Dimension(:,:,:),Allocatable :: childarray
      Integer                 :: j,k,size,pglob
      Logical                 :: res
      Integer,Parameter       :: etiquette = 100
      Integer,Dimension(MPI_STATUS_SIZE) :: statut  
      Real,Dimension(:,:,:,:),Allocatable  :: childvalues
C
#endif      
C
C
C     Calculation of the boundaries of the current grid necessary for the update of
C     the parent grid
      Call Agrif_Childbounds(nbdim,lbound(child%var%array4),
     &                       ubound(child%var%array4),pttab_child,
     &                       petab_child,pttruetab,cetruetab) 
C
C     Calculation of the boundaries of the parent grid
      parentglob(:,1) = lbound(parent % var % array4)
      parentglob(:,2) = ubound(parent % var % array4)
C
#ifdef AGRIF_MPI 
C
      Call Agrif_ChildGrid_to_ParentGrid()
C
      Do j = 1,2
C
        Do i = 1,nbdim
C  
          pglob = parentglob(i,j)
C
          Call Agrif_Invloc(pglob,Agrif_ProcRank,i,parentglob(i,j))
C
        End Do
C
      End Do
C
      Call Agrif_ParentGrid_to_ChildGrid()            
C
#endif
C
      Call Agrif_Prtbounds(nbdim,indmin,indmax,s_Parent_temp,
     &                     s_Child_temp,s_Child,ds_Child,
     &                     s_Parent,ds_Parent,
     &                     pttruetab,cetruetab,pttab_Child,
     &                     pttab_Parent,parentglob)      
C
C     Values on the parent grid
      allocate(tempP(indmin(1):indmax(1),indmin(2):indmax(2),
     &               indmin(3):indmax(3),indmin(4):indmax(4)))
C
C     Values on the child grid
      Allocate(tempC(pttruetab(1):cetruetab(1),
     &               pttruetab(2):cetruetab(2),
     &               pttruetab(3):cetruetab(3),
     &               pttruetab(4):cetruetab(4)))
C      
      tempC = 0.
C
C
#ifdef AGRIF_MPI
C
C
C     For a global index on the current grid, childarray gives the corresponding 
C     local index on the current processor
C
      Allocate(childarray(minval(pttruetab):maxval(cetruetab),nbdim,2))
C
      Call Agrif_GlobtoLocInd(childarray,lbound(child%var%array4),
     &                        ubound(child%var%array4),pttruetab,
     &                        cetruetab,nbdim,Agrif_Procrank)      
C   
C
      tempC(pttruetab(1):cetruetab(1),
     &      pttruetab(2):cetruetab(2),
     &      pttruetab(3):cetruetab(3),
     &      pttruetab(4):cetruetab(4)) = 
     &child % var % array4(childarray(pttruetab(1),1,2):
     &                     childarray(cetruetab(1),1,2),
     &                     childarray(pttruetab(2),2,2):
     &                     childarray(cetruetab(2),2,2),
     &                     childarray(pttruetab(3),3,2):
     &                     childarray(cetruetab(3),3,2),
     &                     childarray(pttruetab(4),4,2):
     &                     childarray(cetruetab(4),4,2))
C
#else
C
      tempC(pttruetab(1):cetruetab(1),
     &      pttruetab(2):cetruetab(2),
     &      pttruetab(3):cetruetab(3),
     &      pttruetab(4):cetruetab(4)) = 
     &child % var % array4(pttruetab(1):cetruetab(1),
     &                     pttruetab(2):cetruetab(2),
     &                     pttruetab(3):cetruetab(3),
     &                     pttruetab(4):cetruetab(4))        
C
#endif           
C
C     Special values on the child grid
      If (Agrif_UseSpecialValueFineGrid) Then
C
          noraftab(1:nbdim) = 
     &    child % var % root_var % interptab(1:nbdim) .EQ. 'noint'
C
#ifdef AGRIF_MPI
C
          Allocate(childvalues(pttruetab(1):cetruetab(1),
     &                         pttruetab(2):cetruetab(2),
     &                         pttruetab(3):cetruetab(3),
     &                         pttruetab(4):cetruetab(4)))
C
          childvalues = tempC
C
          Call Agrif_CheckMask4D(tempC,childvalues,
     &                           pttruetab(1:nbdim),cetruetab(1:nbdim),
     &                           pttruetab(1:nbdim),cetruetab(1:nbdim),
     &                           noraftab(1:nbdim))
C
          Deallocate(childvalues)
C
#else
C
          Call Agrif_CheckMask4D(tempC,child%var%array4,
     &                           pttruetab(1:nbdim),cetruetab(1:nbdim),
     &                           lbound(child%var%array4),
     &                           ubound(child%var%array4),
     &                           noraftab(1:nbdim))
C
#endif   	  
C
      End If
C
C
C     Update of the parent grid (tempP) from the child grid (tempC)
      Call Agrif_Update_4D_recursive(TypeUpdate,tempP,tempC,
     &           indmin,indmax,
     &           pttruetab,cetruetab,
     &           s_Child_temp,s_Parent_temp,
     &           ds_Child,ds_Parent,nbdim)
C
C
#ifdef AGRIF_MPI
C
C     For a global index on the parent grid, parentarray gives the corresponding 
C     local index on the current processor
C
      Allocate(parentarray(minval(parentglob(:,1)):
     &                     maxval(parentglob(:,2)),nbdim,2))
C
      Call Agrif_ChildGrid_to_ParentGrid()
C	
      Call Agrif_GlobtoLocInd(parentarray,lbound(parent%var%array4),
     &                        ubound(parent%var%array4),
     &                        parentglob(:,1),parentglob(:,2),
     &                        nbdim,Agrif_Procrank)      
C
      Call Agrif_ParentGrid_to_ChildGrid()  
C
#endif    
C
C
C     Special values on the parent grid
      If (Agrif_UseSpecialValue) Then
C
#ifdef AGRIF_MPI
C
          Where (parent % var % array4(
     &           parentarray(indmin(1),1,2):parentarray(indmax(1),1,2),
     &           parentarray(indmin(2),2,2):parentarray(indmax(2),2,2),
     &           parentarray(indmin(3),3,2):parentarray(indmax(3),3,2),
     &           parentarray(indmin(4),4,2):parentarray(indmax(4),4,2)) 
     &            == Agrif_SpecialValue)
C
#else
C	   
          Where (parent % var % array4(indmin(1):indmax(1),
     &                                 indmin(2):indmax(2),
     &                                 indmin(3):indmax(3),
     &                                 indmin(4):indmax(4)) 
     &            == Agrif_SpecialValue) 
C
#endif	
C   
          tempP(indmin(1):indmax(1),indmin(2):indmax(2),
     &            indmin(3):indmax(3),indmin(4):indmax(4))
     &      = Agrif_SpecialValue
C	      
          End Where
C
      End If	   
C
#ifdef AGRIF_MPI   
C 
      tab1(:,1) = indmin(:)
      tab1(:,2) = indmax(:) 
C
      Call MPI_BARRIER(MPI_COMM_WORLD,code)
C
C     tab2 contains boundaries of the parent grid on each processor
      Allocate(tab2(nbdim,2,0:Agrif_Nbprocs-1))
C     
      Call MPI_ALLGATHER(parentglob,2*nbdim,MPI_INTEGER,tab2,2*nbdim,
     &                   MPI_INTEGER,MPI_COMM_WORLD,code)                  
C      
C
C     Sentoproc indicates if some values of tempP on the current processor 
C     are sent to the others processors
C     Imin, imax, iminloc, imaxloc are the necessary boundaries of tempP
      Call Agrif_Sendtoproc2(Agrif_Nbprocs,Agrif_Nbprocs,sendtoproc,
     &                      nbdim,imin,imax,iminloc,imaxloc,tab1,
     &                      tab2,parentarray,lbound(parentarray,1),
     &                      ubound(parentarray,1))
C
C
      Do k = 0,Agrif_Nbprocs-1        
C
C       Copy of the values of tempP on the parent grid of the current grid
        If (k == Agrif_ProcRank) Then
C
            parent%var%array4(iminloc(1,k):imaxloc(1,k),
     &                        iminloc(2,k):imaxloc(2,k),
     &                        iminloc(3,k):imaxloc(3,k),
     &                        iminloc(4,k):imaxloc(4,k)) = 
     &      tempP(imin(1,k):imax(1,k),imin(2,k):imax(2,k),
     &            imin(3,k):imax(3,k),imin(4,k):imax(4,k))
C
          Else        
C
C           Parts of tempP on the current processor are sent to the others processors
            Call MPI_SEND(sendtoproc(k),1,MPI_LOGICAL,k,etiquette,
     &                    MPI_COMM_WORLD,code)
C
            If (sendtoproc(k) == .true.) Then
C
                Call MPI_SEND(imin(:,k),
     &                        nbdim,MPI_INTEGER,k,etiquette,
     &                        MPI_COMM_WORLD,code)
C
                Call MPI_SEND(imax(:,k),
     &                        nbdim,MPI_INTEGER,k,etiquette,
     &                        MPI_COMM_WORLD,code)
C
                size = 1
C
                Do i = 1,nbdim
C
                  size = size * (imax(i,k)-imin(i,k)+1)
C
                End Do
C
                Call MPI_SEND(tempP(
     &                        imin(1,k):imax(1,k),
     &                        imin(2,k):imax(2,k),
     &                        imin(3,k):imax(3,k),
     &                        imin(4,k):imax(4,k)),
     &                        size,MPI_DOUBLE_PRECISION,k,etiquette,
     &                        MPI_COMM_WORLD,code)  
C
            End If
C
        End If
C
      End Do  
C
C
      Do k = 0,Agrif_Nbprocs-1        
C
        If (k /= Agrif_ProcRank) Then
C
            Call MPI_RECV(res,1,MPI_LOGICAL,k,etiquette,
     &                    MPI_COMM_WORLD,statut,code) 
C
            recvfromproc(k) = res       
C
C           Current processor receives parts of tempP from others processors
            If (recvfromproc(k) == .true.) Then
C
                Call MPI_RECV(imin_recv(:,k),
     &                        nbdim,MPI_INTEGER,k,etiquette,
     &                        MPI_COMM_WORLD,statut,code)
C
                Call MPI_RECV(imax_recv(:,k),
     &                        nbdim,MPI_INTEGER,k,etiquette,
     &                        MPI_COMM_WORLD,statut,code)
C
                size = 1
C
                Do i = 1,nbdim
C
                  size = size * (imax_recv(i,k)-imin_recv(i,k)+1)
C
                End Do
C
                Call MPI_RECV(parent%var%array4(
     &                        parentarray(imin_recv(1,k),1,2):
     &                        parentarray(imax_recv(1,k),1,2),
     &                        parentarray(imin_recv(2,k),2,2):
     &                        parentarray(imax_recv(2,k),2,2),
     &                        parentarray(imin_recv(3,k),3,2):
     &                        parentarray(imax_recv(3,k),3,2),
     &                        parentarray(imin_recv(4,k),4,2):
     &                        parentarray(imax_recv(4,k),4,2)),
     &                        size,MPI_DOUBLE_PRECISION,k,etiquette,
     &                        MPI_COMM_WORLD,statut,code)
C
            End If 
C
        End If
C
      End Do 
C      
#else
C
      parent % var % array4(indmin(1):indmax(1),
     &                      indmin(2):indmax(2),
     &                      indmin(3):indmax(3),
     &                      indmin(4):indmax(4)) 
     &= tempP(indmin(1):indmax(1),
     &        indmin(2):indmax(2),
     &        indmin(3):indmax(3),
     &        indmin(4):indmax(4)) 
C
#endif               
C		  	      	    	      		   				  	       	  	                         		  	            
C     Deallocations      
      deallocate(tempP)
      deallocate(tempC)
C
#ifdef AGRIF_MPI
C
      deallocate(parentarray,childarray) 
C
#endif      
C       
C		  
      End Subroutine Agrif_Update4D
C
C
C
C     **************************************************************************  
CCC   Subroutine Agrif_Update5D  
C     **************************************************************************  
C  
      Subroutine Agrif_Update5D(TypeUpdate,parent,child,
     &                          pttab_Child,petab_child,
     &                          pttab_Parent,petab_Parent,
     &                          s_Child,s_Parent,ds_Child,ds_Parent,
     &                          nbdim)  
C
C     Description:
C     Subroutine to update a 5D grid variable on the parent grid of the current grid. 
C
C     Declarations:
C
      Implicit none
C
#ifdef AGRIF_MPI
C
#include "mpif.h"
C
#endif
C
C     Arguments
      Integer                    :: nbdim
      Integer                    :: TypeUpdate         ! Type of interpolation (linear,...)
      Type(AGRIF_PVARIABLE)      :: parent             ! Variable of the parent grid   
      Type(AGRIF_PVARIABLE)      :: child              ! Variable of the child grid
      Integer,Dimension(nbdim)   :: pttab_Child        ! Index of the first point inside the domain for the child grid variable 
      Integer,Dimension(nbdim)   :: petab_Child              
      Integer,Dimension(nbdim)   :: pttab_Parent       ! Index of the first point inside the domain for the parent grid variable
      Integer,Dimension(nbdim)   :: petab_Parent 
      Real,Dimension(nbdim)      :: s_Child,s_Parent   ! Positions of the parent and child grids
      Real,Dimension(nbdim)      :: ds_Child,ds_Parent ! Space steps of the parent and child grids                         
C
C     Local pointers
      Real, Dimension(:,:,:,:,:), Allocatable  :: tempP      ! Temporary parent grid variable	           							    
      Real, Dimension(:,:,:,:,:), Allocatable  :: tempC      ! Temporary child grid variable      							             
C
C     Local scalars        
      Integer                     :: i,lbglob,ubglob  
      Integer,Dimension(nbdim)    :: pttruetab,cetruetab
      Logical,Dimension(nbdim)    :: noraftab
      Integer,Dimension(nbdim)    :: indmin,indmax
      Real, Dimension(nbdim)      :: dim_newmin,dim_newmax
      Integer, Dimension(nbdim)   :: ind_gmin,ind_gmax  
      Real,Dimension(nbdim)       :: s_Child_temp,s_Parent_temp
      Integer,Dimension(nbdim,2)  :: parentglob
C
#ifdef AGRIF_MPI
C
      Integer                 :: code
      Integer,Dimension(nbdim,2) :: tab1
      Integer,Dimension(:,:,:),Allocatable :: tab2
      Integer,Dimension(nbdim,0:Agrif_Nbprocs-1) :: imin,imax
      Integer,Dimension(nbdim,0:Agrif_Nbprocs-1) :: iminloc,imaxloc
      Integer,Dimension(nbdim,0:Agrif_Nbprocs-1) :: imin_recv,imax_recv
      Logical,Dimension(0:Agrif_Nbprocs-1):: sendtoproc
      Logical,Dimension(0:Agrif_Nbprocs-1) :: recvfromproc
      Integer,Dimension(:,:,:),Allocatable :: parentarray
      Integer,Dimension(:,:,:),Allocatable :: childarray
      Integer                 :: j,k,size,pglob
      Logical                 :: res
      Integer,Parameter       :: etiquette = 100
      Integer,Dimension(MPI_STATUS_SIZE) :: statut  
      Real,Dimension(:,:,:,:,:),Allocatable  :: childvalues
C
#endif      
C
C
C     Calculation of the boundaries of the current grid necessary for the update of
C     the parent grid
      Call Agrif_Childbounds(nbdim,lbound(child%var%array5),
     &                       ubound(child%var%array5),pttab_child,
     &                       petab_child,pttruetab,cetruetab) 
C
C     Calculation of the boundaries of the parent grid
      parentglob(:,1) = lbound(parent % var % array5)
      parentglob(:,2) = ubound(parent % var % array5)
C
#ifdef AGRIF_MPI 
C
      Call Agrif_ChildGrid_to_ParentGrid()
C
      Do j = 1,2
C
        Do i = 1,nbdim
C  
          pglob = parentglob(i,j)
C
          Call Agrif_Invloc(pglob,Agrif_ProcRank,i,parentglob(i,j))
C
        End Do
C
      End Do
C
      Call Agrif_ParentGrid_to_ChildGrid()            
C
#endif
C
      Call Agrif_Prtbounds(nbdim,indmin,indmax,s_Parent_temp,
     &                     s_Child_temp,s_Child,ds_Child,
     &                     s_Parent,ds_Parent,
     &                     pttruetab,cetruetab,pttab_Child,
     &                     pttab_Parent,parentglob)      
C
C     Values on the parent grid
      allocate(tempP(indmin(1):indmax(1),indmin(2):indmax(2),
     &               indmin(3):indmax(3),indmin(4):indmax(4),
     &               indmin(5):indmax(5)))
C
C     Values on the child grid
      Allocate(tempC(pttruetab(1):cetruetab(1),
     &               pttruetab(2):cetruetab(2),
     &               pttruetab(3):cetruetab(3),
     &               pttruetab(4):cetruetab(4),
     &               pttruetab(5):cetruetab(5)))
C      
      tempC = 0.
C
C
#ifdef AGRIF_MPI
C
C
C     For a global index on the current grid, childarray gives the corresponding 
C     local index on the current processor
C
      Allocate(childarray(minval(pttruetab):maxval(cetruetab),nbdim,2))
C
      Call Agrif_GlobtoLocInd(childarray,lbound(child%var%array5),
     &                        ubound(child%var%array5),pttruetab,
     &                        cetruetab,nbdim,Agrif_Procrank)      
C   
C
      tempC(pttruetab(1):cetruetab(1),
     &      pttruetab(2):cetruetab(2),
     &      pttruetab(3):cetruetab(3),
     &      pttruetab(4):cetruetab(4),
     &      pttruetab(5):cetruetab(5)) = 
     &child % var % array5(childarray(pttruetab(1),1,2):
     &                     childarray(cetruetab(1),1,2),
     &                     childarray(pttruetab(2),2,2):
     &                     childarray(cetruetab(2),2,2),
     &                     childarray(pttruetab(3),3,2):
     &                     childarray(cetruetab(3),3,2),
     &                     childarray(pttruetab(4),4,2):
     &                     childarray(cetruetab(4),4,2),
     &                     childarray(pttruetab(5),5,2):
     &                     childarray(cetruetab(5),5,2))
C
#else
C
      tempC(pttruetab(1):cetruetab(1),
     &      pttruetab(2):cetruetab(2),
     &      pttruetab(3):cetruetab(3),
     &      pttruetab(4):cetruetab(4),
     &      pttruetab(5):cetruetab(5)) = 
     &child % var % array5(pttruetab(1):cetruetab(1),
     &                     pttruetab(2):cetruetab(2),
     &                     pttruetab(3):cetruetab(3),
     &                     pttruetab(4):cetruetab(4),
     &                     pttruetab(5):cetruetab(5))        
C
#endif           
C
C     Special values on the child grid
      If (Agrif_UseSpecialValueFineGrid) Then
C
          noraftab(1:nbdim) = 
     &    child % var % root_var % interptab(1:nbdim) .EQ. 'noint'
C
#ifdef AGRIF_MPI
C
          Allocate(childvalues(pttruetab(1):cetruetab(1),
     &                         pttruetab(2):cetruetab(2),
     &                         pttruetab(3):cetruetab(3),
     &                         pttruetab(4):cetruetab(4),
     &                         pttruetab(5):cetruetab(5)))
C
          childvalues = tempC
C
          Call Agrif_CheckMask5D(tempC,childvalues,
     &                           pttruetab(1:nbdim),cetruetab(1:nbdim),
     &                           pttruetab(1:nbdim),cetruetab(1:nbdim),
     &                           noraftab(1:nbdim))
C
          Deallocate(childvalues)
C
#else
C
          Call Agrif_CheckMask5D(tempC,child%var%array5,
     &                           pttruetab(1:nbdim),cetruetab(1:nbdim),
     &                           lbound(child%var%array5),
     &                           ubound(child%var%array5),
     &                           noraftab(1:nbdim))
C
#endif   	  
C
      End If
C
C
C     Update of the parent grid (tempP) from the child grid (tempC)
      Call Agrif_Update_5D_recursive(TypeUpdate,tempP,tempC,
     &           indmin,indmax,
     &           pttruetab,cetruetab,
     &           s_Child_temp,s_Parent_temp,
     &           ds_Child,ds_Parent,nbdim)
C
C
#ifdef AGRIF_MPI
C
C     For a global index on the parent grid, parentarray gives the corresponding 
C     local index on the current processor
C
      Allocate(parentarray(minval(parentglob(:,1)):
     &                     maxval(parentglob(:,2)),nbdim,2))
C
      Call Agrif_ChildGrid_to_ParentGrid()
C	
      Call Agrif_GlobtoLocInd(parentarray,lbound(parent%var%array5),
     &                        ubound(parent%var%array5),
     &                        parentglob(:,1),parentglob(:,2),
     &                        nbdim,Agrif_Procrank)      
C
      Call Agrif_ParentGrid_to_ChildGrid()  
C
#endif    
C
C
C     Special values on the parent grid
      If (Agrif_UseSpecialValue) Then
C
#ifdef AGRIF_MPI
C
          Where (parent % var % array5(
     &           parentarray(indmin(1),1,2):parentarray(indmax(1),1,2),
     &           parentarray(indmin(2),2,2):parentarray(indmax(2),2,2),
     &           parentarray(indmin(3),3,2):parentarray(indmax(3),3,2),
     &           parentarray(indmin(4),4,2):parentarray(indmax(4),4,2),
     &           parentarray(indmin(5),5,2):parentarray(indmax(5),5,2)) 
     &            == Agrif_SpecialValue)
C
#else
C	   
          Where (parent % var % array5(indmin(1):indmax(1),
     &                                 indmin(2):indmax(2),
     &                                 indmin(3):indmax(3),
     &                                 indmin(4):indmax(4),
     &                                 indmin(5):indmax(5)) 
     &            == Agrif_SpecialValue) 
C
#endif	
C   
          tempP(indmin(1):indmax(1),indmin(2):indmax(2),
     &            indmin(3):indmax(3),indmin(4):indmax(4),
     &            indmin(5):indmax(5))
     &      = Agrif_SpecialValue
C	      
          End Where
C
      End If	   
C
#ifdef AGRIF_MPI   
C 
      tab1(:,1) = indmin(:)
      tab1(:,2) = indmax(:) 
C
      Call MPI_BARRIER(MPI_COMM_WORLD,code)
C
C     tab2 contains boundaries of the parent grid on each processor
      Allocate(tab2(nbdim,2,0:Agrif_Nbprocs-1))
C     
      Call MPI_ALLGATHER(parentglob,2*nbdim,MPI_INTEGER,tab2,2*nbdim,
     &                   MPI_INTEGER,MPI_COMM_WORLD,code)                  
C      
C
C     Sentoproc indicates if some values of tempP on the current processor 
C     are sent to the others processors
C     Imin, imax, iminloc, imaxloc are the necessary boundaries of tempP
      Call Agrif_Sendtoproc2(Agrif_Nbprocs,Agrif_Nbprocs,sendtoproc,
     &                      nbdim,imin,imax,iminloc,imaxloc,tab1,
     &                      tab2,parentarray,lbound(parentarray,1),
     &                      ubound(parentarray,1))
C
C
      Do k = 0,Agrif_Nbprocs-1        
C
C       Copy of the values of tempP on the parent grid of the current grid
        If (k == Agrif_ProcRank) Then
C
            parent%var%array5(iminloc(1,k):imaxloc(1,k),
     &                        iminloc(2,k):imaxloc(2,k),
     &                        iminloc(3,k):imaxloc(3,k),
     &                        iminloc(4,k):imaxloc(4,k),
     &                        iminloc(5,k):imaxloc(5,k)) = 
     &      tempP(imin(1,k):imax(1,k),imin(2,k):imax(2,k),
     &            imin(3,k):imax(3,k),imin(4,k):imax(4,k),
     &            imin(5,k):imax(5,k))
C
          Else        
C
C           Parts of tempP on the current processor are sent to the others processors
            Call MPI_SEND(sendtoproc(k),1,MPI_LOGICAL,k,etiquette,
     &                    MPI_COMM_WORLD,code)
C
            If (sendtoproc(k) == .true.) Then
C
                Call MPI_SEND(imin(:,k),
     &                        nbdim,MPI_INTEGER,k,etiquette,
     &                        MPI_COMM_WORLD,code)
C
                Call MPI_SEND(imax(:,k),
     &                        nbdim,MPI_INTEGER,k,etiquette,
     &                        MPI_COMM_WORLD,code)
C
                size = 1
C
                Do i = 1,nbdim
C
                  size = size * (imax(i,k)-imin(i,k)+1)
C
                End Do
C
                Call MPI_SEND(tempP(
     &                        imin(1,k):imax(1,k),
     &                        imin(2,k):imax(2,k),
     &                        imin(3,k):imax(3,k),
     &                        imin(4,k):imax(4,k),
     &                        imin(5,k):imax(5,k)),
     &                        size,MPI_DOUBLE_PRECISION,k,etiquette,
     &                        MPI_COMM_WORLD,code)  
C
            End If
C
        End If
C
      End Do  
C
C
      Do k = 0,Agrif_Nbprocs-1        
C
        If (k /= Agrif_ProcRank) Then
C
            Call MPI_RECV(res,1,MPI_LOGICAL,k,etiquette,
     &                    MPI_COMM_WORLD,statut,code) 
C
            recvfromproc(k) = res       
C
C           Current processor receives parts of tempP from others processors
            If (recvfromproc(k) == .true.) Then
C
                Call MPI_RECV(imin_recv(:,k),
     &                        nbdim,MPI_INTEGER,k,etiquette,
     &                        MPI_COMM_WORLD,statut,code)
C
                Call MPI_RECV(imax_recv(:,k),
     &                        nbdim,MPI_INTEGER,k,etiquette,
     &                        MPI_COMM_WORLD,statut,code)
C
                size = 1
C
                Do i = 1,nbdim
C
                  size = size * (imax_recv(i,k)-imin_recv(i,k)+1)
C
                End Do
C
                Call MPI_RECV(parent%var%array5(
     &                        parentarray(imin_recv(1,k),1,2):
     &                        parentarray(imax_recv(1,k),1,2),
     &                        parentarray(imin_recv(2,k),2,2):
     &                        parentarray(imax_recv(2,k),2,2),
     &                        parentarray(imin_recv(3,k),3,2):
     &                        parentarray(imax_recv(3,k),3,2),
     &                        parentarray(imin_recv(4,k),4,2):
     &                        parentarray(imax_recv(4,k),4,2),
     &                        parentarray(imin_recv(5,k),5,2):
     &                        parentarray(imax_recv(5,k),5,2)),
     &                        size,MPI_DOUBLE_PRECISION,k,etiquette,
     &                        MPI_COMM_WORLD,statut,code)
C
            End If 
C
        End If
C
      End Do 
C      
#else
C
      parent % var % array5(indmin(1):indmax(1),
     &                      indmin(2):indmax(2),
     &                      indmin(3):indmax(3),
     &                      indmin(4):indmax(4),
     &                      indmin(5):indmax(5)) 
     &= tempP(indmin(1):indmax(1),
     &        indmin(2):indmax(2),
     &        indmin(3):indmax(3),
     &        indmin(4):indmax(4),
     &        indmin(5):indmax(5)) 
C
#endif               
C		  	      	    	      		   				  	       	  	                         		  	            
C     Deallocations      
      deallocate(tempP)
      deallocate(tempC)
C
#ifdef AGRIF_MPI
C
      deallocate(parentarray,childarray) 
C
#endif      
C       
C		  
      End Subroutine Agrif_Update5D            
C
C
C
C     **************************************************************************  
CCC   Subroutine Agrif_Prtbounds
C     **************************************************************************
C
      Subroutine Agrif_Prtbounds(nbdim,indmin,indmax,s_Parent_temp,
     &                              s_Child_temp,s_Child,ds_Child,
     &                              s_Parent,ds_Parent,
     &                              pttruetab,cetruetab,pttab_Child,
     &                              pttab_Parent,parentglob)     
C
CCC   Description:
CCC   Subroutine calculating the bounds of the parent grid for the interpolation
CCC   of the child grid     
C
C
C     Declarations:
C
      Implicit none
C
#ifdef AGRIF_MPI
#include "mpif.h"
#endif
C
C     Arguments
      Integer :: nbdim
      Integer,Dimension(nbdim) :: indmin,indmax
      Real,Dimension(nbdim) :: s_Parent_temp,s_child_temp
      Real,Dimension(nbdim) :: s_Child,ds_child
      Real,Dimension(nbdim) :: s_Parent,ds_Parent
      Integer,Dimension(nbdim) :: pttruetab,cetruetab
      Integer,Dimension(nbdim) :: pttab_Child,pttab_Parent
      Integer,Dimension(nbdim,2) :: parentglob     
C
C     Local variables
      Integer :: i,lbglob,ubglob,code
      Real,Dimension(nbdim) :: dim_newmin,dim_newmax      
C
C
      Do i = 1,nbdim
C
        dim_newmin(i) = s_Child(i) + (pttruetab(i) - 
     &                                pttab_Child(i)) * ds_Child(i)
C
        dim_newmax(i) = s_Child(i) + (cetruetab(i) - 
     &                                pttab_Child(i)) * ds_Child(i)         
C     
        indmin(i) = pttab_Parent(i) - 
     &                floor(-(dim_newmin(i)-s_Parent(i))/ds_Parent(i))                  
C
        indmax(i) = pttab_Parent(i) + 
     &                int((dim_newmax(i)-s_Parent(i))/ds_Parent(i))          
C
#ifdef AGRIF_MPI        
C
        Call MPI_ALLREDUCE(parentglob(i,1),lbglob,1,MPI_INTEGER,MPI_MIN,
     &                     MPI_COMM_WORLD,code)  
C
        Call MPI_ALLREDUCE(parentglob(i,2),ubglob,1,MPI_INTEGER,MPI_MAX,
     &                     MPI_COMM_WORLD,code)   
C
#else
C
        lbglob = parentglob(i,1)
        ubglob = parentglob(i,2)
C
#endif
C
        indmin(i) = max(indmin(i),lbglob)
        indmax(i) = min(indmax(i),ubglob)
C
        s_Parent_temp(i) = s_Parent(i) + 
     &                     (indmin(i) - pttab_Parent(i)) * 
     &                      ds_Parent(i) 
C     
        s_Child_temp(i) = dim_newmin(i)
C
      End Do
C
      Return
C
C
      End Subroutine Agrif_Prtbounds
C
C
C
#ifdef AGRIF_MPI
C
C     **************************************************************************  
CCC   Subroutine Agrif_Sendtoproc2
C     **************************************************************************
C
      Subroutine Agrif_Sendtoproc2(Agrif_Nbprocs,New_Nbprocs,sendtoproc,
     &                            nbdim,imin,imax,
     &                            iminloc,imaxloc,parentglob,tab,
     &                            tabarray,lb,ub)   
C
CCC   Description:
CCC   Subroutine calculating the limits of parts of the parent grid sent to others processors   
C
C
C     Declarations:
C
      Implicit none
C
C     Arguments
      Integer :: Agrif_Nbprocs,New_Nbprocs,nbdim
      Integer :: lb,ub
      Logical,Dimension(0:Agrif_Nbprocs-1) :: sendtoproc
      Integer,Dimension(nbdim,0:Agrif_Nbprocs-1) :: imin,imax
      Integer,Dimension(nbdim,0:Agrif_Nbprocs-1) :: iminloc,imaxloc
      Integer,Dimension(nbdim,2) :: parentglob
      Integer,Dimension(nbdim,2,0:Agrif_Nbprocs-1) :: tab
      Integer,Dimension(lb:ub,nbdim,2) :: tabarray 
      
C
C     Local variables
      Integer :: k,i,i1
C
C
      Do k = 0,New_Nbprocs-1
C
        sendtoproc(k) = .true.
C
        Do i = 1,nbdim
C
          imin(i,k) = max(parentglob(i,1),tab(i,1,k))
          imax(i,k) = min(parentglob(i,2),tab(i,2,k))
C
          If (imin(i,k) > imax(i,k)) Then
C
              sendtoproc(k) = .false.
C
            Else
C
              Do i1 = 1,tabarray(imin(i,k),i,1)
C
                iminloc(i,k) = tabarray(imin(i,k),i,i1+1)
C         
              End Do
C
              Do i1 = 1,tabarray(imax(i,k),i,1)
C
                imaxloc(i,k) = tabarray(imax(i,k),i,i1+1)
C
              End Do       
C
          End If
C
        End Do
C
      End Do       
C
      Return
C
C
      End Subroutine Agrif_Sendtoproc2
C
#endif
C
C
C
C     **************************************************************************  
CCC   Subroutine Agrif_Update_1D_Recursive 
C     **************************************************************************
C
      Subroutine Agrif_Update_1D_recursive(TypeUpdate,tempP,tempC,
     &                                     indmin,indmax, 
     &                                     pttab_child,petab_child,
     &                                     s_child,s_parent,
     &                                     ds_child,ds_parent,nbdim)     
C
CCC   Description:
CCC   Subroutine to update a 1D grid variable on the parent grid.    
C
CC    Method:
C
C     Declarations:
C
      Implicit none
C
C     Arguments
      Integer                   :: nbdim
      Integer                   :: TypeUpdate
      Integer, Dimension(nbdim) :: indmin,indmax
      Integer, Dimension(nbdim) :: pttab_child,petab_child
      Real, Dimension(nbdim)    :: s_child,s_parent
      Real, Dimension(nbdim)    :: ds_child,ds_parent
      Real, Dimension(indmin(nbdim):indmax(nbdim))           :: tempP	
      Real, Dimension(pttab_child(nbdim):petab_child(nbdim)) :: tempC
C
C
      Call Agrif_UpdateBase(TypeUpdate,
     &                  tempP(indmin(nbdim):indmax(nbdim)),
     &                  tempC(pttab_child(nbdim):petab_child(nbdim)),
     &                  indmin(nbdim),indmax(nbdim),           
     &                  pttab_child(nbdim),petab_child(nbdim),
     &                  s_parent(nbdim),s_child(nbdim),
     &                  ds_parent(nbdim),ds_child(nbdim))
C		
      Return
C
C
      End Subroutine Agrif_Update_1D_recursive
C
C
C
C     **************************************************************************  
CCC   Subroutine Agrif_Update_2D_Recursive 
C     **************************************************************************
C
      Subroutine Agrif_Update_2D_recursive(TypeUpdate,tempP,tempC,
     &                                     indmin,indmax,   
     &                                     pttab_child,petab_child,
     &                                     s_child,s_parent,
     &                                     ds_child,ds_parent,nbdim)
C
CCC   Description:
CCC   Subroutine to update a 2D grid variable on the parent grid. 
CCC   It calls Agrif_Update_1D_Recursive and Agrif_UpdateBase.   
C
CC    Method:
C
C     Declarations:
C
      Implicit none
C     
      Integer                   :: nbdim
      Integer                   :: TypeUpdate
      Integer, Dimension(nbdim) :: indmin,indmax
      Integer, Dimension(nbdim) :: pttab_child,petab_child
      Real, Dimension(nbdim)    :: s_child,s_parent
      Real, Dimension(nbdim)    :: ds_child,ds_parent
      Real, Dimension(indmin(1):indmax(1),
     &                indmin(2):indmax(2))           :: tempP	
      Real, Dimension(pttab_child(1):petab_child(1),
     &                pttab_child(2):petab_child(2)) :: tempC
C
C     Local variables      
      Real, Dimension(:,:), Allocatable :: tabtemp
      Integer :: i,j
C
C	
      Allocate(tabtemp(indmin(1):indmax(1),
     &                 pttab_child(2):petab_child(2)))
C
      Do j = pttab_child(nbdim),petab_child(nbdim)
C	
        Call Agrif_Update_1D_recursive(TypeUpdate,    
     &         tabtemp(indmin(nbdim-1):indmax(nbdim-1),j),
     &         tempC(pttab_child(nbdim-1):petab_child(nbdim-1),j),
     &         indmin(1:nbdim-1),indmax(1:nbdim-1),
     &         pttab_child(1:nbdim-1),petab_child(1:nbdim-1),
     &         s_child(1:nbdim-1),s_parent(1:nbdim-1),
     &         ds_child(1:nbdim-1),ds_parent(1:nbdim-1),nbdim-1)
C	
      End Do
C	
      Do i = indmin(1),indmax(1)
C
        Call Agrif_UpdateBase(TypeUpdate,
     &           tempP(i,indmin(nbdim):indmax(nbdim)),
     &           tabtemp(i,pttab_child(nbdim):petab_child(nbdim)),
     &           indmin(nbdim),indmax(nbdim),
     &           pttab_child(nbdim),petab_child(nbdim),
     &           s_parent(nbdim),s_child(nbdim),
     &           ds_parent(nbdim),ds_child(nbdim))
C        
      End Do
C		
      Deallocate(tabtemp)
C
      Return
C
C
      End Subroutine Agrif_Update_2D_recursive
C
C
C
C     **************************************************************************  
CCC   Subroutine Agrif_Update_3D_Recursive 
C     **************************************************************************
C
      Subroutine Agrif_Update_3D_recursive(TypeUpdate,tempP,tempC,
     &                                     indmin,indmax,   
     &                                     pttab_child,petab_child,
     &                                     s_child,s_parent,
     &                                     ds_child,ds_parent,nbdim)
C
CCC   Description:
CCC   Subroutine to update a 3D grid variable on the parent grid. 
CCC   It calls Agrif_Update_2D_Recursive and Agrif_UpdateBase.   
C
CC    Method:
C
C     Declarations:
C
      Implicit none
C     
      Integer                   :: nbdim
      Integer                   :: TypeUpdate
      Integer, Dimension(nbdim) :: indmin,indmax
      Integer, Dimension(nbdim) :: pttab_child,petab_child
      Real, Dimension(nbdim)    :: s_child,s_parent
      Real, Dimension(nbdim)    :: ds_child,ds_parent
      Real, Dimension(indmin(1):indmax(1),
     &                indmin(2):indmax(2),
     &                indmin(3):indmax(3))           :: tempP	
      Real, Dimension(pttab_child(1):petab_child(1),
     &                pttab_child(2):petab_child(2),
     &                pttab_child(3):petab_child(3)) :: tempC
C
C     Local variables      
      Real, Dimension(:,:,:), Allocatable :: tabtemp
      Integer :: i,j,k
C
C	
      Allocate(tabtemp(indmin(1):indmax(1),
     &                 indmin(2):indmax(2), 
     &                 pttab_child(3):petab_child(3)))
C
      Do k = pttab_child(nbdim),petab_child(nbdim)
C	
        Call Agrif_Update_2D_recursive(TypeUpdate,    
     &         tabtemp(indmin(nbdim-2):indmax(nbdim-2),
     &                 indmin(nbdim-1):indmax(nbdim-1),k),
     &         tempC(pttab_child(nbdim-2):petab_child(nbdim-2),
     &               pttab_child(nbdim-1):petab_child(nbdim-1),k),
     &         indmin(1:nbdim-1),indmax(1:nbdim-1),
     &         pttab_child(1:nbdim-1),petab_child(1:nbdim-1),
     &         s_child(1:nbdim-1),s_parent(1:nbdim-1),
     &         ds_child(1:nbdim-1),ds_parent(1:nbdim-1),nbdim-1)
C	
      End Do
C
      Do j = indmin(2),indmax(2)
C	
        Do i = indmin(1),indmax(1)
C
          Call Agrif_UpdateBase(TypeUpdate,
     &           tempP(i,j,indmin(nbdim):indmax(nbdim)),
     &           tabtemp(i,j,pttab_child(nbdim):petab_child(nbdim)),
     &           indmin(nbdim),indmax(nbdim),
     &           pttab_child(nbdim),petab_child(nbdim),
     &           s_parent(nbdim),s_child(nbdim),
     &           ds_parent(nbdim),ds_child(nbdim))
C
        End Do 
C        
      End Do
C		
      Deallocate(tabtemp)
C
      Return
C
C
      End Subroutine Agrif_Update_3D_recursive
C
C
C
C     **************************************************************************  
CCC   Subroutine Agrif_Update_4D_Recursive 
C     **************************************************************************
C
      Subroutine Agrif_Update_4D_recursive(TypeUpdate,tempP,tempC,
     &                                     indmin,indmax,   
     &                                     pttab_child,petab_child,
     &                                     s_child,s_parent,
     &                                     ds_child,ds_parent,nbdim)
C
CCC   Description:
CCC   Subroutine to update a 4D grid variable on the parent grid. 
CCC   It calls Agrif_Update_3D_Recursive and Agrif_UpdateBase.    
C
CC    Method:
C
C     Declarations:
C
      Implicit none
C     
      Integer                   :: nbdim
      Integer                   :: TypeUpdate
      Integer, Dimension(nbdim) :: indmin,indmax
      Integer, Dimension(nbdim) :: pttab_child,petab_child
      Real, Dimension(nbdim)    :: s_child,s_parent
      Real, Dimension(nbdim)    :: ds_child,ds_parent
      Real, Dimension(indmin(1):indmax(1),
     &                indmin(2):indmax(2),
     &                indmin(3):indmax(3),
     &                indmin(4):indmax(4))           :: tempP	
      Real, Dimension(pttab_child(1):petab_child(1),
     &                pttab_child(2):petab_child(2),
     &                pttab_child(3):petab_child(3),
     &                pttab_child(4):petab_child(4)) :: tempC
C
C     Local variables      
      Real, Dimension(:,:,:,:), Allocatable :: tabtemp
      Integer :: i,j,k,l
C
C	
      Allocate(tabtemp(indmin(1):indmax(1),
     &                 indmin(2):indmax(2),
     &                 indmin(3):indmax(3), 
     &                 pttab_child(4):petab_child(4)))
C
      Do l = pttab_child(nbdim),petab_child(nbdim)
C	
        Call Agrif_Update_3D_recursive(TypeUpdate,    
     &         tabtemp(indmin(nbdim-3):indmax(nbdim-3),
     &                 indmin(nbdim-2):indmax(nbdim-2),
     &                 indmin(nbdim-1):indmax(nbdim-1),l),
     &         tempC(pttab_child(nbdim-3):petab_child(nbdim-3),
     &               pttab_child(nbdim-2):petab_child(nbdim-2),
     &               pttab_child(nbdim-1):petab_child(nbdim-1),l),
     &         indmin(1:nbdim-1),indmax(1:nbdim-1),
     &         pttab_child(1:nbdim-1),petab_child(1:nbdim-1),
     &         s_child(1:nbdim-1),s_parent(1:nbdim-1),
     &         ds_child(1:nbdim-1),ds_parent(1:nbdim-1),nbdim-1)
C	
      End Do
C
      Do k = indmin(3),indmax(3)
C
        Do j = indmin(2),indmax(2)
C	
          Do i = indmin(1),indmax(1)
C
            Call Agrif_UpdateBase(TypeUpdate,
     &           tempP(i,j,k,indmin(nbdim):indmax(nbdim)),
     &           tabtemp(i,j,k,pttab_child(nbdim):petab_child(nbdim)),
     &           indmin(nbdim),indmax(nbdim),
     &           pttab_child(nbdim),petab_child(nbdim),
     &           s_parent(nbdim),s_child(nbdim),
     &           ds_parent(nbdim),ds_child(nbdim))
C
          End Do 
C
        End Do 
C        
      End Do
C		
      Deallocate(tabtemp)
C
      Return
C
C
      End Subroutine Agrif_Update_4D_recursive
C
C
C
C     **************************************************************************  
CCC   Subroutine Agrif_Update_5D_Recursive 
C     **************************************************************************
C
      Subroutine Agrif_Update_5D_recursive(TypeUpdate,tempP,tempC,
     &                                     indmin,indmax,   
     &                                     pttab_child,petab_child,
     &                                     s_child,s_parent,
     &                                     ds_child,ds_parent,nbdim)
C
CCC   Description:
CCC   Subroutine to update a 5D grid variable on the parent grid. 
CCC   It calls Agrif_Update_4D_Recursive and Agrif_UpdateBase.    
C
CC    Method:
C
C     Declarations:
C
      Implicit none
C     
      Integer                   :: nbdim
      Integer                   :: TypeUpdate
      Integer, Dimension(nbdim) :: indmin,indmax
      Integer, Dimension(nbdim) :: pttab_child,petab_child
      Real, Dimension(nbdim)    :: s_child,s_parent
      Real, Dimension(nbdim)    :: ds_child,ds_parent
      Real, Dimension(indmin(1):indmax(1),
     &                indmin(2):indmax(2),
     &                indmin(3):indmax(3),
     &                indmin(4):indmax(4),
     &                indmin(5):indmax(5))           :: tempP	
      Real, Dimension(pttab_child(1):petab_child(1),
     &                pttab_child(2):petab_child(2),
     &                pttab_child(3):petab_child(3),
     &                pttab_child(4):petab_child(4),
     &                pttab_child(5):petab_child(5)) :: tempC
C
C     Local variables      
      Real, Dimension(:,:,:,:,:), Allocatable :: tabtemp
      Integer :: i,j,k,l,m
C
C	
      Allocate(tabtemp(indmin(1):indmax(1),
     &                 indmin(2):indmax(2),
     &                 indmin(3):indmax(3),
     &                 indmin(4):indmax(4),   
     &                 pttab_child(5):petab_child(5)))
C
      Do m = pttab_child(nbdim),petab_child(nbdim)
C	
        Call Agrif_Update_4D_recursive(TypeUpdate,    
     &         tabtemp(indmin(nbdim-4):indmax(nbdim-4),
     &                 indmin(nbdim-3):indmax(nbdim-3),
     &                 indmin(nbdim-2):indmax(nbdim-2),
     &                 indmin(nbdim-1):indmax(nbdim-1),m),
     &         tempC(pttab_child(nbdim-4):petab_child(nbdim-4),
     &               pttab_child(nbdim-3):petab_child(nbdim-3),
     &               pttab_child(nbdim-2):petab_child(nbdim-2),
     &               pttab_child(nbdim-1):petab_child(nbdim-1),m),
     &         indmin(1:nbdim-1),indmax(1:nbdim-1),
     &         pttab_child(1:nbdim-1),petab_child(1:nbdim-1),
     &         s_child(1:nbdim-1),s_parent(1:nbdim-1),
     &         ds_child(1:nbdim-1),ds_parent(1:nbdim-1),nbdim-1)
C	
      End Do
C
      Do l = indmin(4),indmax(4)
C
        Do k = indmin(3),indmax(3)
C
          Do j = indmin(2),indmax(2)
C	
            Do i = indmin(1),indmax(1)
C
              Call Agrif_UpdateBase(TypeUpdate,
     &           tempP(i,j,k,l,indmin(nbdim):indmax(nbdim)),
     &           tabtemp(i,j,k,l,
     &                   pttab_child(nbdim):petab_child(nbdim)),
     &           indmin(nbdim),indmax(nbdim),
     &           pttab_child(nbdim),petab_child(nbdim),
     &           s_parent(nbdim),s_child(nbdim),
     &           ds_parent(nbdim),ds_child(nbdim))
C
            End Do
C
          End Do 
C
        End Do 
C        
      End Do
C		
      Deallocate(tabtemp)
C
      Return
C
C
      End Subroutine Agrif_Update_5D_recursive
C
C
C
C     **************************************************************************  
CCC   Subroutine Agrif_UpdateBase  
C     **************************************************************************  
C  
      Subroutine Agrif_UpdateBase(TypeUpdate,
     &                            parenttab,childtab,
     &                            indmin,indmax,pttab_child,petab_child,
     &                            s_parent,s_child,ds_parent,ds_child)   
C
CCC   Description:
CCC   Subroutine calling the updating method chosen by the user (copy, average or 
CCC   full-weighting).    
C
CC    Method:
C
C     Declarations:
C
      Implicit none
C
      Integer :: TypeUpdate
      Integer :: indmin,indmax
      Integer :: pttab_child,petab_child
      Real,Dimension(indmin:indmax)           :: parenttab       
      Real,Dimension(pttab_child:petab_child) :: childtab      
      Real    :: s_parent,s_child
      Real    :: ds_parent,ds_child       
C
C
      If (TypeUpdate == AGRIF_Update_copy) then
C             
          Call copy1D
     &         (parenttab,childtab,
     &          indmax-indmin+1,petab_child-pttab_child+1,
     &          s_parent,s_child,ds_parent,ds_child)     
C
        Else If (TypeUpdate == AGRIF_Update_average) then
C             
          Call average1D
     &         (parenttab,childtab,
     &          indmax-indmin+1,petab_child-pttab_child+1,
     &          s_parent,s_child,ds_parent,ds_child)    
C
        else If (TypeUpdate == AGRIF_Update_full_weighting) then
C             
          Call full_weighting1D
     &         (parenttab,childtab,
     &          indmax-indmin+1,petab_child-pttab_child+1,
     &          s_parent,s_child,ds_parent,ds_child)
C
      End if 
C
      Return                
C
C      
      End Subroutine Agrif_UpdateBase
C
C
C
      End Module Agrif_Update



      
