!
! $Id: modupdatebasic.F,v 1.2 2003/12/17 13:56:11 pmarches Exp $
!
C     AGRIF (Adaptive Grid Refinement In Fortran)
C
C     Copyright (C) 2003 Laurent Debreu (Laurent.Debreu@imag.fr)
C                        Christophe Vouland (Christophe.Vouland@imag.fr)    
C
C     This program is free software; you can redistribute it and/or modify
C     it under the terms of the GNU General Public License as published by
C     the Free Software Foundation; either version 2 of the License, or
C     (at your option) any later version.
C
C     This program is distributed in the hope that it will be useful,
C     but WITHOUT ANY WARRANTY; without even the implied warranty of
C     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
C     GNU General Public License for more details.
C
C     You should have received a copy of the GNU General Public License
C     along with this program; if not, write to the Free Software
C     Foundation, Inc., 59 Temple Place -  Suite 330, Boston, MA 02111-1307, USA.
C
C
C
CCC   Module Agrif_Updatebasic
C      
C
      Module Agrif_Updatebasic
C
CCC   Description:
CCC   Module containing different procedures of update (copy,average,
CCC   full_weighting) used in the Agrif_Update module.
C
C     Modules used:
C             
      CONTAINS
C     Define procedures contained in this module
C
C	
C
C     **************************************************************************  
CCC   Subroutine Copy1d  
C     ************************************************************************** 
C	
      Subroutine copy1d(x,y,np,nc,
     &                  s_parent,s_child,ds_parent,ds_child) 
C
CCC   Description:
CCC   Subroutine to do a copy on a parent grid (vector x) from its child grid 
CCC   (vector y).  
C
CC    Method:
C
C     Declarations:
C
      Implicit none
C        
C     Arguments
      Integer             :: np,nc      
      Real, Dimension(np) :: x      
      Real, Dimension(nc) :: y  
      Real                :: s_parent,s_child
      Real                :: ds_parent,ds_child
C
C     Local variables
      Integer :: i,locind_child_left,coeffraf
      Real    :: xpos      
C 
C
      coeffraf = nint(ds_parent/ds_child)
C
      If (coeffraf == 1) Then
C
          locind_child_left = 1 + nint((s_parent - s_child)/ds_child)
C        
          x(1:np) = y(locind_child_left:locind_child_left+np-1)
C
          return
C
      End If
C
      xpos = s_parent      
C
      Do i = 1,np
C
        locind_child_left = 1 + nint((xpos - s_child)/ds_child)
C
        x(i) = y(locind_child_left)                
C
        xpos = xpos + ds_parent
C
      End Do
C
      Return 
C
C	
      End Subroutine copy1d
C
C
C
C     **************************************************************************  
CCC   Subroutine Average1d  
C     ************************************************************************** 
C   
      Subroutine average1d(x,y,np,nc,
     &                     s_parent,s_child,ds_parent,ds_child) 
C
CCC   Description:
CCC   Subroutine to do an update by average on a parent grid (vector x) from its 
CCC   child grid (vector y).
C
C     Arguments
      Integer             :: np,nc      
      Real, Dimension(np) :: x      
      Real, Dimension(nc) :: y  
      Real                :: s_parent,s_child
      Real                :: ds_parent,ds_child
C
C     Local variables
      Integer :: i,locind_child_left,coeffraf
      Real    :: xpos 
C	 
C
      coeffraf = nint(ds_parent/ds_child)
C
      If (coeffraf == 1) Then
C
          locind_child_left = 1 + nint((s_parent - s_child)/ds_child)
C        
          x(1:np) = y(locind_child_left:locind_child_left+np-1)
C
          return
C
      End If
C
      xpos = s_parent      
C
      Do i = 1,np
C
        locind_child_left = 1 + nint((xpos - s_child)/ds_child)
C
        If ((locind_child_left-1 < 1) 
     &      .OR. (locind_child_left+1 > nc)) Then 
C
            x(i) = y(locind_child_left)                
C
          Else         
C
            x(i) = (y(locind_child_left-1)+y(locind_child_left)+
     &              y(locind_child_left+1))/3.                
C
        End If
C
        xpos = xpos + ds_parent
C
      End Do	
C
      Return 
C            
C	   	  	
      End Subroutine average1d
C
C
C
C     **************************************************************************  
CCC   Subroutine Full_weighting1d  
C     **************************************************************************  
C
      Subroutine full_weighting1D(x,y,np,nc,
     &                            s_parent,s_child,ds_parent,ds_child) 
C
CCC   Description:
CCC   Subroutine to do an update by full_weighting on a parent grid (vector x) 
CCC   from its child grid (vector y).
C	 
C     Arguments
      Integer             :: np,nc      
      Real, Dimension(np) :: x      
      Real, Dimension(nc) :: y  
      Real                :: s_parent,s_child
      Real                :: ds_parent,ds_child
C
C     Local variables
      Integer :: i,locind_child_left,coeffraf
      Real    :: xpos 
C	 
C
      coeffraf = nint(ds_parent/ds_child)
C
      If (coeffraf == 1) Then
C
          locind_child_left = 1 + nint((s_parent - s_child)/ds_child)
C        
          x(1:np) = y(locind_child_left:locind_child_left+np-1)
C
          return
C
      End If
C
      xpos = s_parent      
C
      Do i = 1,np
C
        locind_child_left = 1 + nint((xpos - s_child)/ds_child)
C
        If ((locind_child_left-1 < 1) 
     &      .OR. (locind_child_left+1 > nc)) Then 
C
            x(i) = y(locind_child_left)                
C
          Else
C        
            x(i) = (y(locind_child_left-1)+2.*y(locind_child_left)+
     &              y(locind_child_left+1))/4.                
C
        End If
C
        xpos = xpos + ds_parent
C
      End Do	
C
      Return 
C            
C	   	  	
      End Subroutine full_weighting1D      
C
C
C
      End module AGRIF_updatebasic
