%{
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include "decl.h"
extern int line_num;
extern int line_num_common;
%}

%union {
       int ival;
       char na[LONGNOM];
       listdim * d;
       listnom * ln;
       variable *var;
       listvar *l;
       point *pt;
       typedim dim1;
       lpoint *lpt;
       lnum *lnu;
       }

%token <na> TOK_TOPARENT       
%token TOK_INTERP		/* interpolation of the grid variables */
%token TOK_SEP
%token TOK_NEXTLINE
%token TOK_INCLUDE
%token TOK_USE
%token <na> TOK_TYPE		/* type of the grid variable */
%token <na> TOK_NAME
%token <na> TOK_NAME_CHAR
%token <na> TOK_PROBTYPE	/* dimension of the problem */
%token <na> TOK_INTERPTYPE	/* kind of interpolation */
%token <na> TOK_VARTYPE		/* position of the grid variable on the cells of the mesh */
%token <na> TOK_COMMON		
%token <na> TOK_SLASH
%token <na> TOK_RAF		/* type of each space dimension */
%token <na> TOK_UPDATETYPE	/* kind of update */
%token <na> TOK_TORESTAURE	/* restoration */
%token <na> TOK_TOUPDATE	/* update */
%token <na> TOK_BC		/* calculation of the boundary conditions */
%token <na> TOK_OP
%token <ival> TOK_NUM
%token <na> TOK_BCINTERP	/* kind of interpolation for the boundary conditions */
%token <na> TOK_EFFICIENCY	/* efficiency for the adaptive refinement */
%token <na> TOK_RAFMAX
%token <na> TOK_RAFMAXX
%token <na> TOK_RAFMAXY
%token <na> TOK_RAFMAXZ
%token TOK_MINWIDTH	/* minimum width for the adaptive refinement */
%token <na> TOK_STATIC_COMMON
%token <na> TOK_STATIC_VARS
%token TOK_COEFFRAFX		/* space refinement factor in the x direction */
%token TOK_REGRIDDING
%token TOK_COEFFRAFY		/* space refinement factor in the y direction */
%token TOK_COEFFRAFZ		/* space refinement factor in the z direction */
%token TOK_COEFFRAFTX		/* time refinement factor in the x direction */
%token TOK_COEFFRAFTY		/* time refinement factor in the y direction */
%token TOK_COEFFRAFTZ		/* time refinement factor in the z direction */
%token TOK_COMMONFILE		/* name of the common file */
%token TOK_COMMENTS
%token <na> TOK_FILENAME
%token <na> TOK_USEITEM
%token ','
%token ';'
%token ':'
%token '('
%token ')'   
%token '['
%token ']'
%type <l> liste
%type <l> liste1
%type <l> common_decl
%type <l> type_common_decl
%type <l> type_common_temp
%type <l> liste_ss_param
%type <l> decl
%type <d> DIM
%type <d> DIMINV
%type <d> PARAM
%type <ln> LIST_NAME
%type <lnu> LIST_NUM
%type <var> variable
%type <var> variable_ss_param 
%type <dim1> DIM1
%type <na> DIM1_FIN
%type <na> EXP
%type <na> FACTEUR
%type <pt> point
%type <pt> DIM2
%type <ival> DIM2_FIN
%type <ival> DIM3

%left TOK_OP
%%
input :
      | input line
;
line :'\n'
      | TOK_TOPARENT TOK_SEP LIST_NAME ';' {toparent($3);}  
      | TOK_TORESTAURE TOK_SEP LIST_NAME ';' {torestaure($3);}
      | TOK_PROBTYPE EXP ';'	{initdimprob(1,$2,"0","0");}
      | TOK_PROBTYPE EXP ',' EXP ';'	{initdimprob(2,$2,$4,"0");}   
      | TOK_PROBTYPE EXP ',' EXP ',' EXP ';'	{initdimprob(3,$2,$4,$6);}
      | TOK_COEFFRAFX TOK_NUM ';' {coeffrafx=$2;}
      | TOK_REGRIDDING TOK_NUM ';' {regridding=$2;}
      | TOK_COEFFRAFY TOK_NUM ';' {coeffrafy=$2;}
      | TOK_COEFFRAFZ TOK_NUM ';' {coeffrafz=$2;}
      | TOK_COEFFRAFTX TOK_NUM ';' {coeffraftx=$2;}
      | TOK_COEFFRAFTY TOK_NUM ';' {coeffrafty=$2;}
      | TOK_COEFFRAFTZ TOK_NUM ';' {coeffraftz=$2;}
      | TOK_COMMONFILE TOK_FILENAME ';' {strcpy(commonfile_main,$2);
      					 existe_commonfile=1;}
      | TOK_STATIC_COMMON LIST_NAME ';'	{add_static_commons($2);}			      
      | TOK_STATIC_VARS LIST_NAME ';'	{add_static_vars($2);}
      | decl '\n' {ajoutevar($1);}
      | decl_raf '\n'
      | decl_type '\n'
      | interp ';'       
      | bc ';'
      | update ';'
      | bcinterp ';'
      | TOK_EFFICIENCY TOK_NUM ';' {userefficiency=1;efficiency=$2;}
      | TOK_RAFMAX TOK_NUM ';' {rafmaxx=$2,rafmaxy=$2,rafmaxz=$2;}
      | TOK_RAFMAXX TOK_NUM ';' {rafmaxx=$2;}
      | TOK_RAFMAXY TOK_NUM ';' {rafmaxy=$2;}
      | TOK_RAFMAXZ TOK_NUM ';' {rafmaxz=$2;}
      | TOK_MINWIDTH TOK_NUM ';'  {userminwidth=1;minwidth=$2;}
      | TOK_INCLUDE TOK_FILENAME {firstpass = 1; processfortran($2); firstpass = 0; processfortran($2);}
      | TOK_USE TOK_USEITEM ';' {if (!strcmp($2,"FIXED_GRIDS")) fixedgrids=1;
      				if (!strcmp($2,"ONLY_FIXED_GRIDS")) onlyfixedgrids=1;
				}
      
;

/* list of variables whom boundary conditions are calculated */
bc : TOK_BC TOK_SEP liste_bc
;
liste_bc: bc_var | liste_bc ',' bc_var
;
bc_var : LIST_NAME ':' '(' DIM2 ')' {change_bc($1,$4);}
;

/* list of variables interpolated with kind of interpolation */
interp : TOK_INTERP TOK_SEP liste_interp 
;
liste_interp : interpvar | liste_interp ',' interpvar
;
interpvar : LIST_NAME ':' TOK_INTERPTYPE 
				{
				strcpy(tempname,"AGRIF_");
				strcat(tempname,$3);
				change_interp($1,tempname);}
;

/* kind of interpolation for the calculation of the boundary conditions */
bcinterp : TOK_BCINTERP TOK_SEP liste_interpbc
;
liste_interpbc : interpbc | liste_interpbc ',' interpbc
;
interpbc : LIST_NAME ':' TOK_INTERPTYPE 
				{
				strcpy(tempname,"AGRIF_");
				strcat(tempname,$3);				
				change_interpbc($1,tempname);}
;

/* list of variables updated with kind of update */
update : TOK_TOUPDATE TOK_SEP liste_update 
;
liste_update : upd | liste_update ',' upd
;
upd : LIST_NAME ':' TOK_UPDATETYPE 
				{change_update($1,$3);}
;

/* declaration of the variable in the common files */
decl : TOK_TYPE PARAM TOK_SEP liste_ss_param ';' 
		{set_pos($4);change_dim($2,$4);$$=settype($1,$4);}
     | TOK_COMMON common_decl {$$=$2;}
     | type_common_decl {$$=$1;}
;
common_decl : TOK_SLASH TOK_NAME TOK_SLASH liste
		{declareincommon($2,$4); $$=$4;}
     |  common_decl TOK_SLASH TOK_NAME TOK_SLASH liste
		{declareincommon($3,$5); $$=concat_var($5,$1);}				
;	
type_common_decl : TOK_TYPE type_common_temp
		{$$ = settype($1,$2);};
type_common_temp : TOK_OP DIM liste 
              {change_dim_char($2,$3);$$=$3;}	 
     |           liste
		{$$ = $1;}	
     |             liste TOK_OP DIM
              {change_dim_char($3,$1);$$=$1;}
     |   type_common_temp ',' liste TOK_OP DIM
	{}     		
;

/* positions of the variables on the mesh with indexes of the first point in the domain */	
decl_type : TOK_VARTYPE PARAM point TOK_SEP LIST_NAME ';'
     		{change_type($5,$2,$3);}
;

/* type of each space dimension for each variable */
decl_raf : TOK_RAF PARAM TOK_SEP LIST_NAME ';'
           {change_raf($4,$2);}
;

point : '[' TOK_NUM ',' TOK_NUM ',' TOK_NUM ']' {$$=create_point($2,$4,$6);}
	| '[' TOK_NUM ',' TOK_NUM ']' {$$=create_point($2,$4,0);}
	| '[' TOK_NUM ']' {$$ = create_point ($2,0,0);}
;

liste_ss_param : variable_ss_param	  {$$=insertvar(NULL,$1);}
	| liste_ss_param ',' variable_ss_param {$$=insertvar($1,$3);}
;

liste : liste1 {$$=$1;}
;

liste1 : variable 		{$$=insertvar(NULL,$1);}
      | liste1 ',' variable 	{$$=insertvar($1,$3);}   
      
;

variable : TOK_NAME PARAM 	{$$=createvar($1,$2);}
|  TOK_NAME_CHAR {$$=createvar_char($1);}
;

variable_ss_param : TOK_NAME 	 {$$=createvar($1,NULL);}     
;

PARAM : 			{$$=(listdim *)NULL;}
      | '(' DIM ')' 		{$$=$2;}
;

DIM : DIMINV 			{$$=reverse($1);}
;

DIMINV : DIM1 			{$$=insertdim(NULL,$1);}
      | DIMINV ',' DIM1 	{$$=insertdim($1,$3);}
;

DIM1 : EXP DIM1_FIN
{
if(!strcmp($2,"")){
strcpy($$.first,"1");strcpy($$.last,$1);}
else{
strcpy($$.first,$1);strcpy($$.last,$2);}}
;

DIM1_FIN :			{strcpy($$,"");}
	| ':' EXP		{strcpy($$,$2);}
;

DIM2 : DIM3 DIM2_FIN
{
if($2==-999){$$=create_point($1,$1,$1);}
else{$$=create_point($1,$2,$2);}}
;

DIM2_FIN :			{$$=-999;}
	| ':' DIM3		{$$=$2;}
;

DIM3 : TOK_NUM {$$=$1;}
	| TOK_OP TOK_NUM {$$=-$2;}
;
	
LIST_NAME : TOK_NAME  		{$$=insertnom(NULL,$1);}
      | LIST_NAME TOK_NAME 	{$$=insertnom($1,$2);}
;

LIST_NUM : TOK_NUM  		{$$=insertnum(NULL,$1);}
      | LIST_NUM TOK_NUM 	{$$=insertnum($1,$2);}
;

FACTEUR :TOK_NUM		{
sprintf($$,"%d",$1);}
	| TOK_NAME		{strcpy($$,$1);}
;

EXP	: '(' EXP ')'		{
strcpy($$,"(");strcat($$,$2);
strcat($$,")");}
	| EXP TOK_OP EXP	{sprintf($$,"%s%s%s",$1,$2,$3);}
        | TOK_OP EXP	        {sprintf($$,"%s%s",$1,$2);}
	| FACTEUR		{
	strcpy($$,$1);}
;
%%


yyerror(char *s)
{
if (!strcmp(curfile,mainfile))
{
printf("%s line %d, fichier %s\n",s,line_num,curfile);
}
else
{
printf("%s line %d, fichier %s\n",s,line_num_common,curfile);
}
exit(0);
}


int main(int argc,char *argv[])
{
extern FILE * yyin ;
listvar *parcours;
variable *v;
int i;
int writeinfo;

if (argc < 2) {
    printf("usage : conv <file> [-rm] [-incdir <directory>] [-comdirin <directory>] [-comdirout <directory>]\n");
    exit(0);
    }
globliste=(listvar *)NULL;
nosenselines=(listparameters *)NULL;
if ((yyin=fopen(argv[1],"r"))==NULL) {
    printf("the file %s doesn't exist \n",argv[1]);
    exit(0);    
    }
strcpy(mainfile,argv[1]);    
strcpy(nomdir,"AGRIF_INC");
strcpy(commondirin,".");
strcpy(commondirout,".");
checkexistcommon=1;

i=2;
while (i<argc)
{
 if (!strcmp(argv[i],"-incdir")) {     
   strcpy(nomdir,argv[i+1]);
   i++;
   }
 else if (!strcmp(argv[i],"-comdirin")) {     
   strcpy(commondirin,argv[i+1]);
   i++;
   }
 else if (!strcmp(argv[i],"-comdirout")) {     
   strcpy(commondirout,argv[i+1]);
   i++;
   }   
 else if (!strcmp(argv[i],"-rm")) {     
   checkexistcommon=0;
   }
 else {
   printf("Unkwon option : %s\n",argv[i]);
   }
  i++;       
 }  



mindimstatic=0;
instatic=0;
userefficiency=0;
userminwidth=0;
erreurwidth=0;
onlyfixedgrids=0;
fixedgrids=0;
static_commons_names=NULL;
rafmaxx=0;
rafmaxy=0;
rafmaxz=0;
minwidth=1;
coeffrafx=0;
coeffrafy=0;
coeffrafz=0;

yyparse();

writeinfo=1;

creefichiercommon(NULL,writeinfo);
/*creenosenselines();*/
creefichieramr();

}


int yywrap()
{
}
