!
! $Id: ana_initracer.F,v 1.8 2005/09/14 13:00:11 pmarches Exp $
!
#include "cppdefs.h"

      subroutine ana_initracer (tile)
      implicit none
#include "param.h"
      integer tile
#include "compute_tile_bounds.h"
      call ana_initracer_tile (Istr,Iend,Jstr,Jend)
      return
      end

      subroutine ana_initracer_tile (Istr,Iend,Jstr,Jend)
!
!--------------------------------------------------------------------
!  Set initial conditions for tracer variables using
!  analytical expressions if not find in netcdf file.
!--------------------------------------------------------------------
!
      implicit none
#include "param.h"
#include "grid.h"
#include "ocean2d.h"
#include "ocean3d.h"
#include "scalars.h"
#include "sediment.h"
      integer Istr,Iend,Jstr,Jend, i,j,k
     &      , itrc
#ifdef BIOLOGY
      real temp, SiO4
#endif
!
# include "compute_auxiliary_bounds.h"
!
#ifdef EW_PERIODIC
#  define IR_RANGE IstrR,IendR
#else
#  define IR_RANGE IstrR,IendR
#endif

#ifdef NS_PERIODIC
#  define JR_RANGE Jstr,Jend
#else
#  define JR_RANGE JstrR,JendR
#endif

#ifdef SOLVE3D

# if defined INNERSHELF || defined UPWELLING || defined BASIN    \
  || defined CANYON_A   || defined CANYON_B  || defined GRAV_ADJ \
  || defined OVERFLOW   || defined SEAMOUNT  || defined SOLITON  \
  || defined SHELFRONT  || defined RIVER     || defined VORTEX
      do k=1,N
        do j=JR_RANGE
          do i=IR_RANGE
            if (.not.got_tini(itemp)) then
              t(i,j,k,1,itemp)=1.e30
              t(i,j,k,2,itemp)=t(i,j,k,1,itemp)
            endif
#  ifdef SALINITY
            if (.not.got_tini(isalt)) then
              t(i,j,k,1,isalt)=1.e30
              t(i,j,k,2,isalt)=t(i,j,k,1,isalt)
            endif
#  endif
#  if defined PASSIVE_TRACER && !defined ANA_PSOURCE
            if (.not.got_tini(itpas)) then
              if (h(i,j).lt.300) then
                t(i,j,k,1,itpas)=1.
              else
                t(i,j,k,1,itpas)=0
              endif
            endif
#  endif

          enddo
        enddo
      enddo
# elif defined REGIONAL
      do k=1,N
        do j=JR_RANGE
          do i=IR_RANGE
            if (.not.got_tini(itemp)) then
#define Z0 (-80.)
#define THKNSS 50.
#define Z1 0.
#define STRAT 1000.
              t(i,j,k,1,itemp)=14.+5.*THKNSS*log(
     &                            cosh((z_w(i,j,k )-Z0)/THKNSS)
     &                           /cosh((z_w(i,j,k-1)-Z0)/THKNSS)
     &                               )/(z_w(i,j,k)-z_w(i,j,k-1))
     &               +((z_w(i,j,k)+z_w(i,j,k-1))/2.-Z1)/STRAT
#undef Z0
#undef THKNSS
#undef Z1
#undef STRAT
              t(i,j,k,2,itemp)=t(i,j,k,1,itemp)
            endif
#  ifdef SALINITY
            if (.not.got_tini(isalt)) then
              t(i,j,k,1,isalt)=35.
              t(i,j,k,2,isalt)=t(i,j,k,1,isalt)
            endif
#  endif
#  if defined PASSIVE_TRACER && !defined ANA_PSOURCE
            if (.not.got_tini(itpas)) then
              if (h(i,j).lt.300) then
                t(i,j,k,1,itpas)=1
              else
                 t(i,j,k,1,itpas)=0
              endif
            endif
#  endif
#  ifdef BIOLOGY
            if (.not.got_tini(iNO3_)) then
              temp=t(i,j,k,1,itemp)
               if (temp.lt.8.) then
                  SiO4=30.
               elseif (temp.ge.8. .and. temp.le.11.) then
                  SiO4=30.-((temp-8.)*(20./3.))
               elseif (temp.gt.11. .and. temp.le.13.) then
                  SiO4=10.-((temp-11.)*(8./2.))
               elseif (temp.gt.13. .and. temp.le.16.) then
                  SiO4=2.-((temp-13.)*(2./3.))
               elseif (temp.gt.16.) then
                 SiO4=0.
               endif
               t(i,j,k,1,iNO3_)=1.67+0.5873*SiO4+0.0144*SiO4**2
     &                               +0.0003099*SiO4**3
               t(i,j,k,2,iNO3_)=t(i,j,k,1,iNO3_)
            endif
#   ifdef BIO_NChlPZD
            if (.not.got_tini(iChla)) then
               t(i,j,k,1,iChla)=0.08
               t(i,j,k,2,iChla)=t(i,j,k,1,iChla)
            endif
            if (.not.got_tini(iPhy1)) then
               t(i,j,k,1,iPhy1)=0.1
               t(i,j,k,2,iPhy1)=t(i,j,k,1,iPhy1)
            endif
            if (.not.got_tini(iZoo1)) then
               t(i,j,k,1,iZoo1)=0.06
               t(i,j,k,2,iZoo1)=t(i,j,k,1,iZoo1)
            endif
            if (.not.got_tini(iDet1)) then
               t(i,j,k,1,iDet1)=0.02
               t(i,j,k,2,iDet1)=t(i,j,k,1,iDet1)
            endif
#   elif defined BIO_N2ChlPZD2
            if (.not.got_tini(iNH4_))  then
               t(i,j,k,1,iNH4_)=0.1
               t(i,j,k,2,iNH4_)=t(i,j,k,1,iNH4_)
            endif
            if (.not.got_tini(iChla)) then
               t(i,j,k,1,iChla)=0.08
               t(i,j,k,2,iChla)=t(i,j,k,1,iChla)
            endif
            if (.not.got_tini(iPhy1)) then
               t(i,j,k,1,iPhy1)=0.06
               t(i,j,k,2,iPhy1)=t(i,j,k,1,iPhy1)
            endif
            if (.not.got_tini(iZoo1)) then
               t(i,j,k,1,iZoo1)=0.04
               t(i,j,k,2,iZoo1)=t(i,j,k,1,iZoo1)
            endif
            if (.not.got_tini(iDet1)) then
               t(i,j,k,1,iDet1)=0.02
               t(i,j,k,2,iDet1)=t(i,j,k,1,iDet1)
            endif
            if (.not.got_tini(iDet2)) then
               t(i,j,k,1,iDet2)=0.02
               t(i,j,k,2,iDet2)=t(i,j,k,1,iDet2)
            endif
#   elif defined BIO_N2P2Z2D2
            if (.not.got_tini(iNH4_))  then
               t(i,j,k,1,iNH4_)=0.1
               t(i,j,k,2,iNH4_)=t(i,j,k,1,iNH4_)
            endif
            if (.not.got_tini(iPhy1)) then
               t(i,j,k,1,iPhy1)=0.04
               t(i,j,k,2,iPhy1)=t(i,j,k,1,iPhy1)
            endif
            if (.not.got_tini(iPhy2)) then
               t(i,j,k,1,iPhy2)=0.06
               t(i,j,k,2,iPhy2)=t(i,j,k,1,iPhy2)
            endif
            if (.not.got_tini(iZoo1)) then
               t(i,j,k,1,iZoo1)=0.04
               t(i,j,k,2,iZoo1)=t(i,j,k,1,iZoo1)
            endif
            if (.not.got_tini(iZoo2)) then
               t(i,j,k,1,iZoo2)=0.04
               t(i,j,k,2,iZoo2)=t(i,j,k,1,iZoo2)
            endif
            if (.not.got_tini(iDet1)) then
               t(i,j,k,1,iDet1)=0.02
               t(i,j,k,2,iDet1)=t(i,j,k,1,iDet1)
            endif
            if (.not.got_tini(iDet2)) then
               t(i,j,k,1,iDet2)=0.02
               t(i,j,k,2,iDet2)=t(i,j,k,1,iDet2)
            endif
#   endif
#  endif
#  ifdef SEDIMENT
            if (.not.got_tini(isand)) then
              t(i,j,k,1,isand)=Csed(1)
              t(i,j,k,2,isand)=t(i,j,k,1,isand)
            endif
            if (.not.got_tini(isilt)) then
              t(i,j,k,1,isilt)=Csed(2)
              t(i,j,k,2,isilt)=t(i,j,k,1,isilt)
            endif
#  endif
          enddo
        enddo
      enddo
# else
      do itrc=1,NT
        do k=1,N
          do j=JR_RANGE
            do i=IR_RANGE
              t(i,j,k,1,itrc)=???
              t(i,j,k,2,itrc)=???
            enddo
          enddo
        enddo
      enddo
# endif

# if defined EW_PERIODIC || defined NS_PERIODIC || defined  MPI
      do itrc=1,NT
        call exchange_r3d_tile (Istr,Iend,Jstr,Jend,
     &                   t(START_2D_ARRAY,1,1,itrc))
      enddo
# endif
#endif /* SOLVE3D */
#undef IR_RANGE
#undef JR_RANGE
      return
      end


