!
! $Id: bio_N2P2Z2D2.F,v 1.1 2004/12/01 14:37:07 pmarches Exp $
!
#include "cppdefs.h"
#if defined BIOLOGY && defined BIO_N2P2Z2D2

      subroutine biology_tile (Istr,Iend,Jstr,Jend)
!
! Compute biological forcing functions as defined by the
! Fasham et al. [JMR, 48, 591-639, 1990]
!
! In this particular implementation there is 7 compartments:
! NO3, NH4, Chlorophyl, PHYTOplankton, ZOOplanknton, Small Detritus, 
! and Large Detritus.
!
      implicit none
      integer Istr,Iend,Jstr,Jend
#include "param.h"
#include "grid.h"
#include "ocean3d.h"
#include "scalars.h"
#include "forces.h"
      real kwater, kChla, palpha1, palpha2,
     &     K1_NO3, K2_NO3, K1_NH4, K2_NH4, 
     &     mu_P1_Sd, mu_P2_Sd, gmax1, gmax2, 
     &     K_Zoo1, K_Zoo2, beta1, beta2,
     &     e_11, e_12, e_21, e_22, e_ZZ,                !mu_agg
     &     mu_Sd_A, mu_Z1_A, mu_Z2_A, mu_Z1_Sd, mu_Z2_Ld,  
     &     mu_A_N, mu_Ld_A, CN_Phyt, theta_m, wPhy2, wSDet, wLDet!,chla_C,
      integer ITERMAX
      parameter (
     &  ITERMAX = 3,      ! number of small implicit time steps
!
! Parameters as in Table 1; Fasham et al. [JMR, 48, 591-639, 1990]
!
     &  kwater   = 0.04,    ! light attenuation due to sea water  [m-1]
     &  kChla    = 0.024,   ! light attenuation by Chlorophyl  
                            !                         [(m^2 mg Chla)-1]
     &  palpha1  = 0.025,   !Small Phyto initial slope of the P-I curve
     &  palpha2  = 0.04,    !Large Phyto initial slope of the P-I curve
                            !                             [(W m-2 d)-1]
     &  CN_Phyt  = 6.625,   ! C:N ratio for phytoplankton
                            !                       [mMol C (mMol N)-1]
     &  theta_m  =0.020,    ! maximum Cellular Chlorophyll to Carbon
                            !                      Ratio [mg Chla/mg C]
     &  K1_NO3   = 1./1.0,  ! inverse half-saturation for Small Phytoplankton
     &  K2_NO3   = 1./2.,   ! inverse half-saturation for Large Phytoplankton
                            !         NO3 uptake       [1/(mMol N m-3)]
     &  K1_NH4   = 1./0.5,  ! inverse half-saturation for Small Phytoplankton
     &  K2_NH4   = 1./0.7,  ! inverse half-saturation for Large Phytoplankton
                            !         NH4 uptake       [1/(mMol N m-3)]
     &  mu_A_N   = 0.1,     ! Oxidation of NH4 to NO3 (Nitrification) [d-1]
     &  mu_P1_Sd = 0.027,   ! Small Phyto mortality to SDet rate        [d-1]
     &  mu_P2_Sd = 0.030,   ! Large Phyto mortality to SDet rate        [d-1]  
     &  gmax1    = 1.2,     ! maximum Small Zooplankton growth rate     [d-1]
     &  gmax2    = 0.95,    ! maximum Large Zooplankton growth rate     [d-1]
     &  beta1    = 0.75,    ! Small Zooplankton assimilation efficiency of
                            !                       Food [n.d.]
     &  beta2    = 0.70,    ! Large Zooplankton assimilation efficiency of
                            !                       Food [n.d.]
     &  K_Zoo1   = 1.0,     ! Small Zooplankton half-saturation constant
                            !                       for ingestion [d-1]
     &  K_Zoo2   = 2.0,     ! Large Zooplankton half-saturation constant
                            !                       for ingestion [d-1]
     &  e_11     = 1.,      ! Small Zooplankton preference for Small Phytoplankton
     &  e_12     = 0.4,     ! Small Zooplankton preference for Large Phytoplankton
     &  e_21     = 0.5,     ! Large Zooplankton preference for Small Phytoplankton
     &  e_22     = 1.,      ! Large Zooplankton preference for Large Phytoplankton
     &  e_zz     = 0.4,     ! Large Zooplankton preference for Small Zooplankton -->0.5
     &  mu_Z1_A  = 0.10,    ! Small Zooplankton specific excretion rate [d-1]
     &  mu_Z2_A  = 0.05,    ! Large Zooplankton specific excretion rate [d-1]
     &  mu_Z1_Sd = 0.025,   ! Small Zooplankton mortality to Detritus   [d-1]
     &  mu_Z2_Ld = 0.05,    ! Large Zooplankton mortality to Detritus   [d-1] 
     &  mu_Sd_A  = 0.10,    ! Small Detrital breakdown to NH4 rate [d-1]
     &  mu_Ld_A  = 0.05,    ! Large Detrital recycling to NH4 rate [d-1]
     &  wPhy2    = 0.5,     ! sinking velocities for Large Phytoplankton [m.d-1]
     &  wSDet    = 1.0,     !                        Small Detritus
     &  wLDet    =10.0  )   !                        Large Detritus
!
      integer i,j,k, ITER, iB
      real    NO3(N), NH4(N), Phyt_1(N),Phyt_2(N),  
     &        Zoo_1(N), Zoo_2(N), SDet(N), LDet(N),  
     &        aJ_1(N), aJ_2(N), FC(0:N), Ft_1(N), Ft_2(N),
     &        PAR, attn, Vp1, Vp2, Epp1, Epp2, cu, aL,aR, dtdays,
     &        cff, cff1, cff2, cff6, cffa, cffb,
     &        SB(N,3),dSB(0:N,3),wSB(3)
!
# include "compute_auxiliary_bounds.h"
!
      dtdays=dt/(24.*3600.*float(ITERMAX))  ! time step as fraction of day.
!
!
! Since the following solver is iterative to achieve implicit
! discretization of the biological interaction, two time slices are
! required, BIO where BIO is understood as vector of
! biological state variables: BIO=[NO3,NH4,Phyt,Zoo,SDet]. Assume
! that the iterations converge, the newly obtained state variables
! satisfy equations
!
!           BIO = BIO + dtdays * rhs(BIO)
! 
! where rhs(BIO) is the vector of biological r.h.s. computed at
! the new time step. During the iterative procedure a series of
! fractional time steps is performed in a chained mode (splitting
! by different biological conversion processes) in sequence NO3 -- 
! NH4 -- Phyt -- Zoo -- SDet, that is the main food chain. In all 
! stages the concentration of the component being consumed is
! treated in fully implicit manner, so that the algorithm guarantees
! non-negative values, no matter how strong is the concentration of
! active consuming component (Phyto or Zoo).
!
! The overall algorithm, as well as any stage of it is formulated
! in conservative form (except explicit sinking) in sense that the
! sum of concentration of all five components is conserved.
!

/*
#  ifdef EW_PERIODIC
#   define I_RANGE Istr,Iend
#  else
#   define I_RANGE IstrR,IendR
#  endif
#  ifdef NS_PERIODIC
#   define J_RANGE Jstr,Jend
#  else
#   define J_RANGE JstrR,JendR
#  endif
*/
#  define I_RANGE Istr,Iend
#  define J_RANGE Jstr,Jend

      do j=J_RANGE
        do i=I_RANGE
!
! Extract biological variables from tracer arrays; place them into
! scratch variables; restrict their values to be positive definite.
!
      do k=1,N
        NO3(k)   =max(t(i,j,k,nnew,iNO3_)  ,0.)  ! Nitrate
        NH4(k)   =max(t(i,j,k,nnew,iNH4_)  ,0.)  ! Ammonium
        Phyt_1(k)=max(t(i,j,k,nnew,iPhy1)  ,0.)  ! Small Phytoplankton
        Phyt_2(k)=max(t(i,j,k,nnew,iPhy2)  ,0.)  ! Large Phytoplankton
        Zoo_1(k) =max(t(i,j,k,nnew,iZoo1)  ,0.)  ! Small Zooplankton
        Zoo_2(k) =max(t(i,j,k,nnew,iZoo2)  ,0.)  ! Large Zooplankton
        SDet(k)  =max(t(i,j,k,nnew,iDet1)  ,0.)  ! Small Detritus
        LDet(k)  =max(t(i,j,k,nnew,iDet2)  ,0.)  ! Large Detritus
      enddo


      DO ITER=1,ITERMAX      !--> Start internal iterations to achieve
                             !    nonlinear backward-implicit solution.

        PAR=srflx(i,j)*rho0*Cp*0.43

        if (PAR.gt.0.) then
!
!   *** SUN IS UP ***
!
! Calulate aJ: Set Photosynthetically Available Radiation (PAR) at
! surface from solar radiation x 0.43. Then, within each grid box
! compute attenuation coefficient based on the concentration of
! Phytoplankton inside the grid box, and attenuate PAR from surface
! down (thus, PAR at certain depth depends on the whole distribution
! of Phytoplankton above). To compute aJ, one needs PAR somewhat in
! the middle of the gridbox, so that attenuation "attn" corresponds
! to half of the grid box height, while PAR is multiplied by it
! twice: once to get it in the middle of grid-box and once the
! compute on trhe lower grid-box interface;
!
          do k=N,1,-1     !<-- irreversible

            attn=exp(-0.5*(kwater+kChla *
     &               theta_m*(Phyt_1(k)+Phyt_2(k))*CN_Phyt*12.+1.e-20)*
     &               (z_w(i,j,k)-z_w(i,j,k-1)))

            PAR=PAR*attn
            Vp1=0.5570*(1.066**t(i,j,k,nnew,itemp))   ! From Eppley
            Vp2=0.8356*(1.066**t(i,j,k,nnew,itemp))
            cff1=PAR*palpha1
            cff2=PAR*palpha2
            Epp1=Vp1/sqrt(Vp1*Vp1+cff1*cff1)
            Epp2=Vp2/sqrt(Vp2*Vp2+cff2*cff2)
            aJ_1(k)=Epp1*cff1
            aJ_2(k)=Epp2*cff2

            PAR=PAR*attn
          enddo
!
! (1) NO3 uptake by Small Phyto
!
          do k=1,N
            cff1=dtdays*Phyt_1(k)*aJ_1(k)*K1_NO3
     &               /(1.+K1_NO3*NO3(k)+K1_NH4*NH4(k))
            NO3(k)=NO3(k)/(1.+cff1)
            Phyt_1(k)=Phyt_1(k)+cff1*NO3(k) 
          enddo
!
! (1) NO3 uptake by Large Phyto
!
          do k=1,N
            cff1=dtdays*Phyt_2(k)*aJ_2(k)*K2_NO3
     &               /(1.+K2_NO3*NO3(k)+K2_NH4*NH4(k))
            NO3(k)=NO3(k)/(1.+cff1)
            Phyt_2(k)=Phyt_2(k)+cff1*NO3(k) 
          enddo
!
! (1) NH4 uptake by Small Phyto
! (2) nitrification of NH4 ==> NO3
!
          do k=1,N
            cff1=dtdays*Phyt_1(k)*aJ_1(k)*K1_NH4
     &               /(1.+K1_NO3*NO3(k)+K1_NH4*NH4(k))
            cff2=dtdays*mu_A_N
            NH4(k)=NH4(k)/(1.+cff1+cff2)
            Phyt_1(k)=Phyt_1(k)+NH4(k)*cff1
            NO3(k)=NO3(k)+NH4(k)*cff2
          enddo
!
! (1) NH4 uptake by Large Phyto
!
          do k=1,N
            cff1=dtdays*Phyt_2(k)*aJ_2(k)*K2_NH4
     &               /(1.+K2_NO3*NO3(k)+K2_NH4*NH4(k))
            NH4(k)=NH4(k)/(1.+cff1)
            Phyt_2(k)=Phyt_2(k)+NH4(k)*cff1
          enddo
!
        else
!
!   *** SUN IS DOWN ***
!
! (1) nitrification of NH4 ==> NO3
!
          do k=1,N
            cff1=dtdays*mu_A_N
            NH4(k)=NH4(k)/(1.+cff1)
            NO3(k)=NO3(k)+NH4(k)*cff1
          enddo
!
        endif
!
!---> Ft_1:Total Food available for Small Zooplankton
! (1) Small Phytoplankton grazing by Small Zooplankton to Zoo and SDet
! (2) Large Phytoplankton grazing by Small Zooplankton to Zoo and SDet
! (a) Small Phytoplankton mortality to SDet (mu_P1_Sd)
! (b) Large Phytoplankton mortality to SDet (mu_P2_Ld)
!
        do k=1,N
          Ft_1(k)=e_11*Phyt_1(k)+e_12*Phyt_2(k)
          cff=dtdays*gmax1*Zoo_1(k)/(k_Zoo1+Ft_1(k))
          cff1=cff*e_11
          cff2=cff*e_12
          cffa=dtdays*mu_P1_Sd
          cffb=dtdays*mu_P2_Sd
          Phyt_1(k)=Phyt_1(k)/(1.+cff1+cffa)
          Phyt_2(k)=Phyt_2(k)/(1.+cff2+cffb)
          Zoo_1(k)=Zoo_1(k)+(Phyt_1(k)*cff1+Phyt_2(k)*cff2)*beta1
          SDet(k)=SDet(k)+Phyt_1(k)*(cff1*(1.-beta1)+cffa)+
     &                    Phyt_2(k)*(cff2*(1.-beta1)+cffb)
        enddo
!
!---> Ft_2:Total Food available for Large Zooplankton
! (1) Small Phytoplankton grazing by Large Zooplankton to Zoo and LDet
! (2) Large Phytoplankton grazing by Large Zooplankton to Zoo and LDet
! (a) Small Zooplankton grazing by Large Zooplankton to Zoo and LDet
!
        do k=1,N
          Ft_2(k)=e_21*Phyt_1(k)+e_22*Phyt_2(k)+e_zz*Zoo_1(k)
          cff=dtdays*gmax2*Zoo_2(k)/(k_Zoo2+Ft_2(k))
          cff1=cff*e_21
          cff2=cff*e_22
          cffa=cff*e_zz
          Phyt_1(k)=Phyt_1(k)/(1.+cff1)
          Phyt_2(k)=Phyt_2(k)/(1.+cff2)
          Zoo_1(k)=Zoo_1(k)/(1.+cffa)
          Zoo_2(k)=Zoo_2(k)+(Phyt_1(k)*cff1+Phyt_2(k)*cff2+
     &                                 Zoo_1(k)*cffa)*beta2
          LDet(k)=LDet(k)+(Phyt_1(k)*cff1+Phyt_2(k)*cff2+
     &                            Zoo_1(k)*cffa)*(1.-beta2)
        enddo
!
! (1) Small Zoo excretion to NH4  (rate mu_Z1_A)
! (2) Small Zoo mortality to SDet (rate mu_Z1_Sd)
!
        do k=1,N
          cff1=dtdays*mu_Z1_A
          cff2=dtdays*mu_Z1_Sd
          Zoo_1(k)=Zoo_1(k)/(1.+cff1+cff2)
          NH4(k)=NH4(k)+Zoo_1(k)*cff1
          SDet(k)=SDet(k)+Zoo_1(k)*cff2
        enddo
!
! (1) Large Zoo excretion to NH4  (rate mu_Z2_A)
! (2) Large Zoo mortality to LDet (rate mu_Z2_Ld)
!
        do k=1,N
          cff1=dtdays*mu_Z2_A
          cff2=dtdays*mu_Z2_Ld
          Zoo_2(k)=Zoo_2(k)/(1.+cff1+cff2)
          NH4(k)=NH4(k)+Zoo_2(k)*cff1
          LDet(k)=LDet(k)+Zoo_2(k)*cff2
        enddo
!
! (1) Coagulation of Phyt+SDet to LDet
!
!        do k=1,N
!          cff1=dtdays*mu_Agg*(SDet(k)+PHyt(k))
!          cff2=1./(1.+cff1)
!          Phyt(k)=Phyt(k)*cff2
!          SDet(k)=SDet(k)*cff2
!          LDet(k)=LDet(k)+(Phyt(k)+SDet(k))*cff1
!        enddo

!
! (1) SDet breakdown to NH4
!
        do k=1,N
          cff1=dtdays*mu_Sd_A
          SDet(k)=SDet(k)/(1.+cff1)
          NH4(k)=NH4(k)+SDet(k)*cff1
        enddo
!
! (1) LDet recycling to NH4 (remineralization)
!
        do k=1,N
          cff1=dtdays*mu_Ld_A
          LDet(k)=LDet(k)/(1.+cff1)
          NH4(k)=NH4(k)+LDet(k)*cff1
        enddo

!
! Vertical sinking: Vertical advection algorithm based on monotonic,
! continuous conservative parabolic splines.
!
        do k=1,N
      !    SB(k,1)=theta(k)*Phyt(k)*CN_Phyt*12.
          SB(k,1)=Phyt_2(k)
          SB(k,2)=SDet(k)
          SB(k,3)=LDet(k)
        enddo
     !   wSB(1)=wPhyt
        wSB(1)=wPhy2
        wSB(2)=wSDet
        wSB(3)=wLDet

        do iB=1,3

! Part (i): Construct parabolic splines: compute vertical derivatives
! of the fields SB. The derivatives are located at W-points;
! Neumann boundary conditions are assumed on top and bottom.
!
          dSB(0,iB)=0.
          FC(0)=0.
          cff6=6.
          do k=1,N-1
            cff=1./(2.*Hz(i,j,k+1)+Hz(i,j,k)*(2.-FC(k-1)))
            FC(k)=cff*Hz(i,j,k+1)
            dSB(k,iB)=cff*(cff6*(SB(k+1,iB)-SB(k,iB))
     &                             -Hz(i,j,k)*dSB(k-1,iB))
          enddo
          dSB(N,iB)=0.
          do k=N-1,1,-1     !<-- irreversible
            dSB(k,iB)=dSB(k,iB)-FC(k)*dSB(k+1,iB)
          enddo
!
! Part (ii): Convert dSB [which are now vertical derivatives
! of fields SB at the grid box interfaces] into field values
! at these interfaces, assuming parabolic profiles within each grid
! box. Restrict these values to lie between bounds determined from
! box-averaged values of grid boxes adjscent from above and below.
! (This restriction is part of PPM-like monotonization procedure.)
!
          cff=1./3.
          dSB(0,iB)=SB(1,iB) !-cff*Hz(1)*(dSB(0,iB)+0.5*dSB(1,iB))
          dSB(N,iB)=SB(N,iB) !+cff*Hz(N)*(dSB(N,iB)+0.5*dSB(N-1,iB))
          do k=2,N          !<-- irreversible
            dSB(k-1,iB)=SB(k,iB)
     &                 -cff*Hz(i,j,k)*(0.5*dSB(k,iB)+dSB(k-1,iB))
            dSB(k-1,iB)=max(dSB(k-1,iB),min(SB(k-1,iB),SB(k,iB)))
            dSB(k-1,iB)=min(dSB(k-1,iB),max(SB(k-1,iB),SB(k,iB)))
          enddo
!  
! Part (iii): Convert dSB into flux-integrated values,
! complete PPM flux limiting. This procedure starts from assigning
! Left and Right (aR,aL) values of the interpolating parabolae, then
! monotonicity conditions are checked and aL,aR are modified to fit.
! Overall, from this moment and further on it follows Colella--
! --Woodward, 1984 bombmaking code almost exactly.
!
          do k=1,N           !<-- irreversible
            FC(k)=dtdays/Hz(i,j,k)
            aR=dSB(k,iB)
            aL=dSB(k-1,iB)
            cff1=(aR-aL)*6.*(SB(k,iB)-.5*(aR+aL))
            cff2=(aR-aL)**2
            if ((aR-SB(k,iB))*(SB(k,iB)-aL).lt.0.) then
              aL=SB(k,iB)
              aR=SB(k,iB)
            elseif (cff1.gt.cff2) then
              aL=3.*SB(k,iB)-2.*aR
            elseif (cff1.lt.-cff2) then
              aR=3.*SB(k,iB)-2.*aL
            endif
            cu=wSB(iB)*FC(k)
            dSB(k-1,iB)=SB(k,iB)-(1.-cu)*(.5*(aR-aL)-(.5*(aR+aL)
     &                                   -SB(k,iB) )*(1.-2.*cu))
          enddo
          dSB(N,iB)=0.   ! Set no-flux boundary conditions at top.
!
! Apply fluxes:
!
          do k=1,N
            SB(k,iB)=SB(k,iB)+wSB(iB)*FC(k)*(dSB(k,iB)-dSB(k-1,iB))
          enddo
        enddo  ! <-- iB

        do k=1,N
!          theta(k)=SB(k,1)/(SB(k,2)*CN_Phyt*12.+1.E-20)
          Phyt_2(k) =SB(k,1)
          SDet(k)   =SB(k,2)
          LDet(k)   =SB(k,3)
        enddo
      ENDDO  ! <-- ITER
!
! Write back
!
          do k=1,N
           t(i,j,k,nnew,iNO3_)=min(t(i,j,k,nnew,iNO3_),0.) +NO3(k)
           t(i,j,k,nnew,iNH4_)=min(t(i,j,k,nnew,iNH4_),0.) +NH4(k)
           t(i,j,k,nnew,iPhy1)=min(t(i,j,k,nnew,iPhy1),0.) +Phyt_1(k)
           t(i,j,k,nnew,iPhy2)=min(t(i,j,k,nnew,iPhy2),0.) +Phyt_2(k)
           t(i,j,k,nnew,iZoo1)=min(t(i,j,k,nnew,iZoo1),0.) +Zoo_1(k)
           t(i,j,k,nnew,iZoo2)=min(t(i,j,k,nnew,iZoo2),0.) +Zoo_2(k)
           t(i,j,k,nnew,iDet1)=min(t(i,j,k,nnew,iDet1),0.) +SDet(k)
           t(i,j,k,nnew,iDet2)=min(t(i,j,k,nnew,iDet2),0.) +LDet(k)
        
           t(i,j,k,nnew,iNO3_)=max(t(i,j,k,nnew,iNO3_),0.) 
           t(i,j,k,nnew,iNH4_)=max(t(i,j,k,nnew,iNH4_),0.) 
           t(i,j,k,nnew,iPhy1)=max(t(i,j,k,nnew,iPhy1),0.) 
           t(i,j,k,nnew,iPhy2)=max(t(i,j,k,nnew,iPhy2),0.) 
           t(i,j,k,nnew,iZoo1)=max(t(i,j,k,nnew,iZoo1),0.) 
           t(i,j,k,nnew,iZoo2)=max(t(i,j,k,nnew,iZoo2),0.) 
           t(i,j,k,nnew,iDet1)=max(t(i,j,k,nnew,iDet1),0.) 
           t(i,j,k,nnew,iDet2)=max(t(i,j,k,nnew,iDet2),0.) 
          enddo
        enddo
      enddo

#else
      subroutine bio_N2P2Z2D2_empty ()
#endif
      return
      end

