!
! $Id: zoom.F,v 1.13 2005/10/26 15:18:53 pmarches Exp $
!
#include "cppdefs.h"
#ifdef AGRIF
!
! AGRIF related routines:
!
! Agrif_initworkspace      : initialize the array sizes.
! Agrif_initvalues         : perform the ROMS initial procedures 
!                            (grid, initial conditions, etc...)
! Agrif_transfer_floatsp2c : checks if a float must be transferred 
!                            from one grid to the child grid
! Agrif_laststep           : determines whether this time step is 
!                            the last one before a ROOT time step.
! Agrif_Invloc             : MPI related shit.
! Agrif_update             : Update in the case of 2-way nesting
!
! Agrif_u3dbc_interp_tile  : Perform the interpolations for u3dbc
!                            to get ubry_east or uclm.
! Agrif_v3dbc_interp_tile  : Perform the interpolations for v3dbc
!                            to get vbry_east or vclm.
! Agrif_t3dbc_interp_tile  : Perform the interpolations for t3dbc
!                            to get tbry_east or tclm.
! Agrif_u2dbc_interp_tile  : Perform the interpolations for u2dbc
!                            to get ubarbry_east or ubclm.
! Agrif_v2dbc_interp_tile  : Perform the interpolations for v2dbc
!                            to get vbarbry_east or vbclm.
! Agrif_zetabc_interp_tile : Perform the interpolations for zetabc
!                            to get zetabry_east or zetaclm.
! 
! Agrif_storebaro_tile     : Store the barotropic velocities for 
!                            the child boundary conditions.
! 
!
!====================================================================
!                   subroutine Agrif_initworkspace
!====================================================================
!
      subroutine Agrif_initworkspace()
      implicit none
# ifdef MPI
      integer ierr  
# endif          
# include "param.h"
# include "private_scratch.h"
# include "ncscrum.h"
# include "scalars.h"
      integer size_XI,size_ETA,se,sse, sz,ssz
# include "dynderivparam.h"
# ifdef MPI
      Call MPI_Setup(ierr)
# endif      
      return
      end
!
!====================================================================
!                   subroutine Agrif_initvalues
!====================================================================
!
      subroutine Agrif_initvalues()
      use AGRIF_UTIL
      implicit none
      integer tile, ierr, trd, subs
# ifdef SEDIMENT
     & , Agrif_lev_sedim
# endif
# include "param.h"
# include "private_scratch.h"
# include "ncscrum.h"
# include "scalars.h"
# include "grid.h"
# include "ocean2d.h"
# include "zoom.h"
      integer size_XI,size_ETA,se,sse, sz,ssz
# include "compute_tile_bounds.h"
# include "dynderivparam.h"
# ifdef SEDIMENT
      Agrif_lev_sedim=Agrif_Nb_Fine_Grids()
# endif
# ifdef MPI
      call MPI_Setup (ierr)
      if (ierr.ne.0) goto 100                            !--> ERROR
# endif

      call read_inp (ierr)           ! Read in tunable model
      if (ierr.ne.0) goto 100        ! parameters.
      call init_scalars (ierr)       ! Also initialize global
      if (ierr.ne.0) goto 100        ! scalar variables.

# ifdef SEDIMENT
      if (Agrif_Fixed().EQ.Agrif_lev_sedim) then
        call init_sediment           ! Read sediment initial values and
                                     ! parameters from sediment.in file
      endif
# endif

C$OMP PARALLEL DO PRIVATE(tile)
      do tile=0,NSUB_X*NSUB_E-1      ! Create parallel threads;
c        call start_timers()          ! start timers for each thread;
        call init_arrays (tile)      ! initialize (FIRST-TOUCH) model 
      enddo                          ! global arrays (most of them 
CR      write(*,*) '-11' MYID        ! are just set to to zero).

# ifdef ANA_GRID
C$OMP PARALLEL DO PRIVATE(tile)      ! Set horizontal curvilinear 
      do tile=0,NSUB_X*NSUB_E-1      ! grid and model bathymetry 
        call ana_grid (tile)         ! (analyticaly or read from 
      enddo                          ! GRID NetCDF file).
# else
                                     ! Also read Land/Sea mask
      call get_grid                  ! from GRID NetCDF file). 
      if (may_day_flag.ne.0) goto 99     !-->  EXIT
# endif
C$OMP PARALLEL DO PRIVATE(tile)
      do tile=0,NSUB_X*NSUB_E-1      ! Compute various metric
        call setup_grid1 (tile)      ! term combinations.
      enddo
CR      write(*,*) '-10' MYID
C$OMP PARALLEL DO PRIVATE(tile)
      do tile=0,NSUB_X*NSUB_E-1
        call setup_grid2 (tile)
      enddo
CR      write(*,*) ' -9' MYID
                                     ! Set up vertical S-coordinate
# ifdef SOLVE3D
      call set_scoord                ! and fast-time averaging  
      call set_weights               ! weights for coupling of 
                                     ! split-explicit baroropic mode.
# endif
CR      write(*,*) ' -8' MYID 
# ifdef SOLVE3D
C$OMP PARALLEL DO PRIVATE(tile)      ! Create three-dimensional
      do tile=0,NSUB_X*NSUB_E-1      ! S-coordinate system, which
        call set_depth (tile)        ! may be neded by ana_ninitial
      enddo                          ! (here it is assumed that free
CR      write(*,*) ' -7' MYID        ! surface zeta=0).
C$OMP PARALLEL DO PRIVATE(tile)      ! Create three-dimensional
      do tile=0,NSUB_X*NSUB_E-1      ! S-coordinate system, which
        call grid_stiffness (tile) 
      enddo                          ! (here it is assumed that free
# endif
# ifdef ANA_INITIAL
      if (nrrec.eq.0) then
C$OMP PARALLEL DO PRIVATE(tile)      ! Set initial conditions
        do tile=0,NSUB_X*NSUB_E-1    ! for primitive variables
          call ana_initial (tile)    ! (analytically or read
        enddo                        ! from initial conditions
      else                           ! NetCDF file).
# endif
        call get_initial
C$OMP PARALLEL DO PRIVATE(tile)      ! Set analytical initial 
        do tile=0,NSUB_X*NSUB_E-1    ! conditions for tracers
          call ana_initracer (tile)  ! if not found in initial
        enddo                        ! conditions NetCDF file). 
# ifdef ANA_INITIAL
      endif
# endif
CR      write(*,*) ' -6' MYID 
      if (may_day_flag.ne.0) goto 99     !-->  EXIT

# ifdef SOLVE3D
C$OMP PARALLEL DO PRIVATE(tile)      ! Create three-dimensional
      do tile=0,NSUB_X*NSUB_E-1      ! S-coordinate system: at this
        call set_depth (tile)        ! time free surface is set to
      enddo                          ! a non-zero field, either 
CR      write(*,*)  ' -5' MYID       ! analytically or from restart.
C$OMP PARALLEL DO PRIVATE(tile)
      do tile=0,NSUB_X*NSUB_E-1
        call set_HUV (tile)
      enddo
CR      write(*,*)  ' -4' MYID
C$OMP PARALLEL DO PRIVATE(tile)
      do tile=0,NSUB_X*NSUB_E-1
        call omega (tile)
        call rho_eos (tile)
      enddo
CR      write(*,*)  ' -3' MYID
# endif
!
! Create climatological environment:
!-------------------------------------------------------
! Set nudging coefficient for sea surface hight and tracer
!                 climatology;
! Set bottom sediment grain size [m] and density [kg/m^3]
!                 used in bottom boundary layer formulation;
! Set initial analytical tracer climatology fields;
! Set initial sea surface height climatology;
!
# if defined TNUDGING || defined ZNUDGING || defined SPONGE \
      || (defined BBL && defined ANA_BSEDIM)
C$OMP PARALLEL DO PRIVATE(tile)
      do tile=0,NSUB_X*NSUB_E-1
#  if defined TNUDGING || defined ZNUDGING || defined SPONGE
        call set_nudgcof_fine (tile)
#  endif
#  if defined BBL && defined ANA_BSEDIM
        if (Agrif_Fixed().EQ.Agrif_lev_sedim) then
          call ana_bsedim (tile)
        endif
#  endif
#  if defined SEDIMENT && defined ANA_SEDIMENT
        if (Agrif_Fixed().EQ.Agrif_lev_sedim) then
          call ana_sediment (tile)
        endif
#  endif
      enddo
# endif
CR      write(*,*) ' -2' MYID 
!
!  Read bottom sediment grain size [m] and density [kg/m^3] from
!           input NetCDF file;
!
# if defined BBL && !defined ANA_BSEDIM && !defined SEDIMENT
      if (Agrif_Fixed().EQ.Agrif_lev_sedim) then
          call get_bsedim
      endif
# endif
# if defined SEDIMENT && !defined ANA_SEDIMENT
      if (Agrif_Fixed().EQ.Agrif_lev_sedim) then
          call get_sediment
      endif
# endif
      call get_vbc
# if defined BBL && !defined ANA_WWAVE
      if (Agrif_Fixed().EQ.Agrif_lev_sedim) then
        call get_wwave
      endif
# endif /* BBL && !ANA_WWAVE*/
CR      write(*,*) ' -1' MYID 
      if (may_day_flag.ne.0) goto 99     !-->  EXIT

      if (ldefhis .and. wrthis(indxTime)) call wrt_his
CR      write(*,*) '  0' MYID
      if (may_day_flag.ne.0) goto 99     !-->  EXIT

 99   continue
 100  continue
 
      next_kstp=kstp
      time_start=time

      U2DTimeindex = -1
      V2DTimeindex = -1
      ZetaTimeindex = -1
# ifdef SOLVE3D
      Ttimeindex = -1
      Utimeindex = -1
      Vtimeindex = -1
# endif

# ifdef AGRIF_STORE_BAROT_CHILD
      Agrif_Parent_ic1=Agrif_Ix()
      Agrif_Parent_ic2=Agrif_Ix()+Lm/Agrif_IRhox()
      Agrif_Parent_jc1=Agrif_Iy()
      Agrif_Parent_jc2=Agrif_Iy()+Mm/Agrif_IRhoy()
# endif
      return
      end
!
!====================================================================
!                   subroutine Agrif_transfer_floatsp2c
!====================================================================
!
# if defined FLOATS || defined STATIONS
        subroutine Agrif_transfer_floatsp2c(rank,xfloat,yfloat,delta)
        use Agrif_Util
!----------------------------------------------
! checks if a float must be transferred from one grid to the child grid
! margin is the sum of the distance between rho and velocity points (0.5)
! plus a safe value to ensure that the float can stay inside the child
! at least coeffreft time steps.
!----------------------------------------------
        type(Agrif_grid), Pointer :: childgrid
        type(Agrif_pgrid), Pointer :: parcours

        real xfloat,yfloat,margin,delta,tmp1,tmp2
        integer rank,timeindex

        parcours=>Agrif_Curgrid%child_grids
        do while (associated(parcours))
          childgrid=>parcours%gr
          margin=0.5+delta
          tmp1=childgrid%spacerefx*(xfloat+0.5-childgrid%ix)+0.5
          if ((tmp1 .ge. margin).and.
     &        (tmp1 .le. (childgrid%nbx+1-margin))) then
            tmp2=childgrid%spacerefy*(yfloat+0.5-childgrid%iy)+0.5
            if ((tmp2 .ge. margin).and.
     &          (tmp2 .le. (childgrid%nby+1-margin))) then
              rank=childgrid%fixedrank
              xfloat=tmp1
              yfloat=tmp2
              exit
            endif
          endif
          parcours=>parcours%next
        enddo
        end subroutine Agrif_transfer_floatsp2c
!
!====================================================================
!                   subroutine Agrif_Invloc
!====================================================================
!
        subroutine Agrif_laststep(logic)
        use Agrif_Util

!----------------------------------------------
! determines whether this time step is the last one before
! a ROOT time step.
!----------------------------------------------
        Implicit none
        logical logic
        type(Agrif_grid), Pointer :: parcours
        integer tmp, tmp2,rhot,stepmod

        logic=.false.
        tmp=0
        tmp2=1
        parcours=>Agrif_Curgrid

        Do While (associated(parcours%parent))
          rhot=1
          rhot=max(rhot,parcours%timerefx,parcours%timerefy)
          stepmod=mod(parcours%ngridstep,int(rhot))
          tmp2=tmp2*rhot
          tmp=stepmod+rhot*tmp
          parcours=>parcours%parent
        End Do
        tmp2=tmp2-1
        if (tmp2 .eq. tmp) logic=.true.
        return
        end subroutine

# endif /* FLOATS || STATIONS */
!
!====================================================================
!                   subroutine Agrif_Invloc
!====================================================================
!
# if defined MPI
        Subroutine Agrif_Invloc(indloc,proc,dir,indglob)
        implicit none
        integer indloc, proc, dir, indglob
# include "param.h"        
        If (dir == 1) Then
           indglob = indloc + iminmpi-1          
        Else If (dir == 2) Then
           indglob = indloc + jminmpi-1
        Else 
           indglob = indloc
        End If
        Return
        End
# endif
!
!====================================================================
!                   subroutine Agrif_update
!====================================================================
!
# if defined SOLVE3D && defined AGRIF_2WAY
#  undef UPDATE_UV
      subroutine Agrif_update
!
! Update in case of 2-way nesting
!
      use Agrif_Util
      implicit none
#  include "param.h"
#  include "grid.h"
#  include "ocean3d.h"
#  include "ocean2d.h"
#  include "scalars.h"
      integer NNO,NNOX,NNOY,ich,jch,k,itrc,ipr,jpr,nnewpr
      integer irmin,irmax,jrmin,jrmax,iumin,iumax,jvmin,jvmax
      integer ii,jj,iint,jint
      real eps,cff,fxc,surfc 
      parameter (eps=1.E-20)
!
      nnewpr=Agrif_Parent_nnew
 
! Define NNO, the number of points from the boundary
! not included in the update
!
      NNO=2
!
      NNOX=NNO*Agrif_IRhox()
      NNOY=NNO*Agrif_IRhoy()
!
      irmin=1+int(Agrif_IRhox()/2)+NNOX
      irmax=Lm-int(Agrif_IRhox()/2)-NNOX
      jrmin=1+int(Agrif_IRhoy()/2)+NNOY
      jrmax=Mm-int(Agrif_IRhoy()/2)-NNOY
      iumin=1+Agrif_IRhox()+NNOX
      iumax=1+Lm-Agrif_IRhox()-NNOX
      jvmin=1+Agrif_IRhoy()+NNOY
      jvmax=1+Mm-Agrif_IRhoy()-NNOY
!
      iint=int(Agrif_IRhox()/2)
      jint=int(Agrif_IRhoy()/2)
!
      do jch=jrmin,jrmax,Agrif_IRhoy()
        do ich=irmin,irmax,Agrif_IRhox()
          if (rmask(ich,jch).eq.1) then
            ipr=Agrif_Ix()+int((float(ich)-Agrif_rhox()/2.-0.5)
     &                          /Agrif_rhox())
            jpr=Agrif_Iy()+int((float(jch)-Agrif_rhoy()/2.-0.5)
     &                           /Agrif_rhoy())
            do itrc=1,NT
              do k=1,N
                Agrif_Parent_t(ipr,jpr,k,nnewpr,itrc)=
     &                       t(ich,jch,k,nnew,itrc)
              enddo
            enddo
          endif
        enddo
      enddo
#  ifdef UPDATE_UV
      do jch=jrmin,jrmax,Agrif_IRhoy()
        do ich=iumin,iumax,Agrif_IRhox()
          ipr=Agrif_Ix()+int((float(ich)-1.)/Agrif_rhox())
          jpr=Agrif_Iy()+int((float(jch)-Agrif_rhoy()/2.-0.5)
     &                           /Agrif_rhoy())    
          do k=1,N
            fxc=0.
            surfc=0.
            do jj=-jint,jint
              cff=umask(ich,jch+jj)
     &            *0.5*(Hz(ich,jch+jj,k)+Hz(ich-1,jch+jj,k))
     &            *on_u(ich,jch+jj)
              fxc=fxc+cff*u(ich,jch+jj,k,nnew)
              surfc=surfc+cff
            enddo
            surfc=max(surfc,eps)
            Agrif_Parent_u(ipr,jpr,k,nnewpr)=fxc/surfc
          enddo
        enddo
      enddo
      do jch=jvmin,jvmax,Agrif_IRhoy()
        do ich=irmin,irmax,Agrif_IRhox()
          ipr=Agrif_Ix()+int((float(ich)-Agrif_rhox()/2.-0.5)
     &                          /Agrif_rhox())
          jpr=Agrif_Iy()+int((float(jch)-1.)/Agrif_rhoy())
          do k=1,N
            fxc=0.
            surfc=0.
            do ii=-iint,iint
              cff=vmask(ich+ii,jch)
     &            *0.5*(Hz(ich+ii,jch,k)+Hz(ich+ii,jch-1,k))
     &            *om_v(ich+ii,jch)
              fxc=fxc+cff*v(ich+ii,jch,k,nnew)
              surfc=surfc+cff
            enddo
            surfc=max(surfc,eps)
            Agrif_Parent_v(ipr,jpr,k,nnewpr)=fxc/surfc
          enddo
        enddo
      enddo
#  endif
      return
      end
# endif /* AGRIF_2WAY */
!
!====================================================================
!                   subroutine Agrif_u3dbc_interp_tile
!====================================================================
!
# if defined SOLVE3D
      subroutine Agrif_u3dbc_interp_tile(Istr,Iend,Jstr,Jend)
      use AGRIF_Util
      implicit none
#  include "param.h"
#  include "boundary.h"  
#  include "climat.h"
#  include "grid.h"
#  include "scalars.h"
#  include "zoom.h"
      integer Istr,Iend,Jstr,Jend, i,j,k
      real, dimension(:,:,:,:), allocatable :: uext
      integer itrcind
      real tinterp
      logical ptinterp
!
#  include "compute_auxiliary_bounds.h"
!
#  ifdef MPI
#   define LOCALLM Lmmpi
#   define LOCALMM Mmmpi
#  else
#   define LOCALLM Lm
#   define LOCALMM Mm
#  endif
C$OMP MASTER
      if (Utimeindex.LT.Agrif_Parent_Nb_Step()) then
        Allocate(uext(GLOBAL_2D_ARRAY,N,3))
        ptinterp=.TRUE.
        tinterp=1. 
      
        Agrif_UseSpecialvalue=.true.
        Agrif_Specialvalue=0.
        uext=0.   
        call Agrif_Bc_u(uext(START_2D_ARRAY,1,1),tinterp,
     &                                           ptinterp)
        Agrif_UseSpecialValue=.false.

        do itrcind = 1,3
          do k=1,N
#  ifdef AGRIF_OBC_SOUTH
#   ifdef MPI        
            if (.not.SOUTH_INTER) then
#   endif            
              do i=0,LOCALLM+1
                U_south(i,0:0,k,itrcind)=
     &             uext(i,0:0,k,itrcind)
              enddo
#   ifdef MPI           
            endif          
#   endif            
#  endif            
#  ifdef AGRIF_OBC_NORTH
#   ifdef MPI        
            if (.not.NORTH_INTER) then  
#   endif
              do i=0,LOCALLM+1
                U_north(i,LOCALMM+1:LOCALMM+1,k,itrcind)=
     &             uext(i,LOCALMM+1:LOCALMM+1,k,itrcind)
              enddo          
#   ifdef MPI                    
            endif  
#   endif            
#  endif            
#  ifdef AGRIF_OBC_WEST
#   ifdef MPI        
            if (.not.WEST_INTER) then
#   endif                        
              do j=0,LOCALMM+1
                U_west(1:1,j,k,itrcind)=
     &            uext(1:1,j,k,itrcind)
              enddo
#   ifdef MPI            
            endif
#   endif            
#  endif            
#  ifdef AGRIF_OBC_EAST
#   ifdef MPI        
            if (.not.EAST_INTER) then
#   endif
              do j=0,LOCALMM+1
                U_east(LOCALLM+1:LOCALLM+1,j,k,itrcind)=
     &            uext(LOCALLM+1:LOCALLM+1,j,k,itrcind)
              enddo          
#   ifdef MPI               
            endif  
#   endif                      	      
#  endif                      	      
          enddo
        enddo
        deallocate(uext)
        Utimeindex = Agrif_Parent_Nb_Step()
      endif
C$OMP END MASTER
C$OMP BARRIER
      tinterp=float(Agrif_NBSTEPINT()+1)/Agrif_rhot()
      if (nnew.eq.3) then
        tinterp=tinterp-1./(2.*Agrif_rhot())
      endif
      if (Agrif_NB_STEP().lt.int(Agrif_rhot())) tinterp=1.

#  ifdef AGRIF_OBC_SOUTH
      if (SOUTHERN_EDGE) then
        do k=1,N      
          do i=IstrR,IendR
#   if defined M3_FRC_BRY     
            ubry_south(i,k)=
#   else
            uclm(i,Jstr-1,k)=
#   endif
     &    (tinterp*U_south(i,Jstr-1,k,Agrif_Parent_nnew)
     &    +(1.-tinterp)*U_south(i,Jstr-1,k,Agrif_Parent_nstp))
#   ifdef MASKING     
     &    *umask(i,Jstr-1)
#   endif
          enddo
        enddo
      endif
#  endif
#  ifdef AGRIF_OBC_NORTH
      if (NORTHERN_EDGE) then      
        do k=1,N      
          do i=IstrR,IendR
#   if defined M3_FRC_BRY     
            ubry_north(i,k)=
#   else   
            uclm(i,Jend+1,k)=
#   endif
     &    (tinterp*U_north(i,Jend+1,k,Agrif_Parent_nnew)
     &    +(1.-tinterp)*U_north(i,Jend+1,k,Agrif_Parent_nstp))
#   ifdef MASKING     
     &    *umask(i,Jend+1)
#   endif     
          enddo
        enddo
      endif      
#  endif
#  ifdef AGRIF_OBC_WEST
      if (WESTERN_EDGE) then      
        do k=1,N      
          do j=JstrR,JendR
#   if defined M3_FRC_BRY     
            ubry_west(j,k)=
#   else     
            uclm(Istr,j,k)=
#   endif
     &    (tinterp*U_west(Istr,j,k,Agrif_Parent_nnew)
     &    +(1.-tinterp)*U_west(Istr,j,k,Agrif_Parent_nstp))
#   ifdef MASKING     
     &    *umask(Istr,j)
#   endif     
          enddo
        enddo
      endif      
#  endif
#  ifdef AGRIF_OBC_EAST
      if (EASTERN_EDGE) then      
        do k=1,N      
          do j=JstrR,JendR
#   if defined M3_FRC_BRY  
            ubry_east(j,k)=
#   else
            uclm(Iend+1,j,k)=
#   endif
     &    (tinterp*U_east(Iend+1,j,k,Agrif_Parent_nnew)
     &    +(1.-tinterp)*U_east(Iend+1,j,k,Agrif_Parent_nstp))
#   ifdef MASKING     
     &    *umask(Iend+1,j)
#   endif     
          enddo
        enddo
      endif
#  endif     
      return
      end
# endif /* SOLVE3D */
!
!====================================================================
!                   subroutine Agrif_v3dbc_interp_tile
!====================================================================
!
# if defined SOLVE3D
      subroutine Agrif_v3dbc_interp_tile(Istr,Iend,Jstr,Jend)
      use AGRIF_Util
      implicit none
#  include "param.h"
#  include "boundary.h"  
#  include "climat.h"
#  include "grid.h"
#  include "scalars.h"
#  include "zoom.h"
      integer Istr,Iend,Jstr,Jend, i,j,k
      real, dimension(:,:,:,:), allocatable :: vext
      integer itrcind
      real tinterp
      logical ptinterp
!
#  include "compute_auxiliary_bounds.h"
!
#  ifdef MPI
#   define LOCALLM Lmmpi
#   define LOCALMM Mmmpi
#  else
#   define LOCALLM Lm
#   define LOCALMM Mm
#  endif
C$OMP MASTER
      if (Vtimeindex.LT.AGRIF_Parent_Nb_Step()) then
        allocate(vext(GLOBAL_2D_ARRAY,N,3))
        ptinterp=.TRUE.
        tinterp=1.

        AGRIF_UseSpecialvalue=.true.
        AGRIF_Specialvalue=0.
        vext=0.   
        call AGRIF_Bc_v(vext(START_2D_ARRAY,1,1),tinterp,
     &                                           ptinterp)
        AGRIF_UseSpecialValue=.false.

        do itrcind = 1,3
          do k=1,N
#  ifdef AGRIF_OBC_SOUTH
#   ifdef MPI  
            if (.not.SOUTH_INTER) then
#   endif             
              do i=0,LOCALLM+1
                V_south(i,1:1,k,itrcind)=
     &             vext(i,1:1,k,itrcind)
              enddo
#   ifdef MPI           
            endif          
#   endif
#  endif
#  ifdef AGRIF_OBC_NORTH
#   ifdef MPI  
            if (.not.NORTH_INTER) then  
#   endif
              do i=0,LOCALLM+1
                V_north(i,LOCALMM+1:LOCALMM+1,k,itrcind)=
     &             vext(i,LOCALMM+1:LOCALMM+1,k,itrcind)
              enddo          
#   ifdef MPI                    
            endif  
#   endif
#  endif
#  ifdef AGRIF_OBC_WEST
#   ifdef MPI           
            if (.not.WEST_INTER) then
#   endif                            
              do j=0,LOCALMM+1
                V_west(0:0,j,k,itrcind)=
     &            vext(0:0,j,k,itrcind)
              enddo
#   ifdef MPI            
            endif
#   endif
#  endif
#  ifdef AGRIF_OBC_EAST
#   ifdef MPI           
            if (.not.EAST_INTER) then
#   endif
              do j=0,LOCALMM+1
                V_east(LOCALLM+1:LOCALLM+1,j,k,itrcind)=
     &            vext(LOCALLM+1:LOCALLM+1,j,k,itrcind)
              enddo          
#   ifdef MPI               
            endif  
#   endif                         	      
#  endif                         	      
         enddo
        enddo
        deallocate(vext)
        Vtimeindex = AGRIF_Parent_Nb_Step() 
      endif
C$OMP END MASTER
C$OMP BARRIER
      tinterp=float(AGRIF_NBSTEPINT()+1)/AGRIF_rhot()
      if (nnew.eq.3) then
        tinterp=tinterp-1./(2.*AGRIF_rhot())
      endif
      if (AGRIF_NB_STEP().lt.int(AGRIF_rhot())) tinterp=1.

#  ifdef AGRIF_OBC_SOUTH
      if (SOUTHERN_EDGE) then
        do k=1,N      
          do i=IstrR,IendR
#   if defined M3_FRC_BRY   
            vbry_south(i,k)=
#   else
            vclm(i,Jstr,k)=
#   endif
     &    (tinterp*V_south(i,Jstr,k,AGRIF_Parent_nnew)
     &    +(1.-tinterp)*V_south(i,Jstr,k,AGRIF_Parent_nstp))
#   ifdef MASKING     
     &    *vmask(i,Jstr)
#   endif
          enddo
        enddo
      endif
#  endif
#  ifdef AGRIF_OBC_NORTH
      if (NORTHERN_EDGE) then      
        do k=1,N      
          do i=IstrR,IendR
#   if defined M3_FRC_BRY
            vbry_north(i,k)=
#   else
            vclm(i,Jend+1,k)=
#   endif
     &    (tinterp*V_north(i,Jend+1,k,AGRIF_Parent_nnew)
     &    +(1.-tinterp)*V_north(i,Jend+1,k,AGRIF_Parent_nstp))
#   ifdef MASKING     
     &    *vmask(i,Jend+1)
#   endif     
          enddo
        enddo
      endif      
#  endif
#  ifdef AGRIF_OBC_WEST
      if (WESTERN_EDGE) then      
        do k=1,N      
          do j=JstrR,JendR
#   if defined M3_FRC_BRY
            vbry_west(j,k)=
#   else
            vclm(Istr-1,j,k)=
#   endif
     &    (tinterp*V_west(Istr-1,j,k,AGRIF_Parent_nnew)
     &    +(1.-tinterp)*V_west(Istr-1,j,k,AGRIF_Parent_nstp))
#   ifdef MASKING     
     &    *vmask(Istr-1,j)
#   endif     
          enddo
        enddo
      endif      
#  endif
#  ifdef AGRIF_OBC_EAST
      if (EASTERN_EDGE) then      
        do k=1,N      
          do j=JstrR,JendR
#   if defined M3_FRC_BRY  
            vbry_east(j,k)=
#   else
            vclm(Iend+1,j,k)=
#   endif
     &    (tinterp*V_east(Iend+1,j,k,AGRIF_Parent_nnew)
     &    +(1.-tinterp)*V_east(Iend+1,j,k,AGRIF_Parent_nstp))
#   ifdef MASKING     
     &    *vmask(Iend+1,j)
#   endif     
          enddo
        enddo
      endif
#  endif     
      return
      end
# endif /* SOLVE 3D */
!
!====================================================================
!                   subroutine Agrif_t3dbc_interp_tile
!====================================================================
!
# if defined SOLVE3D
      subroutine Agrif_t3dbc_interp_tile(Istr,Iend,Jstr,Jend,itrc)
      use AGRIF_Util
      implicit none
#  include "param.h"
#  include "boundary.h"  
#  include "climat.h"
#  include "grid.h"
#  include "scalars.h"
#  include "zoom.h"
      integer Istr,Iend,Jstr,Jend,itrc,i,j,k
      real,dimension(:,:,:,:,:),allocatable :: text
      integer itrcind, ntind
      real tinterp
      logical ptinterp
!
#  include "compute_auxiliary_bounds.h"
!
#  ifdef MPI
#   define LOCALLM Lmmpi
#   define LOCALMM Mmmpi
#  else
#   define LOCALLM Lm
#   define LOCALMM Mm
#  endif
C$OMP MASTER
      if (Ttimeindex.LT.Agrif_Parent_Nb_Step()) then
        allocate(text(GLOBAL_2D_ARRAY,N,3,NT))
        ptinterp = .true.     
        tinterp=1.

        Agrif_UseSpecialvalue=.true.
        Agrif_Specialvalue=0.
        text=0. 
        Call Agrif_Bc_t(text(START_2D_ARRAY,1,1,1),tinterp,
     &           ptinterp) 
        Agrif_UseSpecialValue=.false.

        do ntind = 1,NT	  
          do itrcind = 1,3
            do k=1,N
#  ifdef AGRIF_OBC_SOUTH
#   ifdef MPI        
              if (.not.SOUTH_INTER) Then
#   endif           
                do i=0,LOCALLM+1
                  T_south(i,0:1,k,itrcind,ntind)=
     &                text(i,0:1,k,itrcind,ntind)
                enddo
#   ifdef MPI           
              endif          
#   endif               
#  endif               
#  ifdef AGRIF_OBC_NORTH
#   ifdef MPI                    
              if (.not.NORTH_INTER) then  
#   endif 
                do i=0,LOCALLM+1
                  T_north(i,LOCALMM:LOCALMM+1,k,itrcind,ntind)=
     &                text(i,LOCALMM:LOCALMM+1,k,itrcind,ntind)
                enddo          
#   ifdef MPI                    
              endif  
#   endif               
#  endif               
#  ifdef AGRIF_OBC_WEST
#   ifdef MPI                    
              if (.not.WEST_INTER) then
#   endif               
                do j=0,LOCALMM+1
                  T_west(0:1,j,k,itrcind,ntind)=
     &                text(0:1,j,k,itrcind,ntind)
                enddo
#   ifdef MPI            
              endif
#   endif               
#  endif               
#  ifdef AGRIF_OBC_EAST
#   ifdef MPI                    
              if (.not.EAST_INTER) then
#   endif
                do j=0,LOCALMM+1
                  T_east(LOCALLM:LOCALLM+1,j,k,itrcind,ntind)=
     &                text(LOCALLM:LOCALLM+1,j,k,itrcind,ntind)
                enddo          
#   ifdef MPI               
              endif  
#   endif                     	      
#  endif                     	      
            enddo
          enddo
        enddo            
        deallocate(text)
        Ttimeindex = Agrif_Parent_Nb_Step()        
      endif
C$OMP END MASTER
C$OMP BARRIER
      tinterp=float(Agrif_NBSTEPINT()+1)/Agrif_rhot()
      if (nnew.eq.3) then
        tinterp=tinterp-1./(2.*Agrif_rhot())
      endif
      if (Agrif_NB_STEP().lt.int(Agrif_rhot())) tinterp=1.
!      
#  ifdef AGRIF_OBC_SOUTH
      if (SOUTHERN_EDGE) then
        do k=1,N      
          do i=IstrR,IendR
#   ifdef T_FRC_BRY
            tbry_south(i,k,itrc)=
#   else
            tclm(i,Jstr-1,k,itrc)=
#   endif
     &    (tinterp*T_south(i,Jstr-1,k,Agrif_Parent_nnew,itrc)
     &    +(1.-tinterp)*T_south(i,Jstr-1,k,Agrif_Parent_nstp,itrc))
#   ifdef MASKING     
     &    *rmask(i,Jstr-1)
#   endif
          enddo
        enddo
      endif
#  endif
#  ifdef AGRIF_OBC_NORTH
      if (NORTHERN_EDGE) then      
        do k=1,N      
          do i=IstrR,IendR
#   ifdef T_FRC_BRY
            tbry_north(i,k,itrc)=
#   else
            tclm(i,Jend+1,k,itrc)=
#   endif
     &    (tinterp*T_north(i,Jend+1,k,Agrif_Parent_nnew,itrc)
     &    +(1.-tinterp)*T_north(i,Jend+1,k,Agrif_Parent_nstp,itrc))
#   ifdef MASKING     
     &    *rmask(i,Jend+1)
#   endif     
          enddo
        enddo
      endif      
#  endif
#  ifdef AGRIF_OBC_WEST
      if (WESTERN_EDGE) then      
        do k=1,N      
          do j=JstrR,JendR
#   ifdef T_FRC_BRY
            tbry_west(j,k,itrc)=
#   else
            tclm(Istr-1,j,k,itrc)=
#   endif
     &    (tinterp*T_west(Istr-1,j,k,Agrif_Parent_nnew,itrc)
     &    +(1.-tinterp)*T_west(Istr-1,j,k,Agrif_Parent_nstp,itrc))
#   ifdef MASKING     
     &    *rmask(Istr-1,j)
#   endif
          enddo
        enddo
      endif      
#  endif
#  ifdef AGRIF_OBC_EAST
      if (EASTERN_EDGE) then      
        do k=1,N      
          do j=JstrR,JendR
#   ifdef T_FRC_BRY
            tbry_east(j,k,itrc)= 
#   else
            tclm(Iend+1,j,k,itrc)=
#   endif
     &    (tinterp*T_east(Iend+1,j,k,Agrif_Parent_nnew,itrc)
     &    +(1.-tinterp)*T_east(Iend+1,j,k,Agrif_Parent_nstp,itrc))
#   ifdef MASKING     
     &    *rmask(Iend+1,j)
#   endif     
          enddo
        enddo
      endif
#  endif     
      return
      end
# endif /* SOLVE 3D */
!
!====================================================================
!                   subroutine Agrif_u2dbc_interp_tile
!====================================================================
!
      subroutine Agrif_u2dbc_interp_tile(Istr,Iend,Jstr,Jend,kindx)
      use AGRIF_Util
      implicit none
# include "param.h"
# include "grid.h"
# include "ocean2d.h"
# include "scalars.h"
# include "climat.h"
# include "boundary.h"
# include "zoom.h"
      integer Istr,Iend,Jstr,Jend, i,j,kindx
# ifdef AGRIF_LOCAL_VOLCONS               
      integer irmin,irmax,jrmin,jrmax,ipu,jpu,iint,jint,jj
      real fxp,fxc,surfc
# endif      
# ifdef AGRIF_STORE_BAROT_CHILD      
      integer i1,i2,j1,j2,k1,k2
      real xpr,ypr,tpr,p1,p2,q1,q2
# else
      real t1,t2,t3,t4,t5,t6
      real,Dimension(:,:), Allocatable :: duinterp
      real, Dimension(:,:,:), Allocatable :: duinterp2
      real tinterp
      logical ptinterp
# endif      
# ifdef MPI
#  define LOCALLM Lmmpi
#  define LOCALMM Mmmpi
# else
#  define LOCALLM Lm
#  define LOCALMM Mm
# endif   
!          
# include "compute_auxiliary_bounds.h"
!
C$OMP MASTER
# ifdef AGRIF_STORE_BAROT_CHILD
!
! Get the temporal interpolant
!     
      tpr=float(Agrif_NbStepInt()*ndtfast+iif)/Agrif_rhot()
      k1=int(tpr)
      k2=k1+1
      q2=tpr-float(k1)
      q1=1.0-q2
#  ifdef AGRIF_OBC_WEST
#   ifdef MPI
      if (.not.WEST_INTER) then
#   endif
        do j=0,LOCALMM+1
          ypr=float(Agrif_Iy())+(float(j)-Agrif_rhoy()/2.-0.5)
     &                           /Agrif_rhoy()

          j1=int(ypr)
          j2=j1+1
#   ifdef MASKING
          p2=max(1.-Agrif_Parent_umask(Agrif_Parent_ic1,j1),
     &           ypr-float(j1))
          p1=max(1.-Agrif_Parent_umask(Agrif_Parent_ic1,j2),1.-p2)
#   else
          p2=ypr-float(j1)
          p1=1.-p2
#   endif
          DU_west(j)=(q1*(p1*Agrif_Parent_ubar_westbound(j1,k1)
     &                   +p2*Agrif_Parent_ubar_westbound(j2,k1))
     &               +q2*(p1*Agrif_Parent_ubar_westbound(j1,k2)
     &                   +p2*Agrif_Parent_ubar_westbound(j2,k2)))
#   ifdef AGRIF_FLUX_BC
     &              /Agrif_rhoy()
#   endif 
        enddo
#   ifdef MPI            
      endif
#   endif 
#  endif 
#  ifdef AGRIF_OBC_EAST
#   ifdef MPI            
      if (.not.EAST_INTER) then
#   endif 
        do j=0,LOCALMM+1
          ypr=float(Agrif_Iy())+(float(j)-Agrif_rhoy()/2.-0.5)
     &                           /Agrif_rhoy()
          j1=int(ypr)
          j2=j1+1
#   ifdef MASKING
          p2=max(1.-Agrif_Parent_umask(Agrif_Parent_ic2,j1),
     &           ypr-float(j1))
          p1=max(1.-Agrif_Parent_umask(Agrif_Parent_ic2,j2),1.-p2)
#   else
          p2=ypr-float(j1)
          p1=1.-p2
#   endif
          DU_east(j)=(q1*(p1*Agrif_Parent_ubar_eastbound(j1,k1)
     &                   +p2*Agrif_Parent_ubar_eastbound(j2,k1))
     &               +q2*(p1*Agrif_Parent_ubar_eastbound(j1,k2)
     &                   +p2*Agrif_Parent_ubar_eastbound(j2,k2)))
#   ifdef AGRIF_FLUX_BC
     &              /Agrif_rhoy()
#   endif 
        enddo
#   ifdef MPI                       
      endif
#   endif 
#  endif 
#  ifdef AGRIF_OBC_SOUTH
#   ifdef MPI            
      if (.not.SOUTH_INTER) Then
#   endif
        do i=0,LOCALLM+1
          xpr=float(Agrif_Ix())+(float(i)-1.)/Agrif_rhox()
          i1=int(xpr)
          i2=i1+1
#   ifdef MASKING
          p2=max(1.-max(Agrif_Parent_umask(i1,Agrif_Parent_jc1-1),
     &                  Agrif_Parent_umask(i1,Agrif_Parent_jc1)),
     &           xpr-float(i1))
          p1=max(1.-max(Agrif_Parent_umask(i2,Agrif_Parent_jc1-1),
     &                  Agrif_Parent_umask(i2,Agrif_Parent_jc1)),
     &           1.-p2)
#   else
          p2=xpr-float(i1)
          p1=1.-p2
#   endif
          DU_south(i)=(q1*(p1*Agrif_Parent_ubar_southbound(i1,k1)
     &                    +p2*Agrif_Parent_ubar_southbound(i2,k1))
     &                +q2*(p1*Agrif_Parent_ubar_southbound(i1,k2)
     &                    +p2*Agrif_Parent_ubar_southbound(i2,k2)))
#   ifdef AGRIF_FLUX_BC
     &              /Agrif_rhox()
#   endif 
        enddo          
#   ifdef MPI           
      endif          
#   endif 
#  endif 
#  ifdef AGRIF_OBC_NORTH
#   ifdef MPI            
      if (.not.NORTH_INTER) then  
#   endif
        do i=0,LOCALLM+1
          xpr=float(Agrif_Ix())+(float(i)-1.)/Agrif_rhox()
          i1=int(xpr)
          i2=i1+1
#   ifdef MASKING
          p2=max(1.-max(Agrif_Parent_umask(i1,Agrif_Parent_jc2-1),
     &                  Agrif_Parent_umask(i1,Agrif_Parent_jc2)),
     &           xpr-float(i1))
          p1=max(1.-max(Agrif_Parent_umask(i2,Agrif_Parent_jc2-1),
     &                  Agrif_Parent_umask(i2,Agrif_Parent_jc2)),
     &           1.-p2)
#   else
          p2=xpr-float(i1)
          p1=1.-p2
#   endif
          DU_north(i)=(q1*(p1*Agrif_Parent_ubar_northbound(i1,k1)
     &                    +p2*Agrif_Parent_ubar_northbound(i2,k1))
     &                +q2*(p1*Agrif_Parent_ubar_northbound(i1,k2)
     &                    +p2*Agrif_Parent_ubar_northbound(i2,k2)))
#   ifdef AGRIF_FLUX_BC
     &              /Agrif_rhox()
#   endif 
        enddo          
#   ifdef MPI               
      endif  
#   endif 
#  endif 
# else   
      if (U2DTimeindex.LT.Agrif_Parent_Nb_Step()) then
        ptinterp = .TRUE.
        tinterp=1.
        Allocate(duinterp(GLOBAL_2D_ARRAY))
        Allocate(duinterp2(GLOBAL_2D_ARRAY,2))
        Agrif_UseSpecialValue = .true.
        Agrif_SpecialValue = 0.
        duinterp=0.
        duinterp2=0. 
        call Agrif_Bc_DU_avg2(duinterp,tinterp,ptinterp)
        call Agrif_Bc_DU_avg1(duinterp2,tinterp,ptinterp)
        Agrif_UseSpecialvalue=.false. 
#  ifdef AGRIF_OBC_WEST
#   ifdef MPI           
        if (.not.WEST_INTER) then
#   endif
          do j=0,LOCALMM+1
            DU_west1(j,2)=duinterp(1,j)
            DU_west1(j,3)=duinterp2(1,j,Agrif_parent_nnew)
            DU_west1(j,1)=duinterp2(1,j,Agrif_parent_nstp)
          enddo          
#   ifdef MPI            
        endif
#   endif 
#  endif 
#  ifdef AGRIF_OBC_EAST
#   ifdef MPI            
        if (.not.EAST_INTER) then
#   endif 
          do j=0,LOCALMM+1
            DU_east1(j,2)=duinterp(LOCALLM+1,j)
            DU_east1(j,3)=duinterp2(LOCALLM+1,j,Agrif_parent_nnew)
            DU_east1(j,1)=duinterp2(LOCALLM+1,j,Agrif_parent_nstp)
          enddo
#   ifdef MPI                       
        endif
#   endif 
#  endif 
#  ifdef AGRIF_OBC_SOUTH
#   ifdef MPI            
        if (.not.SOUTH_INTER) then
#   endif
          do i=0,LOCALLM+1
            DU_south1(i,2)=duinterp(i,0)
            DU_south1(i,3)=duinterp2(i,0,Agrif_parent_nnew)
            DU_south1(i,1)=duinterp2(i,0,Agrif_parent_nstp)
          enddo          
#   ifdef MPI           
        endif          
#   endif 
#  endif 
#  ifdef AGRIF_OBC_NORTH
#   ifdef MPI            
        if (.not.NORTH_INTER) then  
#   endif
          do i=0,LOCALLM+1
            DU_north1(i,2)=duinterp(i,LOCALMM+1)
            DU_north1(i,3)=duinterp2(i,LOCALMM+1,Agrif_parent_nnew)
            DU_north1(i,1)=duinterp2(i,LOCALMM+1,Agrif_parent_nstp)
          enddo          
#   ifdef MPI               
        endif  
#   endif                    
#  endif                    
        deallocate(duinterp)
        deallocate(duinterp2)
        U2DTimeIndex = Agrif_Parent_Nb_Step()
      endif
!
! We seek a second order polynomial that equals
! DU_avg1(nstp) at time n
! DU_avg1(nnew) at time n+1
! and has its integral over [n,n+1] equal to DU_avg2
! this polynomial is given by
! 3(u1+u3-2u2)(t**2)+2(-2u1+3u2-u3)*(t)+u1
! u1(3t**2-4t+1)+u2(-6t**2+6t)+u3(3t**2-2t)
! the weights for the flux are then given by the integral
! of this polynomial over the corresponding period
!
#  ifdef AGRIF_POLY_DUAVG
      t4=(real(Agrif_NbStepint()+1)/Agrif_rhot())**3-
     &      (real(Agrif_NbStepint())/Agrif_rhot())**3
      t5=(real(Agrif_NbStepint()+1)/Agrif_rhot())**2-
     &      (real(Agrif_NbStepint())/Agrif_rhot())**2
      t6=1./Agrif_rhot()
      t1=t4-2.*t5+t6
      t2=-2.*t4+3.*t5
      t3=t4-t5 
#  else        
      if (Agrif_Nb_Step() .lt. Agrif_rhot()) then
        t1=1.
        t2=0.
        t3=0.
      elseif (Agrif_NbStepint().eq.0) then
        t1=7./9.
        t2=-2./9.
        t3=4./9.
      elseif (Agrif_NbStepint().eq.1) then
        t1=13./9.
        t2=-2./9.
        t3=-2./9.
      else
        t1=7./9.
        t2=4./9.
        t3=-2./9.
      endif
#  endif
#  ifdef AGRIF_OBC_WEST
#   ifdef MPI
      if (.not.WEST_INTER) then
#   endif
        do j=0,LOCALMM+1
          DU_west(j)=( t1*DU_west1(j,1)
     &                +t2*DU_west1(j,2)
     &                +t3*DU_west1(j,3))/Agrif_rhoy()
      enddo          
#   ifdef MPI            
      endif
#   endif 
#  endif
#  ifdef AGRIF_OBC_EAST
#   ifdef MPI            
      if (.not.EAST_INTER) then
#   endif 
        do j=0,LOCALMM+1
          DU_east(j)=( t1*DU_east1(j,1)
     &                +t2*DU_east1(j,2)
     &                +t3*DU_east1(j,3))/Agrif_rhoy()
        enddo
#   ifdef MPI                       
      endif
#   endif 
#  endif
#  ifdef AGRIF_OBC_SOUTH
#   ifdef MPI      
      if (.not.SOUTH_INTER) Then
#   endif
        do i=0,LOCALLM+1
          DU_south(i)=( t1*DU_south1(i,1)
     &                 +t2*DU_south1(i,2)
     &                 +t3*DU_south1(i,3))/Agrif_rhox()
        enddo          
#   ifdef MPI           
      endif          
#   endif 
#  endif
#  ifdef AGRIF_OBC_NORTH
#   ifdef MPI            
      if (.not.NORTH_INTER) then  
#   endif
        do i=0,LOCALLM+1
          DU_north(i)=( t1*DU_north1(i,1)
     &                 +t2*DU_north1(i,2)
     &                 +t3*DU_north1(i,3))/Agrif_rhox()
        enddo          
#   ifdef MPI               
      endif  
#   endif 
#  endif 
# endif /* AGRIF_STORE_BAROT_CHILD */
!
! The following two code segments correct "DU_west" and "DU_east"
! locally to enforce volume conservation: after this operation sum
! of fluxes within each triad on child grid is equal to corresponding
! flux on the parent grid. This is done by computing and subtraction
! the discrepancy between parent "fxp" and child "fxc" fluxes. The
! correction term is distributed among the three child grid fluxes
! proportionally to local depth, which translates into the same
! correction velocity for the three fluxes.
!
# ifdef AGRIF_LOCAL_VOLCONS
!
      iint=int(Agrif_IRhox()/2)
      jint=int(Agrif_IRhoy()/2)
!             
      irmin=1+iint
      irmax=Lm-iint
      jrmin=1+jint
      jrmax=Mm-jint 
!
! Western boundary
!
#  ifdef AGRIF_OBC_WEST
      ipu=Agrif_Ix()
      do j=jrmin,jrmax,Agrif_IRhoy()
        jpu=Agrif_Iy()+(j-jrmin)/Agrif_IRhoy()
#   ifdef MASKING
        if (Agrif_Parent_umask(ipu,jpu).eq.0.) then
          DU_west(j-jint:j+jint)=0.
        else
#   endif 
          fxp=Agrif_rhoy()*DU_west(j)  ! <-- parent grid flux
          fxc=0.                       ! <--  child grid flux
          surfc=0. 
          do jj=-jint,jint
#   ifdef MASKING
            DU_west(j+jj)=DU_west(j+jj)*umask(1,j+jj)
#   endif 
            fxc=fxc+DU_west(j+jj)
            surfc=surfc+0.5*(h(0,j+jj)+zeta(0,j+jj,kindx)+
     &                       h(1,j+jj)+zeta(1,j+jj,kindx))
#   ifdef MASKING
     &                     *umask(1,j+jj)
#   endif 
          enddo
          if (surfc.gt.0.) surfc=(fxp-fxc)/surfc
          do jj=-jint,jint
            DU_west(j+jj)=DU_west(j+jj)+
     &                     surfc*
     &                     0.5*(h(0,j+jj)+zeta(0,j+jj,kindx)+
     &                          h(1,j+jj)+zeta(1,j+jj,kindx))
#   ifdef MASKING
     &                        *umask(1,j+jj)
#   endif 
          enddo
#   ifdef MASKING
        endif
#   endif 
      enddo
#  endif 
!
! Eastern boundary
!
#  ifdef AGRIF_OBC_EAST
      ipu=Agrif_Ix()+Lm/Agrif_IRhox()
      do j=jrmin,jrmax,Agrif_IRhoy()
        jpu=Agrif_Iy()+(j-jrmin)/Agrif_IRhoy()
#   ifdef MASKING
        if (Agrif_Parent_umask(ipu,jpu).eq.0.) then
          DU_east(j-jint:j+jint)=0.
        else
#   endif
          fxp=Agrif_rhoy()*DU_east(j)
          fxc=0.
          surfc=0.
          do jj=-jint,jint
#   ifdef MASKING
            DU_east(j+jj)=DU_east(j+jj)*umask(Lm+1,j+jj)
#   endif
            fxc=fxc+DU_east(j+jj)
            surfc=surfc+0.5*(h(Lm,j+jj)+zeta(Lm,j+jj,kindx)+
     &                       h(Lm+1,j+jj)+zeta(Lm+1,j+jj,kindx))
#   ifdef MASKING
     &                     *umask(Lm+1,j+jj)
#   endif
          enddo
          if (surfc.gt.0.) surfc=(fxp-fxc)/surfc
          do jj=-jint,jint
            DU_east(j+jj)=DU_east(j+jj)+
     &                     surfc*
     &                     0.5*(h(Lm,j+jj)+zeta(Lm,j+jj,kindx)+
     &                          h(Lm+1,j+jj)+zeta(Lm+1,j+jj,kindx))
#   ifdef MASKING
     &                        *umask(Lm+1,j+jj)
#   endif
          enddo
#   ifdef MASKING
        endif
#   endif
      enddo
#  endif 
# endif 
!
C$OMP END MASTER
C$OMP BARRIER
!
! Apply the value to ubclm or ubarbry
!
# ifdef AGRIF_OBC_WEST
      if (WESTERN_EDGE) then
        do j=JstrR,JendR
#  ifdef M2_FRC_BRY
          ubarbry_west(j)=
#  else
          ubclm(Istr,j)=
#  endif
#  ifdef AGRIF_FLUX_BC
     &              (2.*DU_west(j)/((h(Istr-1,j)+zeta(Istr-1,j,kindx)
     &                                 +h(Istr,j)+zeta(Istr,j,kindx))
     &                                                *on_u(Istr,j)))
#  else
     &                  DU_west(j)
#  endif
#  ifdef MASKING
     &                  *umask(Istr,j)
#  endif
        enddo
      endif
# endif
# ifdef AGRIF_OBC_EAST
      if (EASTERN_EDGE) then
        do j=JstrR,JendR
#  ifdef M2_FRC_BRY
          ubarbry_east(j)=
#  else
          ubclm(Iend+1,j)=
#  endif
#  ifdef AGRIF_FLUX_BC
     &                  (2.*DU_east(j)/(( h(Iend,j)+zeta(Iend,j,kindx)
     &                              +h(Iend+1,j)+zeta(Iend+1,j,kindx))
     &                                              *on_u(Iend+1,j)))
#  else
     &                  DU_east(j)
#  endif
#  ifdef MASKING
     &                  *umask(Iend+1,j)
#  endif
        enddo
      endif
# endif
# ifdef AGRIF_OBC_SOUTH
      if (SOUTHERN_EDGE) then       
        do i=IstrU,IendR
#  ifdef M2_FRC_BRY
          ubarbry_south(i)=
#  else
          ubclm(i,Jstr-1)=
#  endif
#  ifdef AGRIF_FLUX_BC
     &              (2.*DU_south(i)/(( h(i,Jstr-1)+zeta(i,Jstr-1,kindx)
     &                           +h(i-1,Jstr-1)+zeta(i-1,Jstr-1,kindx))
     &                                                *on_u(i,Jstr-1)))
#  else
     &                  DU_south(i)
#  endif
#  ifdef MASKING
     &                  *umask(i,Jstr-1)
#  endif
        enddo
      endif
# endif
# ifdef AGRIF_OBC_NORTH
      if (NORTHERN_EDGE) then
        do i=IstrU,IendR
#  ifdef M2_FRC_BRY
          ubarbry_north(i)=
#  else
          ubclm(i,Jend+1)=
#  endif
#  ifdef AGRIF_FLUX_BC
     &            (2.*DU_north(i)/(( h(i,Jend+1)+zeta(i,Jend+1,kindx)
     &                         +h(i-1,Jend+1)+zeta(i-1,Jend+1,kindx))
     &                                             *on_u(i,Jend+1)))
#  else
     &                  DU_north(i)
#  endif
#  ifdef MASKING
     &                  *umask(i,Jend+1)
#  endif
        enddo
      endif
# endif
      return
      end
!
!====================================================================
!                   subroutine Agrif_v2dbc_interp_tile
!====================================================================
!
      subroutine Agrif_v2dbc_interp_tile(Istr,Iend,Jstr,Jend,kindx)
      use AGRIF_Util
      implicit none
# include "param.h"
# include "grid.h"
# include "ocean2d.h"
# include "scalars.h"
# include "climat.h"
# include "boundary.h"
# include "zoom.h"
      integer Istr,Iend,Jstr,Jend, i,j,kindx
# ifdef AGRIF_LOCAL_VOLCONS               
      integer irmin,irmax,jrmin,jrmax,ipv,jpv,iint,jint,ii
      real fxp,fxc,surfc
# endif      
# ifdef AGRIF_STORE_BAROT_CHILD      
      integer i1,i2,j1,j2,k1,k2
      real xpr,ypr,tpr,p1,p2,q1,q2
# else
      real t1,t2,t3,t4,t5,t6
      real,Dimension(:,:), Allocatable :: dvinterp
      real, Dimension(:,:,:), Allocatable :: dvinterp2
      real tinterp
      logical ptinterp
# endif      
# ifdef MPI
#  define LOCALLM Lmmpi
#  define LOCALMM Mmmpi
# else
#  define LOCALLM Lm
#  define LOCALMM Mm
# endif   
!          
# include "compute_auxiliary_bounds.h"
!
C$OMP MASTER
# ifdef AGRIF_STORE_BAROT_CHILD
!
! Get the temporal interpolant
!     
      tpr=float(Agrif_NbStepInt()*ndtfast+iif)/Agrif_rhot()
      k1=int(tpr)
      k2=k1+1
      q2=tpr-float(k1)
      q1=1.0-q2

#  ifdef AGRIF_OBC_WEST
#   ifdef MPI
      if (.not.WEST_INTER) then
#   endif
        do j=0,LOCALMM+1
          ypr=float(Agrif_Iy())+(float(j)-1.)/Agrif_rhoy()
          j1=int(ypr)
          j2=j1+1
#   ifdef MASKING
          p2=max(1.-max(Agrif_Parent_vmask(Agrif_Parent_ic1,j1),
     &                  Agrif_Parent_vmask(Agrif_Parent_ic1-1,j1)),
     &           ypr-float(j1))
          p1=max(1.-max(Agrif_Parent_vmask(Agrif_Parent_ic1,j2),
     &                  Agrif_Parent_vmask(Agrif_Parent_ic1-1,j2)),
     &           1.-p2)         
#   else
          p2=ypr-float(j1)
          p1=1.-p2        
#   endif
          DV_west(j)=(q1*(p1*Agrif_Parent_vbar_westbound(j1,k1)
     &                   +p2*Agrif_Parent_vbar_westbound(j2,k1))
     &               +q2*(p1*Agrif_Parent_vbar_westbound(j1,k2)
     &                   +p2*Agrif_Parent_vbar_westbound(j2,k2)))
#   ifdef AGRIF_FLUX_BC
     &              /Agrif_rhoy()
#   endif
        enddo
#   ifdef MPI            
      endif
#   endif 
#  endif 
#  ifdef AGRIF_OBC_EAST
#   ifdef MPI            
      if (.not.EAST_INTER) then
#   endif 
        do j=0,LOCALMM+1
          ypr=float(Agrif_Iy())+(float(j)-1.)/Agrif_rhoy()
          j1=int(ypr)
          j2=j1+1
#   ifdef MASKING
          p2=max(1.-max(Agrif_Parent_vmask(Agrif_Parent_ic2,j1),
     &                  Agrif_Parent_vmask(Agrif_Parent_ic2-1,j1)),
     &           ypr-float(j1))
          p1=max(1.-max(Agrif_Parent_vmask(Agrif_Parent_ic2,j2),
     &                  Agrif_Parent_vmask(Agrif_Parent_ic2-1,j2)),
     &           1.-p2)
#   else
          p2=ypr-float(j1)
          p1=1.-p2
#   endif
          DV_east(j)=(q1*(p1*Agrif_Parent_vbar_eastbound(j1,k1)
     &                   +p2*Agrif_Parent_vbar_eastbound(j2,k1))
     &               +q2*(p1*Agrif_Parent_vbar_eastbound(j1,k2)
     &                   +p2*Agrif_Parent_vbar_eastbound(j2,k2)))
#   ifdef AGRIF_FLUX_BC
     &              /Agrif_rhoy()
#   endif
        enddo
#   ifdef MPI                       
      endif
#   endif 
#  endif 
#  ifdef AGRIF_OBC_SOUTH
#   ifdef MPI            
      if (.not.SOUTH_INTER) Then
#   endif
        do i=0,LOCALLM+1
          xpr=float(Agrif_Ix())+(float(i)-Agrif_rhox()/2.-0.5)
     &                                            /Agrif_rhox()
          i1=int(xpr)
          i2=i1+1
#   ifdef MASKING
          p2=max(1.-Agrif_Parent_vmask(i1,Agrif_Parent_jc1),
     &           xpr-float(i1))
          p1=max(1.-Agrif_Parent_vmask(i2,Agrif_Parent_jc1),1.-p2)
#   else
          p2=xpr-float(i1)
          p1=1.-p2
#   endif
          DV_south(i)=(q1*(p1*Agrif_Parent_vbar_southbound(i1,k1)
     &                    +p2*Agrif_Parent_vbar_southbound(i2,k1))
     &                +q2*(p1*Agrif_Parent_vbar_southbound(i1,k2)
     &                    +p2*Agrif_Parent_vbar_southbound(i2,k2)))
#   ifdef AGRIF_FLUX_BC
     &              /Agrif_rhox()
#   endif
        enddo          
#   ifdef MPI           
      endif          
#   endif 
#  endif 
#  ifdef AGRIF_OBC_NORTH
#   ifdef MPI            
      if (.not.NORTH_INTER) then  
#   endif
        do i=0,LOCALLM+1
          xpr=float(Agrif_Ix())+(float(i)-Agrif_rhox()/2.-0.5)
     &                                            /Agrif_rhox()
          i1=int(xpr)
          i2=i1+1
#   ifdef MASKING
          p2=max(1.-Agrif_Parent_vmask(i1,Agrif_Parent_jc2),
     &           xpr-float(i1))
          p1=max(1.-Agrif_Parent_vmask(i2,Agrif_Parent_jc2),1.-p2)
#   else
          p2=xpr-float(i1)
          p1=1.-p2
#   endif
          DV_north(i)=(q1*(p1*Agrif_Parent_vbar_northbound(i1,k1)
     &                    +p2*Agrif_Parent_vbar_northbound(i2,k1))
     &                +q2*(p1*Agrif_Parent_vbar_northbound(i1,k2)
     &                    +p2*Agrif_Parent_vbar_northbound(i2,k2)))
#   ifdef AGRIF_FLUX_BC
     &              /Agrif_rhox()
#   endif
        enddo          
#   ifdef MPI               
      endif  
#   endif 
#  endif 
# else   
      If (V2DTimeindex .LT. Agrif_Parent_Nb_Step()) Then
        ptinterp = .TRUE.
        tinterp=1.
        Allocate(dvinterp(GLOBAL_2D_ARRAY))
        Allocate(dvinterp2(GLOBAL_2D_ARRAY,2))
        Agrif_UseSpecialValue = .true.
        Agrif_SpecialValue = 0.
        dvinterp=0.
        dvinterp2=0. 
        call Agrif_Bc_DV_avg2(dvinterp,tinterp,ptinterp)
        call Agrif_Bc_DV_avg1(dvinterp2,tinterp,ptinterp)
        Agrif_UseSpecialvalue=.false.  
#  ifdef AGRIF_OBC_SOUTH
#   ifdef MPI     
        If (.not.SOUTH_INTER) Then
#   endif            
          do i=0,LOCALLM+1
            DV_south1(i,2)=dvinterp(i,1)
            DV_south1(i,3)=dvinterp2(i,1,Agrif_parent_nnew)
            DV_south1(i,1)=dvinterp2(i,1,Agrif_parent_nstp)
          enddo
#   ifdef MPI           
        endif          
#   endif
#  endif
#  ifdef AGRIF_OBC_NORTH
#   ifdef MPI     
        if (.not.NORTH_INTER) then  
#   endif
          do i=0,LOCALLM+1
            DV_north1(i,2)=dvinterp(i,LOCALMM+1)
            DV_north1(i,3)=dvinterp2(i,LOCALMM+1,Agrif_parent_nnew)
            DV_north1(i,1)=dvinterp2(i,LOCALMM+1,Agrif_parent_nstp)
          enddo          
#   ifdef MPI
        endif
#   endif
#  endif
#  ifdef AGRIF_OBC_WEST
#   ifdef MPI     
        if (.not.WEST_INTER) then
#   endif                 
          do j=0,LOCALMM+1
            DV_west1(j,2)=dvinterp(0,j)
            DV_west1(j,3)=dvinterp2(0,j,Agrif_parent_nnew)
            DV_west1(j,1)=dvinterp2(0,j,Agrif_parent_nstp)
          enddo
#   ifdef MPI            
        endif
#   endif
#  endif
#  ifdef AGRIF_OBC_EAST
#   ifdef MPI     
        if (.not.EAST_INTER) then
#   endif 
          do j=0,LOCALMM+1
            DV_east1(j,2)=dvinterp(LOCALLM+1,j)
            DV_east1(j,3)=dvinterp2(LOCALLM+1,j,Agrif_parent_nnew)
            DV_east1(j,1)=dvinterp2(LOCALLM+1,j,Agrif_parent_nstp)
          enddo          
#   ifdef MPI               
        endif  
#   endif
#  endif
        deallocate(dvinterp)
        deallocate(dvinterp2)
        V2DTimeindex = Agrif_Parent_Nb_Step()  
      endif 
!
! We seek a second order polynomial that equals
! DU_avg1(nstp) at time n
! DU_avg1(nnew) at time n+1
! and has its integral over [n,n+1] equal to DU_avg2
! this polynomial is given by
! 3(u1+u3-2u2)(t**2)+2(-2u1+3u2-u3)*(t)+u1
! u1(3t**2-4t+1)+u2(-6t**2+6t)+u3(3t**2-2t)
! the weights for the flux are then given by the integral
! of this polynomial over the corresponding period
!
#  ifdef AGRIF_POLY_DUAVG
      t4=(real(Agrif_NbStepint()+1)/Agrif_rhot())**3-
     &      (real(Agrif_NbStepint())/Agrif_rhot())**3
      t5=(real(Agrif_NbStepint()+1)/Agrif_rhot())**2-
     &      (real(Agrif_NbStepint())/Agrif_rhot())**2
      t6=1./Agrif_rhot()
      t1=t4-2.*t5+t6
      t2=-2.*t4+3.*t5
      t3=t4-t5 
#  else        
      if (Agrif_Nb_Step() .lt. Agrif_rhot()) then
        t1=1.
        t2=0.
        t3=0.
      elseif (Agrif_NbStepint().eq.0) then
        t1=7./9.
        t2=-2./9.
        t3=4./9.
      elseif (Agrif_NbStepint().eq.1) then
        t1=13./9.
        t2=-2./9.
        t3=-2./9.
      else
        t1=7./9.
        t2=4./9.
        t3=-2./9.
      endif
#  endif
#  ifdef AGRIF_OBC_SOUTH
#   ifdef MPI     
      If (.not.SOUTH_INTER) Then
#   endif            
        do i=0,LOCALLM+1
          DV_south(i)=( t1*DV_south1(i,1)
     &                 +t2*DV_south1(i,2)
     &                 +t3*DV_south1(i,3))/Agrif_rhox()
        enddo
#   ifdef MPI           
      endif          
#   endif
#  endif
#  ifdef AGRIF_OBC_NORTH
#   ifdef MPI           
      if (.not.NORTH_INTER) then  
#   endif
        do i=0,LOCALLM+1
          DV_north(i)=( t1*DV_north1(i,1)
     &                 +t2*DV_north1(i,2)
     &                 +t3*DV_north1(i,3))/Agrif_rhox()
        enddo          
#   ifdef MPI
      endif
#   endif
#  endif
#  ifdef AGRIF_OBC_WEST
#   ifdef MPI           
      if (.not.WEST_INTER) then
#   endif                 
        do j=0,LOCALMM+1
          DV_west(j)=( t1*DV_west1(j,1)
     &                +t2*DV_west1(j,2)
     &                +t3*DV_west1(j,3))/Agrif_rhoy()
        enddo
#   ifdef MPI            
      endif
#   endif
#  endif
#  ifdef AGRIF_OBC_EAST
#   ifdef MPI           
      if (.not.EAST_INTER) then
#   endif 
        do j=0,LOCALMM+1
          DV_east(j)=( t1*DV_east1(j,1)
     &                +t2*DV_east1(j,2)
     &                +t3*DV_east1(j,3))/Agrif_rhoy()
        enddo          
#   ifdef MPI               
      endif  
#   endif
#  endif
# endif /* AGRIF_STORE_BAROT_CHILD */
!
! The following two code segments correct "DV_south" and "DV_north"
! locally to enforce volume conservation: after this operation sum
! of fluxes within each triad on child grid is equal to corresponding
! flux on the parent grid. This is done by computing and subtraction
! the discrepancy between parent "fxp" and child "fxc" fluxes. The
! correction term is distributed among the three child grid fluxes
! proportionally to local depth, which translates into the same
! correction velocity for the three fluxes.
!
# ifdef AGRIF_LOCAL_VOLCONS
!
      iint=int(Agrif_IRhox()/2)
      jint=int(Agrif_IRhoy()/2)
!             
      irmin=1+iint
      irmax=Lm-iint
      jrmin=1+jint
      jrmax=Mm-jint  
!
! Southern boundary
!
#  ifdef AGRIF_OBC_SOUTH
      jpv=Agrif_Iy()
      do i=irmin,irmax,Agrif_IRhox()
        ipv=Agrif_Ix()+(i-irmin)/Agrif_IRhox()
#   ifdef MASKING
        if (Agrif_Parent_vmask(ipv,jpv).eq.0.) then
          DV_south(i-iint:i+iint)=0.
        else
#   endif
          fxp=Agrif_rhox()*DV_south(i) ! <-- parent grid flux
          fxc=0.                       ! <--  child grid flux
          surfc=0. 
          do ii=-iint,iint
#   ifdef MASKING
            DV_south(i+ii)=DV_south(i+ii)*vmask(i+ii,1)
#   endif 
            fxc=fxc+DV_south(i+ii)
            surfc=surfc+0.5*(h(i+ii,0)+zeta(i+ii,0,kindx)+
     &                       h(i+ii,1)+zeta(i+ii,1,kindx))
#   ifdef MASKING
     &                       *vmask(i+ii,1)
#   endif 
          enddo
          if (surfc.gt.0.) surfc=(fxp-fxc)/surfc 
          do ii=-iint,iint
            DV_south(i+ii)=DV_south(i+ii)+
     &                       surfc*
     &                       0.5*(h(i+ii,0)+zeta(i+ii,0,kindx)+
     &                            h(i+ii,1)+zeta(i+ii,1,kindx))
#   ifdef MASKING
     &                          *vmask(i+ii,1)
#   endif 
          enddo
#   ifdef MASKING
        endif
#   endif
      enddo
#  endif 
#  ifdef AGRIF_OBC_NORTH
!
! Northern boundary
!
      jpv=Agrif_Iy()+Mm/Agrif_IRhoy()
      do i=irmin,irmax,Agrif_IRhox()
        ipv=Agrif_Ix()+(i-irmin)/Agrif_IRhox()
#   ifdef MASKING
        if (Agrif_Parent_vmask(ipv,jpv).eq.0.) then
          DV_north(i-iint:i+iint)=0.
        else
#   endif 
          fxp=Agrif_rhox()*DV_north(i)
          fxc=0.
          surfc=0. 
          do ii=-iint,iint
#   ifdef MASKING
            DV_north(i+ii)=DV_north(i+ii)*vmask(i+ii,Mm+1)
#   endif 
            fxc=fxc+DV_north(i+ii)
            surfc=surfc+0.5*(h(i+ii,Mm)+zeta(i+ii,Mm,kindx)+
     &                       h(i+ii,Mm+1)+zeta(i+ii,Mm+1,kindx))
#   ifdef MASKING
     &                       *vmask(i+ii,Mm+1)
#   endif 
          enddo
          if (surfc.gt.0.) surfc=(fxp-fxc)/surfc 
          do ii=-iint,iint
            DV_north(i+ii)=DV_north(i+ii)+
     &                      surfc*
     &                      0.5*(h(i+ii,Mm)+zeta(i+ii,Mm,kindx)+
     &                           h(i+ii,Mm+1)+zeta(i+ii,Mm+1,kindx))
#   ifdef MASKING
     &                          *vmask(i+ii,Mm+1)
#   endif 
          enddo
#   ifdef MASKING
        endif
#   endif 
      enddo
#  endif 
# endif 

C$OMP END MASTER
C$OMP BARRIER

# ifdef AGRIF_OBC_SOUTH
      if (SOUTHERN_EDGE) then
         do i=IstrR,IendR
#  ifdef M2_FRC_BRY
           vbarbry_south(i)=
#  else
           vbclm(i,Jstr)=
#  endif
#  ifdef AGRIF_FLUX_BC
     &              (2.*DV_south(i)/((h(i,Jstr-1)+zeta(i,Jstr-1,kindx)
     &                                  +h(i,Jstr)+zeta(i,Jstr,kindx))
     &                                                 *om_v(i,Jstr)))
#  else
     &                  DV_south(i)
#  endif
#  ifdef MASKING
     &                  *vmask(i,Jstr)
#  endif 
       enddo
      endif
# endif 
# ifdef AGRIF_OBC_NORTH
      if (NORTHERN_EDGE) then
        do i=IstrR,IendR
#  ifdef M2_FRC_BRY
          vbarbry_north(i)=
#  else
          vbclm(i,Jend+1)=
#  endif
#  ifdef AGRIF_FLUX_BC
     &                  (2.*DV_north(i)/(( h(i,Jend)+zeta(i,Jend,kindx)
     &                               +h(i,Jend+1)+zeta(i,Jend+1,kindx))
     &                                               *om_v(i,Jend+1)))
#  else
     &                  DV_north(i)
#  endif
#  ifdef MASKING
     &                  *vmask(i,Jend+1)
#  endif 
        enddo
      endif
# endif 
# ifdef AGRIF_OBC_EAST       
      if (EASTERN_EDGE) then
        do j=JstrV,JendR
#  ifdef M2_FRC_BRY
          vbarbry_east(j)=
#  else
          vbclm(Iend+1,j)=
#  endif
#  ifdef AGRIF_FLUX_BC
     &               (2.*DV_east(j)/(( h(Iend+1,j)+zeta(Iend+1,j,kindx)
     &                           +h(Iend+1,j-1)+zeta(Iend+1,j-1,kindx))
     &                                                *om_v(Iend+1,j)))
#  else
     &                  DV_east(j)
#  endif
#  ifdef MASKING
     &                  *vmask(Iend+1,j)
#  endif
        enddo
      endif       
# endif 
# ifdef AGRIF_OBC_WEST             
      if (WESTERN_EDGE) then
        do j=JstrV,JendR
#  ifdef M2_FRC_BRY
          vbarbry_west(j)=
#  else
          vbclm(Istr-1,j)=
#  endif
#  ifdef AGRIF_FLUX_BC
     &                (2.*DV_west(j)/((h(Istr-1,j)+zeta(Istr-1,j,kindx)
     &                           +h(Istr-1,j-1)+zeta(Istr-1,j-1,kindx))
     &                                                *om_v(Istr-1,j)))
#  else
     &                  DV_west(j)
#  endif
#  ifdef MASKING
     &                  *vmask(Istr-1,j)
#  endif
        enddo
      endif
# endif
      return
      end
!
!====================================================================
!                   subroutine Agrif_zetabc_interp_tile
!====================================================================
!
      subroutine Agrif_zetabc_interp_tile(Istr,Iend,Jstr,Jend)
      use AGRIF_Util
      implicit none
# include "param.h"
# include "boundary.h"
# include "climat.h"
# include "grid.h"
# include "ocean2d.h"
# include "scalars.h"
# include "zoom.h"
      integer Istr,Iend,Jstr,Jend, i,j
# ifdef AGRIF_STORE_BAROT_CHILD
      integer i1,i2,j1,j2,k1,k2
      real xpr,ypr,tpr,p1,p2,q1,q2
# else     
      real tinterp
      logical ptinterp
      real,Dimension(:,:,:), Allocatable :: zext
      Integer itrcind
#  ifdef MPI
#   define LOCALLM Lmmpi
#   define LOCALMM Mmmpi
#  else
#   define LOCALLM Lm
#   define LOCALMM Mm
#  endif      
# endif
!
# include "compute_auxiliary_bounds.h"
!
# ifdef AGRIF_STORE_BAROT_CHILD
!
! Get the temporal interpolant 
!     
      tpr=float(Agrif_NbStepInt()*ndtfast+iif)/Agrif_rhot()
      k1=int(tpr)
      k2=k1+1
      q2=tpr-float(k1)
      q1=1.-q2
#  ifdef AGRIF_OBC_WEST             
      if (WESTERN_EDGE) then      
        do j=JstrR,JendR
          ypr=float(Agrif_Iy())+(float(j)-Agrif_rhoy()/2.-0.5)
     &                           /Agrif_rhoy()
          j1=int(ypr)
          j2=j1+1
#   ifdef MASKING
          p2=max(1.-max(Agrif_Parent_rmask(Agrif_Parent_ic1,j1),
     &                  Agrif_Parent_rmask(Agrif_Parent_ic1-1,j1)),
     &           ypr-float(j1))
          p1=max(1.-max(Agrif_Parent_rmask(Agrif_Parent_ic1,j2),
     &                  Agrif_Parent_rmask(Agrif_Parent_ic1-1,j2)),
     &           1.-p2)         
#   else
          p2=ypr-float(j1)
          p1=1.-p2         
#   endif
#   if defined AGRIF_OBC_M2FLATHER && !defined Z_FRC_BRY
          SSH(1,j)=
     &               (q1*(p1*Agrif_Parent_zeta_westbound(2,j1,k1)
     &                   +p2*Agrif_Parent_zeta_westbound(2,j2,k1))
     &               +q2*(p1*Agrif_Parent_zeta_westbound(2,j1,k2)
     &                   +p2*Agrif_Parent_zeta_westbound(2,j2,k2)))
          SSH(0,j)=
#   else
#    ifdef Z_FRC_BRY
          zetabry_west(j)=
#    else
          SSH(0,j)=
#    endif
#   endif
     &               (q1*(p1*Agrif_Parent_zeta_westbound(1,j1,k1)
     &                   +p2*Agrif_Parent_zeta_westbound(1,j2,k1))
     &               +q2*(p1*Agrif_Parent_zeta_westbound(1,j1,k2)
     &                   +p2*Agrif_Parent_zeta_westbound(1,j2,k2)))
        enddo
      endif
#  endif
#  ifdef AGRIF_OBC_EAST             
      if (EASTERN_EDGE) then      
        do j=JstrR,JendR
          ypr=float(Agrif_Iy())+(float(j)-Agrif_rhoy()/2.-0.5)
     &                           /Agrif_rhoy()
          j1=int(ypr)
          j2=j1+1
#   ifdef MASKING
          p2=max(1.-max(Agrif_Parent_rmask(Agrif_Parent_ic2,j1),
     &                  Agrif_Parent_rmask(Agrif_Parent_ic2-1,j1)),
     &           ypr-float(j1))
          p1=max(1.-max(Agrif_Parent_rmask(Agrif_Parent_ic2,j2),
     &                  Agrif_Parent_rmask(Agrif_Parent_ic2-1,j2)),
     &           1.-p2)
#   else
          p2=ypr-float(j1)
          p1=1.-p2
#   endif
#   if defined AGRIF_OBC_M2FLATHER && !defined Z_FRC_BRY
          SSH(Iend,j)=  
     &               (q1*(p1*Agrif_Parent_zeta_eastbound(2,j1,k1)
     &                   +p2*Agrif_Parent_zeta_eastbound(2,j2,k1))
     &               +q2*(p1*Agrif_Parent_zeta_eastbound(2,j1,k2)
     &                   +p2*Agrif_Parent_zeta_eastbound(2,j2,k2)))
          SSH(Iend+1,j)=  
#   else
#    ifdef Z_FRC_BRY
          zetabry_east(j)= 
#    else
          SSH(Iend+1,j)=
#    endif
#   endif
     &               (q1*(p1*Agrif_Parent_zeta_eastbound(1,j1,k1)
     &                   +p2*Agrif_Parent_zeta_eastbound(1,j2,k1))
     &               +q2*(p1*Agrif_Parent_zeta_eastbound(1,j1,k2)
     &                   +p2*Agrif_Parent_zeta_eastbound(1,j2,k2)))
        enddo
      endif
#  endif
#  ifdef AGRIF_OBC_SOUTH             
      if (SOUTHERN_EDGE) then
        do i=IstrR,IendR
          xpr=float(Agrif_Ix())+(float(i)-Agrif_rhox()/2.-0.5)
     &                          /Agrif_rhox()
          i1=int(xpr)
          i2=i1+1
#   ifdef MASKING
          p2=max(1.-max(Agrif_Parent_rmask(i1,Agrif_Parent_jc1-1),
     &                  Agrif_Parent_rmask(i1,Agrif_Parent_jc1)),
     &           xpr-float(i1))
          p1=max(1.-max(Agrif_Parent_rmask(i2,Agrif_Parent_jc1-1),
     &                  Agrif_Parent_rmask(i2,Agrif_Parent_jc1)),
     &           1.-p2)
#   else
          p2=xpr-float(i1)
          p1=1.-p2
#   endif
#   if defined AGRIF_OBC_M2FLATHER && !defined Z_FRC_BRY
          SSH(i,1)=
     &                (q1*(p1*Agrif_Parent_zeta_southbound(2,i1,k1)
     &                    +p2*Agrif_Parent_zeta_southbound(2,i2,k1))
     &                +q2*(p1*Agrif_Parent_zeta_southbound(2,i1,k2)
     &                    +p2*Agrif_Parent_zeta_southbound(2,i2,k2)))
          SSH(i,0)=
#   else
#    ifdef Z_FRC_BRY
          zetabry_south(i)=
#    else
          SSH(i,0)=
#    endif
#   endif
     &                (q1*(p1*Agrif_Parent_zeta_southbound(1,i1,k1)
     &                    +p2*Agrif_Parent_zeta_southbound(1,i2,k1))
     &                +q2*(p1*Agrif_Parent_zeta_southbound(1,i1,k2)
     &                    +p2*Agrif_Parent_zeta_southbound(1,i2,k2)))
        enddo          
      endif 
#  endif
#  ifdef AGRIF_OBC_NORTH             
      if (NORTHERN_EDGE) then      
        do i=IstrR,IendR 
          xpr=float(Agrif_Ix())+(float(i)-Agrif_rhox()/2.-0.5)
     &                           /Agrif_rhox()
          i1=int(xpr)
          i2=i1+1
#   ifdef MASKING
          p2=max(1.-max(Agrif_Parent_rmask(i1,Agrif_Parent_jc2-1),
     &                  Agrif_Parent_rmask(i1,Agrif_Parent_jc2)),
     &           xpr-float(i1))
          p1=max(1.-max(Agrif_Parent_rmask(i2,Agrif_Parent_jc2-1),
     &                  Agrif_Parent_rmask(i2,Agrif_Parent_jc2)),
     &           1.-p2)
#   else
          p2=xpr-float(i1)
          p1=1.-p2
#   endif
#   if defined AGRIF_OBC_M2FLATHER && !defined Z_FRC_BRY
          SSH(i,Jend)=
     &               (q1*(p1*Agrif_Parent_zeta_northbound(2,i1,k1)
     &                   +p2*Agrif_Parent_zeta_northbound(2,i2,k1))
     &               +q2*(p1*Agrif_Parent_zeta_northbound(2,i1,k2)
     &                   +p2*Agrif_Parent_zeta_northbound(2,i2,k2)))
          SSH(i,Jend+1)=
#   else
#    ifdef Z_FRC_BRY
          zetabry_north(i)=
#    else
          SSH(i,Jend+1)=
#    endif
#   endif
     &               (q1*(p1*Agrif_Parent_zeta_northbound(1,i1,k1)
     &                   +p2*Agrif_Parent_zeta_northbound(1,i2,k1))
     &               +q2*(p1*Agrif_Parent_zeta_northbound(1,i1,k2)
     &                   +p2*Agrif_Parent_zeta_northbound(1,i2,k2)))
        enddo
      endif  
#  endif
!
# else /* AGRIF_STORE_BAROT_CHILD */
!
#  ifdef AGRIF_OBC_M2FLATHER
        if (FIRST_2D_STEP .and. PREDICTOR_2D_STEP) then
#  endif
C$OMP MASTER        
          if (Zetatimeindex .LT. Agrif_Parent_Nb_Step()) then
            Allocate(zext(GLOBAL_2D_ARRAY,3))
            Agrif_UseSpecialvalue=.true.
            Agrif_Specialvalue=0.
            tinterp=1.
            ptinterp=.true.
            zext=0.
            Call Agrif_Bc_zeta(zext(START_2D_ARRAY,1),tinterp,ptinterp)
            Agrif_UseSpecialvalue=.false. 
        
            do itrcind = 1,3
#  ifdef AGRIF_OBC_SOUTH             
#   ifdef MPI        
              if (.not.SOUTH_INTER) then
#   endif
                do i=0,LOCALLM+1
                  Zeta_south(i,0:1,itrcind)=zext(i,0:1,itrcind)
                enddo
#   ifdef MPI         
              endif          
#   endif
#  endif
#  ifdef AGRIF_OBC_NORTH             
#   ifdef MPI         
              if (.not.NORTH_INTER) then  
#   endif           
                do i=0,LOCALLM+1
                  Zeta_north(i,LOCALMM:LOCALMM+1,itrcind)=
     &                                zext(i,LOCALMM:LOCALMM+1,itrcind)
                enddo  
#   ifdef MPI                    
              endif  
#   endif
#  endif
#  ifdef AGRIF_OBC_WEST            
#   ifdef MPI         
              if (.not.WEST_INTER) then
#   endif                           
                do j=0,LOCALMM+1
                  Zeta_west(0:1,j,itrcind)=zext(0:1,j,itrcind)
                enddo
#   ifdef MPI            
              endif
#   endif
#  endif
#  ifdef AGRIF_OBC_EAST             
#   ifdef MPI         
              if (.not.EAST_INTER) then
#   endif           
                do j=0,LOCALMM+1
                  Zeta_east(LOCALLM:LOCALLM+1,j,itrcind)=
     &                             zext(LOCALLM:LOCALLM+1,j,itrcind)
                enddo 
#   ifdef MPI               
              endif  
#   endif                   
#  endif
            enddo
            Zetatimeindex = Agrif_Parent_Nb_Step() 
            Deallocate(zext)
          endif
C$OMP END MASTER
C$OMP BARRIER
          tinterp=float(Agrif_NBSTEPINT())/Agrif_rhot()
#  ifdef  SOLVE3D            
          tinterp=tinterp+1./(Agrif_rhot()*float(ndtfast))
#  else
          tinterp=tinterp+1./Agrif_rhot()
#  endif
#  ifdef SOLVE3D      
          tinterp=tinterp+float(iif-1)/(Agrif_rhot()*float(ndtfast))
#  endif      
          if (Agrif_NB_STEP().lt.int(Agrif_rhot())) tinterp=1.    
!
#  ifdef AGRIF_OBC_SOUTH             
          if (SOUTHERN_EDGE) then
            do i=Istr,Iend
#   if defined AGRIF_OBC_M2FLATHER && !defined Z_FRC_BRY
              SSH(i,1)=
     &             (tinterp*Zeta_south(i,1,Agrif_Parent_knew)
     &        +(1.-tinterp)*Zeta_south(i,1,Agrif_Parent_kstp))
              SSH(i,0)=
#   else
#    ifdef Z_FRC_BRY
              zetabry_south(i)=
#    else
              SSH(i,0)=
#    endif
#   endif
     &             (tinterp*Zeta_south(i,0,Agrif_Parent_knew)
     &        +(1.-tinterp)*Zeta_south(i,0,Agrif_Parent_kstp))    
            enddo
          endif
#  endif
#  ifdef AGRIF_OBC_NORTH            
          if (NORTHERN_EDGE) then      
            do i=Istr,Iend 
#   if defined AGRIF_OBC_M2FLATHER && !defined Z_FRC_BRY
              SSH(i,Jend)=
     &        (tinterp*Zeta_north(i,Jend,Agrif_Parent_knew)
     &   +(1.-tinterp)*Zeta_north(i,Jend,Agrif_Parent_kstp))
              SSH(i,Jend+1)=
#   else
#    ifdef Z_FRC_BRY
              zetabry_north(i)=
#    else
              SSH(i,Jend+1)=
#    endif
#   endif
     &        (tinterp*Zeta_north(i,Jend+1,Agrif_Parent_knew)
     &   +(1.-tinterp)*Zeta_north(i,Jend+1,Agrif_Parent_kstp))
            enddo
          endif      
#  endif
#  ifdef AGRIF_OBC_WEST            
          if (WESTERN_EDGE) then      
            do j=Jstr,Jend
#   if defined AGRIF_OBC_M2FLATHER && !defined Z_FRC_BRY
              SSH(1,j)=
     &             (tinterp*Zeta_west(1,j,Agrif_Parent_knew)
     &        +(1.-tinterp)*Zeta_west(1,j,Agrif_Parent_kstp))
              SSH(0,j)=
#   else
#    ifdef Z_FRC_BRY
              zetabry_west(j)=
#    else
              SSH(0,j)=
#    endif
#   endif
     &             (tinterp*Zeta_west(0,j,Agrif_Parent_knew)
     &        +(1.-tinterp)*Zeta_west(0,j,Agrif_Parent_kstp))
            enddo
          endif      
#  endif
#  ifdef AGRIF_OBC_EAST            
          if (EASTERN_EDGE) then      
            do j=Jstr,Jend
#   if defined AGRIF_OBC_M2FLATHER && !defined Z_FRC_BRY
              SSH(Iend,j)=
     &        (tinterp*Zeta_east(Iend,j,Agrif_Parent_knew)
     &   +(1.-tinterp)*Zeta_east(Iend,j,Agrif_Parent_kstp))
              SSH(Iend+1,j)=  
#   else
#    ifdef Z_FRC_BRY
              zetabry_east(j)=  
#    else
              SSH(Iend+1,j)= 
#    endif
#   endif
     &        (tinterp*Zeta_east(Iend+1,j,Agrif_Parent_knew)
     &   +(1.-tinterp)*Zeta_east(Iend+1,j,Agrif_Parent_kstp))   
            enddo
          endif
#  endif
#  ifdef AGRIF_OBC_M2FLATHER
      endif
#  endif 
# endif /* AGRIF_STORE_BAROT_CHILD */
      return
      end
!
!====================================================================
!                   subroutine Agrif_storebaro_tile
!====================================================================
!
# ifdef AGRIF_STORE_BAROT_CHILD
      subroutine Agrif_storebaro_tile(Istr,Iend,Jstr,Jend,
     &                                istore,kstore)
!
!  store the barotropic velocities for the child boundary conditions
!
      use Agrif_Util
      implicit none
#  include "param.h"
#  include "grid.h"
#  include "ocean2d.h"
#  include "zoom.h"
      real cff1,cff2
      integer Istr,Iend,Jstr,Jend,i,j,istore,kstore
!
#  include "compute_auxiliary_bounds.h"
!
      if (Agrif_Fixed().le.Agrif_Nb_Fine_Grids()-1) then
        cff1=0.5*(1.+1./Agrif_rhox()) 
        cff2=0.5*(1.+1./Agrif_rhoy())
!
! Western child boundary
!
#  ifdef AGRIF_OBC_WEST            
        if ((Istr.le.ic1).and.(Iend.ge.ic1)) then
          do j=JstrR,JendR     
            ubar_westbound(j,istore)=ubar(ic1,j,kstore)
#   ifdef AGRIF_FLUX_BC
     &                   *0.5*(h(ic1,j)+zeta(ic1,j,kstore)
     &                        +h(ic1-1,j)+zeta(ic1-1,j,kstore))
     &                       *on_u(ic1,j)
#   endif
            vbar_westbound(j,istore)=
#   ifdef MASKING
     &              max(1.-vmask(ic1,j),cff1)*vbar(ic1-1,j,kstore)
#   else
     &              cff1*vbar(ic1-1,j,kstore)
#   endif
#   ifdef AGRIF_FLUX_BC
     &                   *0.5*(h(ic1-1,j)+zeta(ic1-1,j,kstore)
     &                        +h(ic1-1,j-1)+zeta(ic1-1,j-1,kstore))
     &                       *om_v(ic1-1,j)
#   endif
#   ifdef MASKING
     &             +max(1.-vmask(ic1-1,j),1.-cff1)*vbar(ic1,j,kstore)
#   else
     &             +(1.-cff1)*vbar(ic1,j,kstore)
#   endif
#   ifdef AGRIF_FLUX_BC
     &                   *0.5*(h(ic1,j)+zeta(ic1,j,kstore)
     &                        +h(ic1,j-1)+zeta(ic1,j-1,kstore))
     &                       *om_v(ic1,j)
#   endif
            zeta_westbound(1,j,istore)=
#   ifdef MASKING
     &              max(1.-rmask(ic1,j),cff1)*zeta(ic1-1,j,kstore)
     &             +max(1.-rmask(ic1-1,j),1.-cff1)*zeta(ic1,j,kstore)
#   else
     &              cff1*zeta(ic1-1,j,kstore)
     &             +(1.-cff1)*zeta(ic1,j,kstore)
#   endif
#   if defined AGRIF_OBC_M2FLATHER && !defined Z_FRC_BRY
#    ifdef MASKING
            zeta_westbound(2,j,istore)=
     &              max(1.-rmask(ic1,j),1.-cff1)*zeta(ic1-1,j,kstore)
     &             +max(1.-rmask(ic1-1,j),cff1)*zeta(ic1,j,kstore)
#    else
            zeta_westbound(2,j,istore)=
     &              (1.-cff1)*zeta(ic1-1,j,kstore)
     &             +cff1*zeta(ic1,j,kstore)
#    endif
#   endif
          enddo
        endif
#  endif
!
! Eastern child boundary
!
#  ifdef AGRIF_OBC_EAST            
        if ((Istr.le.ic2).and.(Iend.ge.ic2)) then
          do j=JstrR,JendR     
           ubar_eastbound(j,istore)=ubar(ic2,j,kstore)
#   ifdef AGRIF_FLUX_BC
     &                   *0.5*(h(ic2,j)+zeta(ic2,j,kstore)
     &                        +h(ic2-1,j)+zeta(ic2-1,j,kstore))
     &                       *on_u(ic2,j)
#   endif
           vbar_eastbound(j,istore)=
#   ifdef MASKING
     &              max(1.-vmask(ic2,j),1.-cff1)*vbar(ic2-1,j,kstore)
#   else
     &              (1.-cff1)*vbar(ic2-1,j,kstore)
#   endif
#   ifdef AGRIF_FLUX_BC
     &                   *0.5*(h(ic2-1,j)+zeta(ic2-1,j,kstore)
     &                        +h(ic2-1,j-1)+zeta(ic2-1,j-1,kstore))
     &                       *om_v(ic2-1,j)
#   endif
#   ifdef MASKING
     &             +max(1.-vmask(ic2-1,j),cff1)*vbar(ic2,j,kstore)
#   else
     &             +cff1*vbar(ic2,j,kstore)
#   endif
#   ifdef AGRIF_FLUX_BC
     &                   *0.5*(h(ic2,j)+zeta(ic2,j,kstore)
     &                        +h(ic2,j-1)+zeta(ic2,j-1,kstore))
     &                       *om_v(ic2,j)
#   endif
            zeta_eastbound(1,j,istore)=
#   ifdef MASKING
     &              max(1.-rmask(ic2,j),1.-cff1)*zeta(ic2-1,j,kstore)
     &             +max(1.-rmask(ic2-1,j),cff1)*zeta(ic2,j,kstore)
#   else
     &              (1.-cff1)*zeta(ic2-1,j,kstore)
     &             +cff1*zeta(ic2,j,kstore)
#   endif
#   if defined AGRIF_OBC_M2FLATHER && !defined Z_FRC_BRY
            zeta_eastbound(2,j,istore)=
#    ifdef MASKING
     &              max(1.-rmask(ic2,j),cff1)*zeta(ic2-1,j,kstore)
     &             +max(1.-rmask(ic2-1,j),1.-cff1)*zeta(ic2,j,kstore)
#    else
     &              cff1*zeta(ic2-1,j,kstore)
     &             +(1.-cff1)*zeta(ic2,j,kstore)
#    endif
#   endif
          enddo
        endif
#  endif
!
! Southern child boundary
!
#  ifdef AGRIF_OBC_SOUTH            
        if ((Jstr.le.jc1).and.(Jend.ge.jc1)) then
          do i=IstrR,IendR
            ubar_southbound(i,istore)=
#   ifdef MASKING
     &              max(1.-umask(i,jc1-1),1.-cff2)*ubar(i,jc1,kstore)
#   else
     &              (1.-cff2)*ubar(i,jc1,kstore)
#   endif
#   ifdef AGRIF_FLUX_BC
     &                   *0.5*(h(i,jc1)+zeta(i,jc1,kstore)
     &                        +h(i-1,jc1)+zeta(i-1,jc1,kstore))
     &                       *on_u(i,jc1)
#   endif
#   ifdef MASKING
     &             +max(1.-umask(i,jc1),cff2)*ubar(i,jc1-1,kstore)
#   else
     &             +cff2*ubar(i,jc1-1,kstore)
#   endif
#   ifdef AGRIF_FLUX_BC
     &                   *0.5*(h(i,jc1-1)+zeta(i,jc1-1,kstore)
     &                        +h(i-1,jc1-1)+zeta(i-1,jc1-1,kstore))
     &                       *on_u(i,jc1-1)
#   endif
            vbar_southbound(i,istore)=vbar(i,jc1,kstore)
#   ifdef AGRIF_FLUX_BC
     &                   *0.5*(h(i,jc1)+zeta(i,jc1,kstore)
     &                        +h(i,jc1-1)+zeta(i,jc1-1,kstore))
     &                       *om_v(i,jc1)
#   endif
            zeta_southbound(1,i,istore)=
#   ifdef MASKING
     &              max(1.-rmask(i,jc1-1),1.-cff2)*zeta(i,jc1,kstore)
     &             +max(1.-rmask(i,jc1),cff2)*zeta(i,jc1-1,kstore)
#   else
     &              (1.-cff2)*zeta(i,jc1,kstore)
     &             +cff2*zeta(i,jc1-1,kstore)
#   endif
#   if defined AGRIF_OBC_M2FLATHER && !defined Z_FRC_BRY
            zeta_southbound(2,i,istore)=
#    ifdef MASKING
     &              max(1.-rmask(i,jc1-1),cff2)*zeta(i,jc1,kstore)
     &             +max(1.-rmask(i,jc1),1.-cff2)*zeta(i,jc1-1,kstore)
#    else
     &              cff2*zeta(i,jc1,kstore)
     &             +(1.-cff2)*zeta(i,jc1-1,kstore)
#    endif
#   endif
          enddo
        endif
#  endif
!
! Northern child boundary
!
#  ifdef AGRIF_OBC_NORTH            
        if ((Jstr.le.jc2).and.(Jend.ge.jc2)) then
          do i=IstrR,IendR
            ubar_northbound(i,istore)=
#   ifdef MASKING
     &              max(1.-umask(i,jc2),1.-cff2)*ubar(i,jc2-1,kstore)
#   else
     &              (1.-cff2)*ubar(i,jc2-1,kstore)
#   endif
#   ifdef AGRIF_FLUX_BC
     &                   *0.5*(h(i,jc2-1)+zeta(i,jc2-1,kstore)
     &                        +h(i-1,jc2-1)+zeta(i-1,jc2-1,kstore))
     &                       *on_u(i,jc2-1)
#   endif
#   ifdef MASKING
     &             +max(1.-umask(i,jc2-1),cff2)*ubar(i,jc2,kstore)
#   else
     &             +cff2*ubar(i,jc2,kstore)
#   endif
#   ifdef AGRIF_FLUX_BC
     &                   *0.5*(h(i,jc2)+zeta(i,jc2,kstore)
     &                        +h(i-1,jc2)+zeta(i-1,jc2,kstore))
     &                       *on_u(i,jc2)
#   endif
            vbar_northbound(i,istore)=vbar(i,jc2,kstore)
#   ifdef AGRIF_FLUX_BC
     &                   *0.5*(h(i,jc2)+zeta(i,jc2,kstore)
     &                        +h(i,jc2-1)+zeta(i,jc2-1,kstore))
     &                       *om_v(i,jc2)
#   endif
            zeta_northbound(1,i,istore)=
#   ifdef MASKING
     &              max(1.-rmask(i,jc2-1),cff2)*zeta(i,jc2,kstore)
     &             +max(1.-rmask(i,jc2),1.-cff2)*zeta(i,jc2-1,kstore)
#   else
     &              cff2*zeta(i,jc2,kstore)
     &             +(1.-cff2)*zeta(i,jc2-1,kstore)
#   endif
#   if defined AGRIF_OBC_M2FLATHER && !defined Z_FRC_BRY
            zeta_northbound(2,i,istore)=
#    ifdef MASKING
     &              max(1.-rmask(i,jc2-1),1.-cff2)*zeta(i,jc2,kstore)
     &             +max(1.-rmask(i,jc2),cff2)*zeta(i,jc2-1,kstore)
#    else
     &              (1.-cff2)*zeta(i,jc2,kstore)
     &             +cff2*zeta(i,jc2-1,kstore)
#    endif
#   endif
          enddo
        endif
#  endif
      endif
      return
      end
# endif /* AGRIF_STORE_BAROT_CHILD */
#else  /* AGRIF */
!
!====================================================================
!                   nothing...
!====================================================================
!
      subroutine zoom_empty
      return
      end
#endif  /* AGRIF */
